@@ -19,8 +19,8 @@
 #include <stdlib.h>                     // calloc, free
 #include <string.h>                     // strdup
 
-#include "libknot/errcode.h"
-#include "libknot/common.h"
+#include "libknot/libknot.h"
+#include "libknot/internal/macros.h"
 
 #include "dnstap/dnstap.h"
 #include "dnstap/reader.h"
@@ -31,7 +31,7 @@ dt_reader_t* dt_reader_create(const char *file_path)
 	struct fstrm_reader_options *ropt = NULL;
 	dt_reader_t *reader = NULL;
 	fstrm_res res;
-	
+
 	reader = calloc(1, sizeof(dt_reader_t));
 	if (reader == NULL) {
 		goto fail;
