@@ -18,8 +18,9 @@
 
 #include "knot/ctl/estimator.h"
 #include "libknot/dname.h"
-#include "common-knot/lists.h"
-#include "common/mem.h"
+#include "libknot/internal/lists.h"
+#include "libknot/internal/mem.h"
+#include "libknot/internal/macros.h"
 #include "knot/zone/node.h"
 #include "libknot/descriptor.h"
 
@@ -179,4 +180,3 @@ int estimator_free_trie_node(value_t *val, void *data)
 
 	return KNOT_EOK;
 }
-
