@@ -16,25 +16,23 @@
 
 #include <assert.h>
 #include <sys/stat.h>
-#include "knot/ctl/remote.h"
-#include "common/log.h"
-#include "common/mem.h"
-#include "common-knot/fdset.h"
-#include "knot/knot.h"
+#include <urcu.h>
+
+#include "dnssec/random.h"
+#include "knot/common/debug.h"
+#include "knot/common/fdset.h"
+#include "knot/common/log.h"
 #include "knot/conf/conf.h"
-#include "knot/server/net.h"
-#include "knot/server/tcp-handler.h"
-#include "libknot/packet/wire.h"
-#include "libknot/descriptor.h"
-#include "common-knot/strlcpy.h"
-#include "libknot/tsig-op.h"
-#include "libknot/rrtype/rdname.h"
-#include "libknot/rrtype/soa.h"
-#include "libknot/dnssec/random.h"
-#include "libknot/packet/wire.h"
-#include "knot/zone/timers.h"
+#include "knot/ctl/remote.h"
 #include "knot/dnssec/zone-sign.h"
 #include "knot/dnssec/zone-nsec.h"
+#include "knot/server/tcp-handler.h"
+#include "knot/zone/timers.h"
+#include "libknot/libknot.h"
+#include "libknot/internal/macros.h"
+#include "libknot/internal/mem.h"
+#include "libknot/internal/net.h"
+#include "libknot/internal/strlcpy.h"
 
 #define KNOT_CTL_REALM "knot."
 #define KNOT_CTL_REALM_EXT ("." KNOT_CTL_REALM)
@@ -43,7 +41,7 @@
 #define KNOT_CTL_SOCKET_UMASK 0007
 
 /*! \brief Remote command structure. */
-typedef struct remote_cmdargs_t {
+typedef struct remote_cmdargs {
 	const knot_rrset_t *arg;
 	unsigned argc;
 	knot_rcode_t rc;
@@ -102,10 +100,10 @@ static void cmdargs_deinit(remote_cmdargs_t *args)
 typedef int (*remote_cmdf_t)(server_t*, remote_cmdargs_t*);
 
 /*! \brief Callback prototype for per-zone operations. */
-typedef int (remote_zonef_t)(zone_t *);
+typedef int (remote_zonef_t)(zone_t *, remote_cmdargs_t *a);
 
 /*! \brief Remote command table item. */
-typedef struct remote_cmd_t {
+typedef struct remote_cmd {
 	const char *name;
 	remote_cmdf_t f;
 } remote_cmd_t;
@@ -121,7 +119,7 @@ static int remote_c_flush(server_t *s, remote_cmdargs_t* a);
 static int remote_c_signzone(server_t *s, remote_cmdargs_t* a);
 
 /*! \brief Table of remote commands. */
-struct remote_cmd_t remote_cmd_tbl[] = {
+struct remote_cmd remote_cmd_tbl[] = {
 	{ "stop",      &remote_c_stop },
 	{ "reload",    &remote_c_reload },
 	{ "refresh",   &remote_c_refresh },
@@ -135,7 +133,7 @@ struct remote_cmd_t remote_cmd_tbl[] = {
 
 /* Private APIs. */
 
-/*! \brief Apply callback to all zones specified by RDATA of CNAME RRs. */
+/*! \brief Apply callback to all zones specified by RDATA of NS RRs. */
 static int remote_rdata_apply(server_t *s, remote_cmdargs_t* a, remote_zonef_t *cb)
 {
 	if (!s || !a || !cb) {
@@ -155,12 +153,15 @@ static int remote_rdata_apply(server_t *s, remote_cmdargs_t* a, remote_zonef_t *
 		uint16_t rr_count = rr->rrs.rr_count;
 		for (uint16_t i = 0; i < rr_count; i++) {
 			const knot_dname_t *dn = knot_ns_name(&rr->rrs, i);
-			rcu_read_lock();
 			zone = knot_zonedb_find(s->zone_db, dn);
-			if (cb(zone) != KNOT_EOK) {
+			if (zone == NULL) {
+				char zname[KNOT_DNAME_MAXLEN];
+				knot_dname_to_str(zname, dn, KNOT_DNAME_MAXLEN);
+				log_warning("remote control, zone %s not found.",
+				            zname);
+			} else if (cb(zone, a) != KNOT_EOK) {
 				a->rc = KNOT_RCODE_SERVFAIL;
 			}
-			rcu_read_unlock();
 		}
 	}
 
@@ -168,9 +169,15 @@ static int remote_rdata_apply(server_t *s, remote_cmdargs_t* a, remote_zonef_t *
 }
 
 /*! \brief Zone refresh callback. */
-static int remote_zone_refresh(zone_t *zone)
+static int remote_zone_refresh(zone_t *zone, remote_cmdargs_t *a)
 {
-	if (!zone_is_slave(zone)) {
+	UNUSED(a);
+
+	rcu_read_lock();
+	bool is_slave = zone_is_slave(zone);
+	rcu_read_unlock();
+
+	if (!is_slave) {
 		return KNOT_EINVAL;
 	}
 
@@ -178,10 +185,25 @@ static int remote_zone_refresh(zone_t *zone)
 	return KNOT_EOK;
 }
 
-/*! \brief Zone refresh callback. */
-static int remote_zone_retransfer(zone_t *zone)
+/*! \brief Zone reload callback. */
+static int remote_zone_reload(zone_t *zone, remote_cmdargs_t *a)
 {
-	if (!zone_is_slave(zone)) {
+	UNUSED(a);
+
+	zone_events_schedule(zone, ZONE_EVENT_RELOAD, ZONE_EVENT_NOW);
+	return KNOT_EOK;
+}
+
+/*! \brief Zone refresh callback. */
+static int remote_zone_retransfer(zone_t *zone, remote_cmdargs_t *a)
+{
+	UNUSED(a);
+
+	rcu_read_lock();
+	bool is_slave = zone_is_slave(zone);
+	rcu_read_unlock();
+
+	if (!is_slave) {
 		return KNOT_EINVAL;
 	}
 
@@ -191,24 +213,29 @@ static int remote_zone_retransfer(zone_t *zone)
 }
 
 /*! \brief Zone flush callback. */
-static int remote_zone_flush(zone_t *zone)
+static int remote_zone_flush(zone_t *zone, remote_cmdargs_t *a)
 {
-	if (zone == NULL) {
-		return KNOT_EINVAL;
-	}
+	UNUSED(a);
 
+	zone->flags |= ZONE_FORCE_FLUSH;
 	zone_events_schedule(zone, ZONE_EVENT_FLUSH, ZONE_EVENT_NOW);
 	return KNOT_EOK;
 }
 
 /*! \brief Sign zone callback. */
-static int remote_zone_sign(zone_t *zone)
+static int remote_zone_sign(zone_t *zone, remote_cmdargs_t *a)
 {
-	if (zone == NULL || !zone->conf->dnssec_enable) {
+	UNUSED(a);
+
+	rcu_read_lock();
+	conf_val_t val = conf_zone_get(conf(), C_DNSSEC_SIGNING, zone->name);
+	bool dnssec_enable = conf_bool(&val);
+	rcu_read_unlock();
+
+	if (!dnssec_enable) {
 		return KNOT_EINVAL;
 	}
 
-
 	zone->flags |= ZONE_FORCE_RESIGN;
 	zone_events_schedule(zone, ZONE_EVENT_DNSSEC, ZONE_EVENT_NOW);
 	return KNOT_EOK;
@@ -231,12 +258,26 @@ static int remote_c_stop(server_t *s, remote_cmdargs_t* a)
  * \brief Remote command 'reload' handler.
  *
  * QNAME: reload
- * DATA: NULL
+ * DATA: NONE for all zones
+ *       NS RRs with zones in RDATA
  */
 static int remote_c_reload(server_t *s, remote_cmdargs_t* a)
 {
-	UNUSED(a);
-	return server_reload(s, conf()->filename);
+	int ret = KNOT_EOK;
+
+	if (a->argc == 0) {
+		/* Reload all. */
+		dbg_server_verb("remote: refreshing all zones\n");
+		ret = server_reload(s, conf()->filename);
+	} else {
+		rcu_read_lock();
+		/* Reload specific zones. */
+		dbg_server_verb("remote: refreshing particular zones");
+		ret = remote_rdata_apply(s, a, &remote_zone_reload);
+		rcu_read_unlock();
+	}
+
+	return (ret != KNOT_EOK) ? ret : KNOT_CTL_ACCEPTED;
 }
 
 /*!
@@ -270,86 +311,97 @@ static char *dnssec_info(const zone_t *zone, char *buf, size_t buf_size)
 	return buf;
 }
 
+static int remote_zonestatus(zone_t *zone, remote_cmdargs_t *a)
+{
+	if (zone == NULL || a == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	int ret = KNOT_EOK;
+
+	/* Fetch latest serial. */
+	const knot_rdataset_t *soa_rrs = NULL;
+	uint32_t serial = 0;
+	if (zone->contents) {
+		soa_rrs = node_rdataset(zone->contents->apex,
+		                        KNOT_RRTYPE_SOA);
+		assert(soa_rrs != NULL);
+		serial = knot_soa_serial(soa_rrs);
+	}
+
+	/* Fetch next zone event. */
+	char when[128] = { '\0' };
+	zone_event_type_t next_type = ZONE_EVENT_INVALID;
+	const char *next_name = "";
+	time_t next_time = zone_events_get_next(zone, &next_type);
+	if (next_type != ZONE_EVENT_INVALID) {
+		next_name = zone_events_get_name(next_type);
+		next_time = next_time - time(NULL);
+		if (next_time < 0) {
+			memcpy(when, "pending", strlen("pending"));
+		} else if (snprintf(when, sizeof(when),
+		                    "in %lldh%lldm%llds",
+		                    (long long)(next_time / 3600),
+		                    (long long)(next_time % 3600) / 60,
+		                    (long long)(next_time % 60)) < 0) {
+			return KNOT_ESPACE;
+		}
+	} else {
+		memcpy(when, "idle", strlen("idle"));
+	}
+
+	/* Prepare zone info. */
+	char buf[512] = { '\0' };
+	char dnssec_buf[128] = { '\0' };
+	char *zone_name = knot_dname_to_str_alloc(zone->name);
+
+	conf_val_t val = conf_zone_get(conf(), C_DNSSEC_SIGNING, zone->name);
+	bool dnssec_enable = conf_bool(&val);
+	bool is_slave = zone_is_slave(zone);
+
+	int n = snprintf(buf, sizeof(buf),
+	                 "%s\ttype=%s | serial=%u | %s %s | %s %s\n",
+	                 zone_name,
+	                 is_slave ? "slave" : "master",
+	                 serial,
+	                 next_name,
+	                 when,
+	                 dnssec_enable ? "automatic DNSSEC, resigning at:" : "DNSSEC signing disabled",
+	                 dnssec_enable ? dnssec_info(zone, dnssec_buf, sizeof(dnssec_buf)) : "");
+	free(zone_name);
+	if (n < 0 || n >= sizeof(buf)) {
+		return KNOT_ESPACE;
+	}
+
+	ret = cmdargs_assure_avail(a, n);
+	if (ret == KNOT_EOK) {
+		memcpy(a->response + a->response_size, buf, n);
+		a->response_size += n;
+	}
+
+	return ret;
+}
+
 /*!
  * \brief Remote command 'zonestatus' handler.
  *
  * QNAME: zonestatus
- * DATA: NONE
+ * DATA: NONE for all zones
+ *       NS RRs with zones in RDATA
  */
 static int remote_c_zonestatus(server_t *s, remote_cmdargs_t* a)
 {
 	dbg_server("remote: %s\n", __func__);
 
-	int ret = KNOT_EOK;
 	rcu_read_lock();
-
-	knot_zonedb_iter_t it;
-	knot_zonedb_iter_begin(s->zone_db, &it);
-	while(!knot_zonedb_iter_finished(&it)) {
-		const zone_t *zone = knot_zonedb_iter_val(&it);
-
-		/* Fetch latest serial. */
-		const knot_rdataset_t *soa_rrs = NULL;
-		uint32_t serial = 0;
-		if (zone->contents) {
-			soa_rrs = node_rdataset(zone->contents->apex,
-			                        KNOT_RRTYPE_SOA);
-			assert(soa_rrs != NULL);
-			serial = knot_soa_serial(soa_rrs);
-		}
-
-		/* Fetch next zone event. */
-		char when[128] = { '\0' };
-		zone_event_type_t next_type = ZONE_EVENT_INVALID;
-		const char *next_name = "";
-		time_t next_time = zone_events_get_next(zone, &next_type);
-		if (next_type != ZONE_EVENT_INVALID) {
-			next_name = zone_events_get_name(next_type);
-			next_time = next_time - time(NULL);
-			if (next_time < 0) {
-				memcpy(when, "pending", strlen("pending"));
-			} else if (snprintf(when, sizeof(when),
-			                    "in %lldh%lldm%llds",
-			                    (long long)(next_time / 3600),
-			                    (long long)(next_time % 3600) / 60,
-			                    (long long)(next_time % 60)) < 0) {
-				ret = KNOT_ESPACE;
-				break;
-			}
-		} else {
-			memcpy(when, "idle", strlen("idle"));
-		}
-
-		/* Prepare zone info. */
-		char buf[512] = { '\0' };
-		char dnssec_buf[128] = { '\0' };
-		int n = snprintf(buf, sizeof(buf),
-		                 "%s\ttype=%s | serial=%u | %s %s | %s %s\n",
-		                 zone->conf->name,
-		                 zone_is_slave(zone) ? "slave" : "master",
-		                 serial,
-		                 next_name,
-		                 when,
-		                 zone->conf->dnssec_enable ? "automatic DNSSEC, resigning at:" : "DNSSEC signing disabled",
-		                 zone->conf->dnssec_enable ? dnssec_info(zone, dnssec_buf, sizeof(dnssec_buf)) : "");
-		if (n < 0 || n >= sizeof(buf)) {
-			ret = KNOT_ESPACE;
-			break;
-		}
-
-		ret = cmdargs_assure_avail(a, n);
-		if (ret != KNOT_EOK) {
-			break;
-		}
-
-		memcpy(a->response + a->response_size, buf, n);
-		a->response_size += n;
-
-		knot_zonedb_iter_next(&it);
+	if (a->argc == 0) {
+		knot_zonedb_foreach(s->zone_db, remote_zonestatus, a);
+	} else {
+		remote_rdata_apply(s, a, remote_zonestatus);
 	}
 	rcu_read_unlock();
 
-	return ret;
+	return KNOT_EOK;
 }
 
 /*!
@@ -357,19 +409,21 @@ static int remote_c_zonestatus(server_t *s, remote_cmdargs_t* a)
  *
  * QNAME: refresh
  * DATA: NONE for all zones
- *       CNAME RRs with zones in RDATA
+ *       NS RRs with zones in RDATA
  */
 static int remote_c_refresh(server_t *s, remote_cmdargs_t* a)
 {
 	dbg_server("remote: %s\n", __func__);
+	rcu_read_lock();
 	if (a->argc == 0) {
 		/* Refresh all. */
 		dbg_server_verb("remote: refreshing all zones\n");
-		knot_zonedb_foreach(s->zone_db, remote_zone_refresh);
+		knot_zonedb_foreach(s->zone_db, remote_zone_refresh, NULL);
 	} else {
 		/* Refresh specific zones. */
-		remote_rdata_apply(s, a, &remote_zone_refresh);
+		remote_rdata_apply(s, a, remote_zone_refresh);
 	}
+	rcu_read_unlock();
 
 	return KNOT_CTL_ACCEPTED;
 }
@@ -378,17 +432,19 @@ static int remote_c_refresh(server_t *s, remote_cmdargs_t* a)
  * \brief Remote command 'retransfer' handler.
  *
  * QNAME: retransfer
- * DATA: CNAME RRs with zones in RDATA
+ * DATA: NS RRs with zones in RDATA
  */
 static int remote_c_retransfer(server_t *s, remote_cmdargs_t* a)
 {
 	dbg_server("remote: %s\n", __func__);
 	if (a->argc == 0) {
-		/* Refresh all. */
-		return KNOT_ENOTSUP;
+		/* Retransfer all. */
+		return KNOT_CTL_ARG_REQ;
 	} else {
-		/* Refresh specific zones. */
-		remote_rdata_apply(s, a, &remote_zone_retransfer);
+		rcu_read_lock();
+		/* Retransfer specific zones. */
+		remote_rdata_apply(s, a, remote_zone_retransfer);
+		rcu_read_unlock();
 	}
 
 	return KNOT_CTL_ACCEPTED;
@@ -400,26 +456,22 @@ static int remote_c_retransfer(server_t *s, remote_cmdargs_t* a)
  *
  * QNAME: flush
  * DATA: NONE for all zones
- *       CNAME RRs with zones in RDATA
+ *       NS RRs with zones in RDATA
  */
 static int remote_c_flush(server_t *s, remote_cmdargs_t* a)
 {
 	dbg_server("remote: %s\n", __func__);
+
+	rcu_read_lock();
 	if (a->argc == 0) {
 		/* Flush all. */
 		dbg_server_verb("remote: flushing all zones\n");
-		rcu_read_lock();
-		knot_zonedb_iter_t it;
-		knot_zonedb_iter_begin(s->zone_db, &it);
-		while(!knot_zonedb_iter_finished(&it)) {
-			remote_zone_flush(knot_zonedb_iter_val(&it));
-			knot_zonedb_iter_next(&it);
-		}
-		rcu_read_unlock();
+		knot_zonedb_foreach(s->zone_db, remote_zone_flush, NULL);
 	} else {
 		/* Flush specific zones. */
-		remote_rdata_apply(s, a, &remote_zone_flush);
+		remote_rdata_apply(s, a, remote_zone_flush);
 	}
+	rcu_read_unlock();
 
 	return KNOT_CTL_ACCEPTED;
 }
@@ -428,16 +480,18 @@ static int remote_c_flush(server_t *s, remote_cmdargs_t* a)
  * \brief Remote command 'signzone' handler.
  *
  */
-static int remote_c_signzone(server_t *server, remote_cmdargs_t* arguments)
+static int remote_c_signzone(server_t *s, remote_cmdargs_t* a)
 {
 	dbg_server("remote: %s\n", __func__);
 
-	if (arguments->argc == 0) {
+	if (a->argc == 0) {
 		/* Resign all. */
-		return KNOT_ENOTSUP;
+		return KNOT_CTL_ARG_REQ;
 	} else {
+		rcu_read_lock();
 		/* Resign specific zones. */
-		remote_rdata_apply(server, arguments, remote_zone_sign);
+		remote_rdata_apply(s, a, remote_zone_sign);
+		rcu_read_unlock();
 	}
 
 	return KNOT_CTL_ACCEPTED;
@@ -452,60 +506,73 @@ static int remote_c_signzone(server_t *server, remote_cmdargs_t* arguments)
  */
 static int remote_senderr(int c, uint8_t *qbuf, size_t buflen)
 {
+	rcu_read_lock();
+	conf_val_t val = conf_get(conf(), C_SRV, C_TCP_REPLY_TIMEOUT);
+	struct timeval timeout = { conf_int(&val), 0 };
+	rcu_read_unlock();
+
 	knot_wire_set_qr(qbuf);
 	knot_wire_set_rcode(qbuf, KNOT_RCODE_REFUSED);
-	struct timeval timeout = { conf()->max_conn_reply, 0 };
 	return tcp_send_msg(c, qbuf, buflen, &timeout);
 }
 
 /* Public APIs. */
 
-int remote_bind(conf_iface_t *desc)
+int remote_bind(struct sockaddr_storage *addr)
 {
-	if (desc == NULL) {
+	if (addr == NULL) {
 		return KNOT_EINVAL;
 	}
 
-	char addr_str[SOCKADDR_STRLEN] = {0};
-	sockaddr_tostr(&desc->addr, addr_str, sizeof(addr_str));
-	log_info("binding remote control interface to '%s'", addr_str);
+	char addr_str[SOCKADDR_STRLEN] = { 0 };
+	sockaddr_tostr(addr_str, sizeof(addr_str), addr);
+	log_info("remote control, binding to '%s'", addr_str);
 
 	/* Create new socket. */
 	mode_t old_umask = umask(KNOT_CTL_SOCKET_UMASK);
-	int sock = net_bound_socket(SOCK_STREAM, &desc->addr);
+	int sock = net_bound_socket(SOCK_STREAM, addr, 0);
+	if (sock == KNOT_EADDRNOTAVAIL) {
+		sock = net_bound_socket(SOCK_STREAM, addr, NET_BIND_NONLOCAL);
+		if (sock >= 0) {
+			log_warning("remote control, address '%s' is not available",
+			            addr_str);
+		}
+	}
 	umask(old_umask);
+
 	if (sock < 0) {
+		log_error("remote control, failed to bind to '%s' (%s)",
+		          addr_str, knot_strerror(sock));
 		return sock;
 	}
 
 	/* Start listening. */
-	int ret = listen(sock, TCP_BACKLOG_SIZE);
-	if (ret < 0) {
-		log_error("failed to bind to '%s'", addr_str);
+	if (listen(sock, TCP_BACKLOG_SIZE) != 0) {
+		log_error("remote control, failed to listen on '%s'", addr_str);
 		close(sock);
-		return ret;
+		return knot_map_errno();
 	}
 
 	return sock;
 }
 
-int remote_unbind(conf_iface_t *desc, int sock)
+int remote_unbind(struct sockaddr_storage *addr, int sock)
 {
-	if (desc == NULL || sock < 0) {
+	if (addr == NULL || sock < 0) {
 		return KNOT_EINVAL;
 	}
 
 	/* Remove control socket file.  */
-	if (desc->addr.ss_family == AF_UNIX) {
-		char addr_str[SOCKADDR_STRLEN] = {0};
-		sockaddr_tostr(&desc->addr, addr_str, sizeof(addr_str));
+	if (addr->ss_family == AF_UNIX) {
+		char addr_str[SOCKADDR_STRLEN] = { 0 };
+		sockaddr_tostr(addr_str, sizeof(addr_str), addr);
 		unlink(addr_str);
 	}
 
 	return close(sock);
 }
 
-int remote_poll(int sock)
+int remote_poll(int sock, const sigset_t *sigmask)
 {
 	/* Wait for events. */
 	fd_set rfds;
@@ -516,7 +583,7 @@ int remote_poll(int sock)
 		sock = -1; /* Make sure n == r + 1 == 0 */
 	}
 
-	return fdset_pselect(sock + 1, &rfds, NULL, NULL, NULL, NULL);
+	return pselect(sock + 1, &rfds, NULL, NULL, NULL, sigmask);
 }
 
 int remote_recv(int sock, struct sockaddr_storage *addr, uint8_t *buf,
@@ -585,7 +652,11 @@ static int remote_send_chunk(int c, knot_pkt_t *query, const char* d, uint16_t l
 		goto failed;
 	}
 
-	struct timeval timeout = { conf()->max_conn_reply, 0 };
+	rcu_read_lock();
+	conf_val_t val = conf_get(conf(), C_SRV, C_TCP_REPLY_TIMEOUT);
+	struct timeval timeout = { conf_int(&val), 0 };
+	rcu_read_unlock();
+
 	ret = tcp_send_msg(c, resp->wire, resp->size, &timeout);
 
 failed:
@@ -600,6 +671,7 @@ static void log_command(const char *cmd, const remote_cmdargs_t* args)
 {
 	char params[CMDARGS_BUFLEN_LOG] = { 0 };
 	size_t rest = CMDARGS_BUFLEN_LOG;
+	size_t pos = 0;
 
 	for (unsigned i = 0; i < args->argc; i++) {
 		const knot_rrset_t *rr = &args->arg[i];
@@ -612,11 +684,13 @@ static void log_command(const char *cmd, const remote_cmdargs_t* args)
 			const knot_dname_t *dn = knot_ns_name(&rr->rrs, j);
 			char *name = knot_dname_to_str_alloc(dn);
 
-			int ret = snprintf(params, rest, " %s", name);
+			int ret = snprintf(params + pos, rest, " %s", name);
 			free(name);
+
 			if (ret <= 0 || ret >= rest) {
 				break;
 			}
+			pos += ret;
 			rest -= ret;
 		}
 	}
@@ -661,11 +735,12 @@ int remote_answer(int sock, server_t *s, knot_pkt_t *pkt)
 	remote_cmdargs_t args = { 0 };
 	int ret = cmdargs_init(&args);
 	if (ret != KNOT_EOK) {
+		free(cmd);
 		return ret;
 	}
 
 	const knot_pktsection_t *authority = knot_pkt_section(pkt, KNOT_AUTHORITY);
-	args.arg = authority->rr;
+	args.arg = knot_pkt_rr(authority, 0);
 	args.argc = authority->count;
 	args.rc = KNOT_RCODE_NOERROR;
 
@@ -721,8 +796,8 @@ static int zones_verify_tsig_query(const knot_pkt_t *query,
 	/*
 	 * 1) Check if we support the requested algorithm.
 	 */
-	knot_tsig_algorithm_t alg = tsig_rdata_alg(query->tsig_rr);
-	if (knot_tsig_digest_length(alg) == 0) {
+	dnssec_tsig_algorithm_t alg = knot_tsig_rdata_alg(query->tsig_rr);
+	if (alg == DNSSEC_TSIG_UNKNOWN) {
 		log_info("TSIG, unsupported algorithm, query NOTAUTH");
 		/*! \todo [TSIG] It is unclear from RFC if I
 		 *               should treat is as a bad key
@@ -753,14 +828,14 @@ static int zones_verify_tsig_query(const knot_pkt_t *query,
 	/* Prepare variables for TSIG */
 	/*! \todo These need to be saved to the response somehow. */
 	//size_t tsig_size = tsig_wire_maxsize(key);
-	size_t digest_max_size = knot_tsig_digest_length(key->algorithm);
+	size_t digest_max_size = dnssec_tsig_algorithm_size(alg);
 	//size_t digest_size = 0;
 	//uint64_t tsig_prev_time_signed = 0;
 	//uint8_t *digest = (uint8_t *)malloc(digest_max_size);
 	//memset(digest, 0 , digest_max_size);
 
 	//const uint8_t* mac = tsig_rdata_mac(tsig_rr);
-	size_t mac_len = tsig_rdata_mac_length(query->tsig_rr);
+	size_t mac_len = knot_tsig_rdata_mac_length(query->tsig_rr);
 
 	int ret = KNOT_EOK;
 
@@ -792,7 +867,7 @@ static int zones_verify_tsig_query(const knot_pkt_t *query,
 		case KNOT_TSIG_EBADTIME:
 			*tsig_rcode = KNOT_TSIG_ERR_BADTIME;
 			// store the time signed from the query
-			*tsig_prev_time_signed = tsig_rdata_time_signed(query->tsig_rr);
+			*tsig_prev_time_signed = knot_tsig_rdata_time_signed(query->tsig_rr);
 			*rcode = KNOT_RCODE_NOTAUTH;
 			break;
 		case KNOT_EMALF:
@@ -806,7 +881,7 @@ static int zones_verify_tsig_query(const knot_pkt_t *query,
 	return ret;
 }
 
-int remote_process(server_t *s, conf_iface_t *ctl_if, int sock,
+int remote_process(server_t *s, struct sockaddr_storage *ctl_addr, int sock,
                    uint8_t* buf, size_t buflen)
 {
 	knot_pkt_t *pkt =  knot_pkt_new(buf, buflen, NULL);
@@ -830,40 +905,38 @@ int remote_process(server_t *s, conf_iface_t *ctl_if, int sock,
 
 	/* Parse packet and answer if OK. */
 	int ret = remote_parse(pkt);
-	if (ret == KNOT_EOK && ctl_if->addr.ss_family != AF_UNIX) {
+	if (ret == KNOT_EOK && ctl_addr->ss_family != AF_UNIX) {
+		char addr_str[SOCKADDR_STRLEN] = { 0 };
+		sockaddr_tostr(addr_str, sizeof(addr_str), &ss);
 
-		/* Check ACL list. */
-		char addr_str[SOCKADDR_STRLEN] = {0};
-		sockaddr_tostr(&ss, addr_str, sizeof(addr_str));
-		knot_tsig_key_t *tsig_key = NULL;
-		const knot_dname_t *tsig_name = NULL;
+		/* Prepare tsig parameters. */
+		knot_tsig_key_t tsig = { 0 };
 		if (pkt->tsig_rr) {
-			tsig_name = pkt->tsig_rr->owner;
+			tsig.name = pkt->tsig_rr->owner;
+			tsig.algorithm = knot_tsig_rdata_alg(pkt->tsig_rr);
 		}
-		conf_iface_t *match = acl_find(&conf()->ctl.allow, &ss, tsig_name);
-		uint16_t ts_rc = 0;
-		uint16_t ts_trc = 0;
-		uint64_t ts_tmsigned = 0;
-		if (match == NULL) {
+
+		/* Check ACL. */
+		rcu_read_lock();
+		conf_val_t acl = conf_get(conf(), C_CTL, C_ACL);
+		bool allowed = acl_allowed(&acl, ACL_ACTION_CONTROL, &ss, &tsig);
+		rcu_read_unlock();
+
+		if (!allowed) {
 			log_warning("remote control, denied '%s', "
 			            "no matching ACL", addr_str);
 			remote_senderr(client, pkt->wire, pkt->size);
 			ret = KNOT_EACCES;
 			goto finish;
-		} else {
-			tsig_key = match->key;
 		}
 
 		/* Check TSIG. */
-		if (tsig_key) {
-			if (pkt->tsig_rr == NULL) {
-				log_warning("remote control, denied '%s', "
-				            "key required", addr_str);
-				remote_senderr(client, pkt->wire, pkt->size);
-				ret = KNOT_EACCES;
-				goto finish;
-			}
-			ret = zones_verify_tsig_query(pkt, tsig_key, &ts_rc,
+		if (tsig.name != NULL) {
+			uint16_t ts_rc = 0;
+			uint16_t ts_trc = 0;
+			uint64_t ts_tmsigned = 0;
+
+			ret = zones_verify_tsig_query(pkt, &tsig, &ts_rc,
 			                              &ts_trc, &ts_tmsigned);
 			if (ret != KNOT_EOK) {
 				log_warning("remote control, denied '%s', "
@@ -897,8 +970,8 @@ knot_pkt_t* remote_query(const char *query, const knot_tsig_key_t *key)
 		return NULL;
 	}
 
-	knot_wire_set_id(pkt->wire, knot_random_uint16_t());
-	knot_pkt_reserve(pkt, tsig_wire_maxsize(key));
+	knot_wire_set_id(pkt->wire, dnssec_random_uint16_t());
+	knot_pkt_reserve(pkt, knot_tsig_wire_maxsize(key));
 
 	/* Question section. */
 	char *qname = strcdup(query, KNOT_CTL_REALM_EXT);
@@ -927,7 +1000,7 @@ int remote_query_sign(uint8_t *wire, size_t *size, size_t maxlen,
 		return KNOT_EINVAL;
 	}
 
-	size_t dlen = knot_tsig_digest_length(key->algorithm);
+	size_t dlen = dnssec_tsig_algorithm_size(key->algorithm);
 	uint8_t *digest = malloc(dlen);
 	if (!digest) {
 		return KNOT_ENOMEM;
