@@ -20,15 +20,13 @@
 #include <string.h>
 #include <limits.h>
 
-#include "common/base32hex.h"
-#include "common/debug.h"
+#include "dnssec/error.h"
+#include "dnssec/nsec.h"
+#include "libknot/internal/base32hex.h"
 #include "libknot/descriptor.h"
-#include "common-knot/hhash.h"
-#include "libknot/dnssec/bitmap.h"
-#include "libknot/util/utils.h"
 #include "libknot/packet/wire.h"
-#include "libknot/rrtype/soa.h"
 #include "libknot/rrtype/nsec3.h"
+#include "libknot/rrtype/soa.h"
 #include "knot/dnssec/nsec-chain.h"
 #include "knot/dnssec/nsec3-chain.h"
 #include "knot/dnssec/zone-nsec.h"
@@ -53,7 +51,6 @@ static int delete_nsec3_chain(const zone_contents_t *zone,
 		return KNOT_EOK;
 	}
 
-	dbg_dnssec_detail("deleting NSEC3 chain\n");
 	zone_tree_t *empty_tree = zone_tree_create();
 	if (!empty_tree) {
 		return KNOT_ENOMEM;
@@ -157,7 +154,7 @@ static int mark_removed_nsec3(changeset_t *out_ch,
 
 	changeset_iter_t itt;
 	changeset_iter_rem(&itt, out_ch, false);
-	
+
 	knot_rrset_t rr = changeset_iter_next(&itt);
 	while (!knot_rrset_empty(&rr)) {
 		int ret = mark_nsec3(&rr, zone->nsec3_nodes);
@@ -168,7 +165,7 @@ static int mark_removed_nsec3(changeset_t *out_ch,
 		rr = changeset_iter_next(&itt);
 	}
 	changeset_iter_clear(&itt);
-	
+
 	return KNOT_EOK;
 }
 
@@ -191,21 +188,34 @@ knot_dname_t *knot_create_nsec3_owner(const knot_dname_t *owner,
 		return NULL;
 	}
 
-	uint8_t *hash = NULL;
-	size_t hash_size = 0;
 	int owner_size = knot_dname_size(owner);
-
 	if (owner_size < 0) {
 		return NULL;
 	}
 
-	if (knot_nsec3_hash(params, owner, owner_size, &hash, &hash_size)
-	    != KNOT_EOK) {
+	dnssec_binary_t data = { 0 };
+	data.data = (uint8_t *)owner;
+	data.size = owner_size;
+
+	dnssec_binary_t hash = { 0 };
+	dnssec_nsec3_params_t xparams = {
+		.algorithm = params->algorithm,
+		.flags = params->flags,
+		.iterations = params->iterations,
+		.salt = {
+			.data = params->salt,
+			.size = params->salt_length
+		}
+	};
+
+	int r = dnssec_nsec3_hash(&data, &xparams, &hash);
+	if (r != DNSSEC_EOK) {
 		return NULL;
 	}
 
-	knot_dname_t *result = knot_nsec3_hash_to_dname(hash, hash_size, zone_apex);
-	free(hash);
+	knot_dname_t *result = knot_nsec3_hash_to_dname(hash.data, hash.size, zone_apex);
+
+	dnssec_binary_free(&hash);
 
 	return result;
 }
@@ -258,8 +268,8 @@ knot_dname_t *knot_nsec3_hash_to_dname(const uint8_t *hash, size_t hash_size,
  */
 int knot_zone_create_nsec_chain(const zone_contents_t *zone,
                                 changeset_t *changeset,
-                                const knot_zone_keys_t *zone_keys,
-                                const knot_dnssec_policy_t *policy)
+                                const zone_keyset_t *zone_keys,
+                                const kdnssec_ctx_t *dnssec_ctx)
 {
 	if (!zone || !changeset) {
 		return KNOT_EINVAL;
@@ -293,5 +303,5 @@ int knot_zone_create_nsec_chain(const zone_contents_t *zone,
 	}
 
 	// Sign newly created records right away
-	return knot_zone_sign_nsecs_in_changeset(zone_keys, policy, changeset);
+	return knot_zone_sign_nsecs_in_changeset(zone_keys, dnssec_ctx, changeset);
 }
