@@ -16,22 +16,32 @@
 
 #include <sys/stat.h>
 
+#include "knot/common/log.h"
 #include "knot/modules/dnstap.h"
-#include "knot/nameserver/query_module.h"
 #include "knot/nameserver/process_query.h"
 #include "dnstap/dnstap.pb-c.h"
 #include "dnstap/writer.h"
 #include "dnstap/message.h"
 #include "dnstap/dnstap.h"
-#include "libknot/descriptor.h"
+#include "libknot/libknot.h"
+
+/* Module configuration scheme. */
+#define MOD_SINK	"\x04""sink"
+
+const yp_item_t scheme_mod_dnstap[] = {
+	{ C_ID,      YP_TSTR, YP_VNONE },
+	{ MOD_SINK,  YP_TSTR, YP_VNONE },
+	{ C_COMMENT, YP_TSTR, YP_VNONE },
+	{ NULL }
+};
 
 /* Defines. */
-#define MODULE_ERR(msg...) log_error("module 'dnstap', " msg)
+#define MODULE_ERR(msg, ...) log_error("module 'dnstap', " msg, ##__VA_ARGS__)
 
 static int log_message(int state, const knot_pkt_t *pkt, struct query_data *qdata, void *ctx)
 {
 	if (pkt == NULL || qdata == NULL || ctx == NULL) {
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	int ret = KNOT_ERROR;
@@ -63,7 +73,7 @@ static int log_message(int state, const knot_pkt_t *pkt, struct query_data *qdat
 	                      protocol,
 	                      pkt->wire, pkt->size, &tv, &tv);
 	if (ret != KNOT_EOK) {
-		return NS_PROC_FAIL; 
+		return KNOT_STATE_FAIL;
 	}
 	Dnstap__Dnstap dnstap = DNSTAP__DNSTAP__INIT;
 	dnstap.type = DNSTAP__DNSTAP__TYPE__MESSAGE;
@@ -74,7 +84,7 @@ static int log_message(int state, const knot_pkt_t *pkt, struct query_data *qdat
 	size_t size = 0;
 	dt_pack(&dnstap, &frame, &size);
 	if (frame == NULL) {
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	/* Submit a request. */
@@ -82,7 +92,7 @@ static int log_message(int state, const knot_pkt_t *pkt, struct query_data *qdat
 	                                   fstrm_free_wrapper, NULL);
 	if (res != fstrm_res_success) {
 		free(frame);
-		state = NS_PROC_FAIL;
+		state = KNOT_STATE_FAIL;
 	}
 
 	return state;
@@ -92,7 +102,7 @@ static int log_message(int state, const knot_pkt_t *pkt, struct query_data *qdat
 static int dnstap_message_log(int state, knot_pkt_t *pkt, struct query_data *qdata, void *ctx)
 {
 	if (pkt == NULL || qdata == NULL || ctx == NULL) {
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	return log_message(state, pkt, qdata, ctx);
@@ -104,7 +114,7 @@ static struct fstrm_writer* dnstap_unix_writer(const char *path)
 	struct fstrm_unix_writer_options *opt = NULL;
 	struct fstrm_writer_options *wopt = NULL;
 	struct fstrm_writer *writer = NULL;
-	
+
 	opt = fstrm_unix_writer_options_init();
 	if (opt == NULL) {
 		goto finish;
@@ -132,7 +142,7 @@ static struct fstrm_writer* dnstap_file_writer(const char *path)
 	struct fstrm_file_options *fopt = NULL;
 	struct fstrm_writer_options *wopt = NULL;
 	struct fstrm_writer *writer = NULL;
-	
+
 	fopt = fstrm_file_options_init();
 	if (fopt == NULL) {
 		goto finish;
@@ -170,9 +180,21 @@ static struct fstrm_writer* dnstap_writer(const char *path)
 
 int dnstap_load(struct query_plan *plan, struct query_module *self)
 {
+	if (plan == NULL || self == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	conf_val_t val = conf_mod_get(self->config, MOD_SINK, self->id);
+	if (val.code != KNOT_EOK) {
+		if (val.code == KNOT_EINVAL) {
+			MODULE_ERR("no sink for '%s'", self->id->data);
+		}
+		return val.code;
+	}
+	const char *sink = conf_str(&val);
+
 	/* Initialize the writer and the options. */
-	int ret = KNOT_ENOMEM;
-	struct fstrm_writer *writer = dnstap_writer(self->param);
+	struct fstrm_writer *writer = dnstap_writer(sink);
 	if (writer == NULL) {
 		goto fail;
 	}
@@ -184,7 +206,8 @@ int dnstap_load(struct query_plan *plan, struct query_module *self)
 	}
 
 	/* Initialize queues. */
-	size_t qcount = conf_udp_threads(self->config) + conf_tcp_threads(self->config);
+	size_t qcount = conf_udp_threads(self->config) +
+	                conf_tcp_threads(self->config);
 	fstrm_iothr_options_set_num_input_queues(opt, qcount);
 
 	/* Create the I/O thread. */
@@ -195,6 +218,7 @@ int dnstap_load(struct query_plan *plan, struct query_module *self)
 		fstrm_writer_destroy(&writer);
 		goto fail;
 	}
+
 	self->ctx = iothread;
 
 	/* Hook to the query plan. */
@@ -204,12 +228,16 @@ int dnstap_load(struct query_plan *plan, struct query_module *self)
 	return KNOT_EOK;
 
 fail:
-	MODULE_ERR("init failed, params '%s' (%s)", self->param, knot_strerror(ret));
-	return ret;
+	MODULE_ERR("failed for init sink '%s'", sink);
+	return KNOT_ENOMEM;
 }
 
 int dnstap_unload(struct query_module *self)
 {
+	if (self == NULL) {
+		return KNOT_EINVAL;
+	}
+
 	struct fstrm_iothr* iothread = self->ctx;
 	fstrm_iothr_destroy(&iothread);
 	return KNOT_EOK;
