@@ -1,5 +1,5 @@
 /*!
- * \file dnstap.h
+ * \file modules/dnstap.h
  *
  * \author Marek Vavrusa <marek.vavrusa@nic.cz>
  *
@@ -22,11 +22,14 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 #pragma once
 
 #include "knot/nameserver/query_module.h"
 
+/*! \brief Module scheme. */
+#define C_MOD_DNSTAP "\x0A""mod-dnstap"
+extern const yp_item_t scheme_mod_dnstap[];
+
 /*! \brief Module interface. */
 int dnstap_load(struct query_plan *plan, struct query_module *self);
 int dnstap_unload(struct query_module *self);
