@@ -14,15 +14,18 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <urcu.h>
+
 #include "knot/nameserver/axfr.h"
 #include "knot/nameserver/internet.h"
 #include "knot/nameserver/process_query.h"
 #include "knot/nameserver/process_answer.h"
 #include "knot/updates/apply.h"
 #include "knot/zone/zonefile.h"
-#include "common/debug.h"
+#include "knot/common/debug.h"
+#include "libknot/libknot.h"
 #include "libknot/descriptor.h"
-#include "common-knot/lists.h"
+#include "libknot/internal/lists.h"
 
 /* AXFR context. @note aliasing the generic xfr_proc */
 struct axfr_proc {
@@ -107,11 +110,11 @@ static int axfr_query_check(struct query_data *qdata)
 {
 	/* Check valid zone, transaction security and contents. */
 	NS_NEED_ZONE(qdata, KNOT_RCODE_NOTAUTH);
-	NS_NEED_AUTH(&qdata->zone->conf->acl.xfr_out, qdata);
+	NS_NEED_AUTH(qdata, qdata->zone->name, ACL_ACTION_TRANSFER);
 	/* Check expiration. */
 	NS_NEED_ZONE_CONTENTS(qdata, KNOT_RCODE_SERVFAIL);
 
-	return NS_PROC_DONE;
+	return KNOT_STATE_DONE;
 }
 
 static int axfr_query_init(struct query_data *qdata)
@@ -120,7 +123,7 @@ static int axfr_query_init(struct query_data *qdata)
 
 	/* Check AXFR query validity. */
 	int state = axfr_query_check(qdata);
-	if (state == NS_PROC_FAIL) {
+	if (state == KNOT_STATE_FAIL) {
 		if (qdata->rcode == KNOT_RCODE_FORMERR) {
 			return KNOT_EMALF;
 		} else {
@@ -132,7 +135,7 @@ static int axfr_query_init(struct query_data *qdata)
 	mm_ctx_t *mm = qdata->mm;
 
 	zone_contents_t *zone = qdata->zone->contents;
-	struct axfr_proc *axfr = mm->alloc(mm->ctx, sizeof(struct axfr_proc));
+	struct axfr_proc *axfr = mm_alloc(mm, sizeof(struct axfr_proc));
 	if (axfr == NULL) {
 		return KNOT_ENOMEM;
 	}
@@ -187,7 +190,7 @@ int xfr_process_list(knot_pkt_t *pkt, xfr_put_cb process_item,
 		if (ret == KNOT_EOK) { /* Finished. */
 			/* Complete change set. */
 			rem_node((node_t *)head);
-			mm->free(head);
+			mm_free(mm, head);
 		} else { /* Packet full or other error. */
 			break;
 		}
@@ -206,13 +209,13 @@ int xfr_process_list(knot_pkt_t *pkt, xfr_put_cb process_item,
 }
 
 /* AXFR-specific logging (internal, expects 'qdata' variable set). */
-#define AXFROUT_LOG(severity, msg...) \
-	QUERY_LOG(severity, qdata, "AXFR, outgoing", msg)
+#define AXFROUT_LOG(severity, msg, ...) \
+	QUERY_LOG(severity, qdata, "AXFR, outgoing", msg, ##__VA_ARGS__)
 
 int axfr_query_process(knot_pkt_t *pkt, struct query_data *qdata)
 {
 	if (pkt == NULL || qdata == NULL) {
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	int ret = KNOT_EOK;
@@ -221,7 +224,7 @@ int axfr_query_process(knot_pkt_t *pkt, struct query_data *qdata)
 	/* If AXFR is disabled, respond with NOTIMPL. */
 	if (qdata->param->proc_flags & NS_QUERY_NO_AXFR) {
 		qdata->rcode = KNOT_RCODE_NOTIMPL;
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	/* Initialize on first call. */
@@ -231,7 +234,7 @@ int axfr_query_process(knot_pkt_t *pkt, struct query_data *qdata)
 		if (ret != KNOT_EOK) {
 			AXFROUT_LOG(LOG_ERR, "failed to start (%s)",
 			            knot_strerror(ret));
-			return NS_PROC_FAIL;
+			return KNOT_STATE_FAIL;
 		} else {
 			AXFROUT_LOG(LOG_INFO, "started, serial %u",
 			           zone_contents_serial(qdata->zone->contents));
@@ -239,25 +242,25 @@ int axfr_query_process(knot_pkt_t *pkt, struct query_data *qdata)
 	}
 
 	/* Reserve space for TSIG. */
-	knot_pkt_reserve(pkt, tsig_wire_maxsize(qdata->sign.tsig_key));
+	knot_pkt_reserve(pkt, knot_tsig_wire_maxsize(&qdata->sign.tsig_key));
 
 	/* Answer current packet (or continue). */
 	struct axfr_proc *axfr = (struct axfr_proc *)qdata->ext;
 	ret = xfr_process_list(pkt, &axfr_process_node_tree, qdata);
 	switch(ret) {
 	case KNOT_ESPACE: /* Couldn't write more, send packet and continue. */
-		return NS_PROC_FULL; /* Check for more. */
+		return KNOT_STATE_PRODUCE; /* Check for more. */
 	case KNOT_EOK:    /* Last response. */
 		gettimeofday(&now, NULL);
 		AXFROUT_LOG(LOG_INFO,
 		            "finished, %.02f seconds, %u messages, %u bytes",
 		            time_diff(&axfr->proc.tstamp, &now) / 1000.0,
 		            axfr->proc.npkts, axfr->proc.nbytes);
-		return NS_PROC_DONE;
+		return KNOT_STATE_DONE;
 		break;
 	default:          /* Generic error. */
 		AXFROUT_LOG(LOG_ERR, "failed (%s)", knot_strerror(ret));
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 }
 #undef AXFROUT_LOG
@@ -304,8 +307,8 @@ static int axfr_answer_init(struct answer_data *data)
 }
 
 /* AXFR-specific logging (internal, expects 'adata' variable set). */
-#define AXFRIN_LOG(severity, msg...) \
-	ANSWER_LOG(severity, adata, "AXFR, incoming", msg)
+#define AXFRIN_LOG(severity, msg, ...) \
+	ANSWER_LOG(severity, adata, "AXFR, incoming", msg, ##__VA_ARGS__)
 
 static int axfr_answer_finalize(struct answer_data *adata)
 {
@@ -326,7 +329,6 @@ static int axfr_answer_finalize(struct answer_data *adata)
 	zone_t *zone = adata->param->zone;
 	zone_contents_t *old_contents =
 	                zone_switch_contents(zone, proc->contents);
-	zone->flags &= ~ZONE_EXPIRED;
 	synchronize_rcu();
 
 	AXFRIN_LOG(LOG_INFO, "finished, "
@@ -356,36 +358,36 @@ static int axfr_answer_packet(knot_pkt_t *pkt, struct xfr_proc *proc)
 	zcreator_t zc = {.z = proc->contents, .master = false, .ret = KNOT_EOK };
 
 	const knot_pktsection_t *answer = knot_pkt_section(pkt, KNOT_ANSWER);
+	const knot_rrset_t *answer_rr = knot_pkt_rr(answer, 0);
 	for (uint16_t i = 0; i < answer->count; ++i) {
-		const knot_rrset_t *rr = &answer->rr[i];
-		if (rr->type == KNOT_RRTYPE_SOA &&
+		if (answer_rr[i].type == KNOT_RRTYPE_SOA &&
 		    node_rrtype_exists(zc.z->apex, KNOT_RRTYPE_SOA)) {
-			return NS_PROC_DONE;
+			return KNOT_STATE_DONE;
 		} else {
-			int ret = zcreator_step(&zc, rr);
+			int ret = zcreator_step(&zc, &answer_rr[i]);
 			if (ret != KNOT_EOK) {
-				return NS_PROC_FAIL;
+				return KNOT_STATE_FAIL;
 			}
 		}
 	}
 
-	return NS_PROC_MORE;
+	return KNOT_STATE_CONSUME;
 }
 
 int axfr_answer_process(knot_pkt_t *pkt, struct answer_data *adata)
 {
 	if (pkt == NULL || adata == NULL) {
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	/* Check RCODE. */
 	uint8_t rcode = knot_wire_get_rcode(pkt->wire);
 	if (rcode != KNOT_RCODE_NOERROR) {
-		knot_lookup_table_t *lut = knot_lookup_by_id(knot_rcode_names, rcode);
+		lookup_table_t *lut = lookup_by_id(knot_rcode_names, rcode);
 		if (lut != NULL) {
-			AXFRIN_LOG(LOG_ERR, "server responded with %s", lut->name);
+			AXFRIN_LOG(LOG_WARNING, "server responded with %s", lut->name);
 		}
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	/* Initialize processing with first packet. */
@@ -395,8 +397,8 @@ int axfr_answer_process(knot_pkt_t *pkt, struct answer_data *adata)
 
 		int ret = axfr_answer_init(adata);
 		if (ret != KNOT_EOK) {
-			AXFRIN_LOG(LOG_ERR, "failed (%s)", knot_strerror(ret));
-			return NS_PROC_FAIL;
+			AXFRIN_LOG(LOG_WARNING, "failed (%s)", knot_strerror(ret));
+			return KNOT_STATE_FAIL;
 		}
 	} else {
 		NS_NEED_TSIG_SIGNED(&adata->param->tsig_ctx, 100);
@@ -404,12 +406,12 @@ int axfr_answer_process(knot_pkt_t *pkt, struct answer_data *adata)
 
 	/* Process answer packet. */
 	int ret = axfr_answer_packet(pkt, (struct xfr_proc *)adata->ext);
-	if (ret == NS_PROC_DONE) {
+	if (ret == KNOT_STATE_DONE) {
 		NS_NEED_TSIG_SIGNED(&adata->param->tsig_ctx, 0);
 		/* This was the last packet, finalize zone and publish it. */
 		int fret = axfr_answer_finalize(adata);
 		if (fret != KNOT_EOK) {
-			ret = NS_PROC_FAIL;
+			ret = KNOT_STATE_FAIL;
 		}
 	}
 
