@@ -14,15 +14,19 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <urcu.h>
+
 #include "knot/nameserver/ixfr.h"
 #include "knot/nameserver/axfr.h"
 #include "knot/nameserver/internet.h"
 #include "knot/nameserver/process_query.h"
 #include "knot/nameserver/process_answer.h"
 #include "knot/updates/apply.h"
-#include "common/debug.h"
+#include "knot/common/debug.h"
+#include "knot/zone/serial.h"
+#include "libknot/libknot.h"
 #include "libknot/descriptor.h"
-#include "libknot/util/utils.h"
+#include "libknot/internal/utils.h"
 #include "libknot/rrtype/soa.h"
 
 /* ------------------------ IXFR-out processing ----------------------------- */
@@ -54,8 +58,8 @@ struct ixfr_proc {
 };
 
 /* IXFR-out-specific logging (internal, expects 'qdata' variable set). */
-#define IXFROUT_LOG(severity, msg...) \
-	QUERY_LOG(severity, qdata, "IXFR, outgoing", msg)
+#define IXFROUT_LOG(severity, msg, ...) \
+	QUERY_LOG(severity, qdata, "IXFR, outgoing", msg, ##__VA_ARGS__)
 
 /*! \brief Helper macro for putting RRs into packet. */
 #define IXFR_SAFE_PUT(pkt, rr) \
@@ -154,7 +158,7 @@ static int ixfr_process_changeset(knot_pkt_t *pkt, const void *item,
 	struct query_data *qdata = ixfr->qdata; /*< Required for IXFROUT_LOG() */
 	const uint32_t serial_from = knot_soa_serial(&chgset->soa_from->rrs);
 	const uint32_t serial_to = knot_soa_serial(&chgset->soa_to->rrs);
-	IXFROUT_LOG(LOG_INFO, "serial %u -> %u", serial_from, serial_to);
+	IXFROUT_LOG(LOG_DEBUG, "serial %u -> %u", serial_from, serial_to);
 
 	return ret;
 }
@@ -162,7 +166,7 @@ static int ixfr_process_changeset(knot_pkt_t *pkt, const void *item,
 #undef IXFR_SAFE_PUT
 
 /*! \brief Loads IXFRs from journal. */
-static int ixfr_load_chsets(list_t *chgsets, zone_t *zone,
+static int ixfr_load_chsets(list_t *chgsets, const zone_t *zone,
                             const knot_rrset_t *their_soa)
 {
 	assert(chgsets);
@@ -171,14 +175,16 @@ static int ixfr_load_chsets(list_t *chgsets, zone_t *zone,
 	/* Compare serials. */
 	uint32_t serial_to = zone_contents_serial(zone->contents);
 	uint32_t serial_from = knot_soa_serial(&their_soa->rrs);
-	int ret = knot_serial_compare(serial_to, serial_from);
+	int ret = serial_compare(serial_to, serial_from);
 	if (ret <= 0) { /* We have older/same age zone. */
 		return KNOT_EUPTODATE;
 	}
 
-	pthread_mutex_lock(&zone->journal_lock);
-	ret = journal_load_changesets(zone, chgsets, serial_from, serial_to);
-	pthread_mutex_unlock(&zone->journal_lock);
+	char *path = conf_journalfile(conf(), zone->name);
+	pthread_mutex_lock((pthread_mutex_t *)&zone->journal_lock);
+	ret = journal_load_changesets(path, zone, chgsets, serial_from, serial_to);
+	pthread_mutex_unlock((pthread_mutex_t *)&zone->journal_lock);
+	free(path);
 
 	if (ret != KNOT_EOK) {
 		changesets_free(chgsets);
@@ -197,19 +203,19 @@ static int ixfr_query_check(struct query_data *qdata)
 	NS_NEED_QTYPE(qdata, KNOT_RRTYPE_IXFR, KNOT_RCODE_FORMERR);
 	/* Need SOA authority record. */
 	const knot_pktsection_t *authority = knot_pkt_section(qdata->query, KNOT_AUTHORITY);
-	const knot_rrset_t *their_soa = &authority->rr[0];
+	const knot_rrset_t *their_soa = knot_pkt_rr(authority, 0);
 	if (authority->count < 1 || their_soa->type != KNOT_RRTYPE_SOA) {
 		qdata->rcode = KNOT_RCODE_FORMERR;
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 	/* SOA needs to match QNAME. */
 	NS_NEED_QNAME(qdata, their_soa->owner, KNOT_RCODE_FORMERR);
 
 	/* Check transcation security and zone contents. */
-	NS_NEED_AUTH(&qdata->zone->conf->acl.xfr_out, qdata);
+	NS_NEED_AUTH(qdata, qdata->zone->name, ACL_ACTION_TRANSFER);
 	NS_NEED_ZONE_CONTENTS(qdata, KNOT_RCODE_SERVFAIL); /* Check expiration. */
 
-	return NS_PROC_DONE;
+	return KNOT_STATE_DONE;
 }
 
 /*! \brief Cleans up ixfr processing context. */
@@ -221,7 +227,7 @@ static void ixfr_answer_cleanup(struct query_data *qdata)
 	ptrlist_free(&ixfr->proc.nodes, mm);
 	changeset_iter_clear(&ixfr->cur);
 	changesets_free(&ixfr->changesets);
-	mm->free(qdata->ext);
+	mm_free(mm, qdata->ext);
 
 	/* Allow zone changes (finished). */
 	rcu_read_unlock();
@@ -232,7 +238,7 @@ static int ixfr_answer_init(struct query_data *qdata)
 {
 	/* Check IXFR query validity. */
 	int state = ixfr_query_check(qdata);
-	if (state == NS_PROC_FAIL) {
+	if (state == KNOT_STATE_FAIL) {
 		if (qdata->rcode == KNOT_RCODE_FORMERR) {
 			return KNOT_EMALF;
 		} else {
@@ -241,7 +247,8 @@ static int ixfr_answer_init(struct query_data *qdata)
 	}
 
 	/* Compare serials. */
-	const knot_rrset_t *their_soa = &knot_pkt_section(qdata->query, KNOT_AUTHORITY)->rr[0];
+	const knot_pktsection_t *authority = knot_pkt_section(qdata->query, KNOT_AUTHORITY);
+	const knot_rrset_t *their_soa = knot_pkt_rr(authority, 0);
 	list_t chgsets;
 	init_list(&chgsets);
 	int ret = ixfr_load_chsets(&chgsets, (zone_t *)qdata->zone, their_soa);
@@ -252,7 +259,7 @@ static int ixfr_answer_init(struct query_data *qdata)
 
 	/* Initialize transfer processing. */
 	mm_ctx_t *mm = qdata->mm;
-	struct ixfr_proc *xfer = mm->alloc(mm->ctx, sizeof(struct ixfr_proc));
+	struct ixfr_proc *xfer = mm_alloc(mm, sizeof(struct ixfr_proc));
 	if (xfer == NULL) {
 		changesets_free(&chgsets);
 		return KNOT_ENOMEM;
@@ -294,52 +301,46 @@ static int ixfr_answer_soa(knot_pkt_t *pkt, struct query_data *qdata)
 {
 	dbg_ns("%s: answering IXFR/SOA\n", __func__);
 	if (pkt == NULL || qdata == NULL) {
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	/* Check query. */
 	int state = ixfr_query_check(qdata);
-	if (state == NS_PROC_FAIL) {
+	if (state == KNOT_STATE_FAIL) {
 		return state; /* Malformed query. */
 	}
 
 	/* Reserve space for TSIG. */
-	knot_pkt_reserve(pkt, tsig_wire_maxsize(qdata->sign.tsig_key));
+	knot_pkt_reserve(pkt, knot_tsig_wire_maxsize(&qdata->sign.tsig_key));
 
 	/* Guaranteed to have zone contents. */
 	const zone_node_t *apex = qdata->zone->contents->apex;
 	knot_rrset_t soa_rr = node_rrset(apex, KNOT_RRTYPE_SOA);
 	if (knot_rrset_empty(&soa_rr)) {
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 	int ret = knot_pkt_put(pkt, 0, &soa_rr, 0);
 	if (ret != KNOT_EOK) {
 		qdata->rcode = KNOT_RCODE_SERVFAIL;
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
-	return NS_PROC_DONE;
+	return KNOT_STATE_DONE;
 }
 
 /* ------------------------- IXFR-in processing ----------------------------- */
 
 /* IXFR-in-specific logging (internal, expects 'adata' variable set). */
-#define IXFRIN_LOG(severity, msg...) \
-	ANSWER_LOG(severity, adata, "IXFR, incoming", msg)
-
-/*! \brief Checks whether IXFR response contains enough data for processing. */
-static bool ixfr_enough_data(const knot_pkt_t *pkt)
-{
-	const knot_pktsection_t *answer = knot_pkt_section(pkt, KNOT_ANSWER);
-	return answer->count >= 2;
-}
+#define IXFRIN_LOG(severity, msg, ...) \
+	ANSWER_LOG(severity, adata, "IXFR, incoming", msg, ##__VA_ARGS__)
 
 /*! \brief Checks whether server responded with AXFR-style IXFR. */
 static bool ixfr_is_axfr(const knot_pkt_t *pkt)
 {
 	const knot_pktsection_t *answer = knot_pkt_section(pkt, KNOT_ANSWER);
-	return answer->rr[0].type == KNOT_RRTYPE_SOA &&
-	       answer->rr[1].type != KNOT_RRTYPE_SOA;
+	return answer->count >= 2 &&
+	       knot_pkt_rr(answer, 0)->type == KNOT_RRTYPE_SOA &&
+	       knot_pkt_rr(answer, 1)->type != KNOT_RRTYPE_SOA;
 }
 
 /*! \brief Cleans up data allocated by IXFR-in processing. */
@@ -402,7 +403,6 @@ static int ixfrin_finalize(struct answer_data *adata)
 
 	/* Switch zone contents. */
 	zone_contents_t *old_contents = zone_switch_contents(ixfr->zone, new_contents);
-	ixfr->zone->flags &= ~ZONE_EXPIRED;
 	synchronize_rcu();
 	update_free_zone(&old_contents);
 
@@ -573,7 +573,7 @@ static bool out_of_zone(const knot_rrset_t *rr, struct ixfr_proc *proc)
  * \param pkt    Packet containing the IXFR reply in wire format.
  * \param adata  Answer data, including processing context.
  *
- * \return NS_PROC_MORE, NS_PROC_DONE, NS_PROC_FAIL
+ * \return KNOT_STATE_CONSUME, KNOT_STATE_DONE, KNOT_STATE_FAIL
  */
 static int process_ixfrin_packet(knot_pkt_t *pkt, struct answer_data *adata)
 {
@@ -588,10 +588,10 @@ static int process_ixfrin_packet(knot_pkt_t *pkt, struct answer_data *adata)
 	for (uint16_t i = 0; i < answer->count; ++i) {
 		if (journal_limit_exceeded(ixfr)) {
 			IXFRIN_LOG(LOG_WARNING, "journal is full");
-			return NS_PROC_FAIL;
+			return KNOT_STATE_FAIL;
 		}
 
-		const knot_rrset_t *rr = &answer->rr[i];
+		const knot_rrset_t *rr = knot_pkt_rr(answer, i);
 		if (out_of_zone(rr, ixfr)) {
 			continue;
 		}
@@ -599,16 +599,16 @@ static int process_ixfrin_packet(knot_pkt_t *pkt, struct answer_data *adata)
 		int ret = ixfrin_step(rr, ixfr);
 		if (ret != KNOT_EOK) {
 			IXFRIN_LOG(LOG_WARNING, "failed (%s)", knot_strerror(ret));
-			return NS_PROC_FAIL;
+			return KNOT_STATE_FAIL;
 		}
 
 		if (ixfr->state == IXFR_DONE) {
 			// Transfer done, do not consume more RRs.
-			return NS_PROC_DONE;
+			return KNOT_STATE_DONE;
 		}
 	}
 
-	return NS_PROC_MORE;
+	return KNOT_STATE_CONSUME;
 }
 
 /* --------------------------------- API ------------------------------------ */
@@ -616,7 +616,7 @@ static int process_ixfrin_packet(knot_pkt_t *pkt, struct answer_data *adata)
 int ixfr_query(knot_pkt_t *pkt, struct query_data *qdata)
 {
 	if (pkt == NULL || qdata == NULL) {
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	int ret = KNOT_EOK;
@@ -648,71 +648,85 @@ int ixfr_query(knot_pkt_t *pkt, struct query_data *qdata)
 			return axfr_query_process(pkt, qdata);
 		default:            /* Server errors. */
 			IXFROUT_LOG(LOG_ERR, "failed to start (%s)", knot_strerror(ret));
-			return NS_PROC_FAIL;
+			return KNOT_STATE_FAIL;
 		}
 	}
 
 	/* Reserve space for TSIG. */
-	knot_pkt_reserve(pkt, tsig_wire_maxsize(qdata->sign.tsig_key));
+	knot_pkt_reserve(pkt, knot_tsig_wire_maxsize(&qdata->sign.tsig_key));
 
 	/* Answer current packet (or continue). */
 	ret = xfr_process_list(pkt, &ixfr_process_changeset, qdata);
 	switch(ret) {
 	case KNOT_ESPACE: /* Couldn't write more, send packet and continue. */
-		return NS_PROC_FULL; /* Check for more. */
+		return KNOT_STATE_PRODUCE; /* Check for more. */
 	case KNOT_EOK:    /* Last response. */
 		gettimeofday(&now, NULL);
 		IXFROUT_LOG(LOG_INFO,
 		            "finished, %.02f seconds, %u messages, %u bytes",
 		            time_diff(&ixfr->proc.tstamp, &now) / 1000.0,
 		            ixfr->proc.npkts, ixfr->proc.nbytes);
-		ret = NS_PROC_DONE;
+		ret = KNOT_STATE_DONE;
 		break;
 	default:          /* Generic error. */
 		IXFROUT_LOG(LOG_ERR, "failed (%s)", knot_strerror(ret));
-		ret = NS_PROC_FAIL;
+		ret = KNOT_STATE_FAIL;
 		break;
 	}
 
 	return ret;
 }
 
+static int check_format(knot_pkt_t *pkt)
+{
+	const knot_pktsection_t *answer = knot_pkt_section(pkt, KNOT_ANSWER);
+
+	if (answer->count >= 1 && knot_pkt_rr(answer, 0)->type == KNOT_RRTYPE_SOA) {
+		return KNOT_EOK;
+	} else {
+		return KNOT_EMALF;
+	}
+}
+
 int ixfr_process_answer(knot_pkt_t *pkt, struct answer_data *adata)
 {
 	if (pkt == NULL || adata == NULL) {
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
-	
-	if (adata->ext == NULL) {
-		if (!ixfr_enough_data(pkt)) {
-			return NS_PROC_FAIL;
+
+	/* Check RCODE. */
+	uint8_t rcode = knot_wire_get_rcode(pkt->wire);
+	if (rcode != KNOT_RCODE_NOERROR) {
+		lookup_table_t *lut = lookup_by_id(knot_rcode_names, rcode);
+		if (lut != NULL) {
+			IXFRIN_LOG(LOG_WARNING, "server responded with %s", lut->name);
 		}
-	
+		return KNOT_STATE_FAIL;
+	}
+
+	if (adata->ext == NULL) {
+		if (check_format(pkt) != KNOT_EOK) {
+			IXFRIN_LOG(LOG_WARNING, "malformed response");
+			return KNOT_STATE_FAIL;
+		}
+
 		/* Check for AXFR-style IXFR. */
 		if (ixfr_is_axfr(pkt)) {
 			IXFRIN_LOG(LOG_NOTICE, "receiving AXFR-style IXFR");
 			adata->response_type = KNOT_RESPONSE_AXFR;
 			return axfr_answer_process(pkt, adata);
 		}
-	}
-	
-	/* Check RCODE. */
-	uint8_t rcode = knot_wire_get_rcode(pkt->wire);
-	if (rcode != KNOT_RCODE_NOERROR) {
-		knot_lookup_table_t *lut = knot_lookup_by_id(knot_rcode_names, rcode);
-		if (lut != NULL) {
-			IXFRIN_LOG(LOG_WARNING, "server responded with %s", lut->name);
-		}
-		return NS_PROC_FAIL;
-	}
-	
-	/* Initialize processing with first packet. */
-	if (adata->ext == NULL) {
+
+		/* Initialize processing with first packet. */
 		NS_NEED_TSIG_SIGNED(&adata->param->tsig_ctx, 0);
 		if (!zone_transfer_needed(adata->param->zone, pkt)) {
-			IXFRIN_LOG(LOG_WARNING, "master sent older serial, "
-			                        "ignoring");
-			return NS_PROC_DONE;
+			if (knot_pkt_section(pkt, KNOT_ANSWER)->count > 1) {
+				IXFRIN_LOG(LOG_WARNING, "old data, ignoring");
+			} else {
+				/* Single-SOA answer. */
+				IXFRIN_LOG(LOG_INFO, "zone is up-to-date");
+			}
+			return KNOT_STATE_DONE;
 		}
 
 		IXFRIN_LOG(LOG_INFO, "starting");
@@ -720,18 +734,18 @@ int ixfr_process_answer(knot_pkt_t *pkt, struct answer_data *adata)
 		int ret = ixfrin_answer_init(adata);
 		if (ret != KNOT_EOK) {
 			IXFRIN_LOG(LOG_WARNING, "failed (%s)", knot_strerror(ret));
-			return NS_PROC_FAIL;
+			return KNOT_STATE_FAIL;
 		}
 	} else {
 		NS_NEED_TSIG_SIGNED(&adata->param->tsig_ctx, 100);
 	}
 
 	int ret = process_ixfrin_packet(pkt, adata);
-	if (ret == NS_PROC_DONE) {
+	if (ret == KNOT_STATE_DONE) {
 		NS_NEED_TSIG_SIGNED(&adata->param->tsig_ctx, 0);
 		int fret = ixfrin_finalize(adata);
 		if (fret != KNOT_EOK) {
-			ret = NS_PROC_FAIL;
+			ret = KNOT_STATE_FAIL;
 		}
 	}
 
