@@ -1,10 +1,26 @@
+/*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#include "libknot/libknot.h"
 #include "knot/nameserver/nsec_proofs.h"
 #include "knot/nameserver/process_query.h"
 #include "knot/nameserver/internet.h"
 #include "knot/dnssec/zone-nsec.h"
 
-#include "libknot/common.h"
-#include "common/debug.h"
+#include "knot/common/debug.h"
 
 #define DNSSEC_ENABLED 1
 
@@ -64,7 +80,7 @@ static int ns_put_nsec3_from_node(const zone_node_t *node,
 		return KNOT_EOK;
 	}
 
-	int res = ns_put_rr(resp, &rrset, &rrsigs, COMPR_HINT_NONE,
+	int res = ns_put_rr(resp, &rrset, &rrsigs, KNOT_COMPR_HINT_NONE,
 	                    KNOT_PF_CHECKDUP, qdata);
 
 	/*! \note TC bit is already set, if something went wrong. */
@@ -286,7 +302,7 @@ static int ns_put_nsec_wildcard(const zone_contents_t *zone,
 		previous = zone_contents_find_previous(zone, qname);
 		assert(previous != NULL);
 
-		while (previous->flags & NODE_FLAGS_NONAUTH) {
+		while (previous->flags != NODE_FLAGS_AUTH) {
 			previous = previous->prev;
 		}
 	}
@@ -297,7 +313,7 @@ static int ns_put_nsec_wildcard(const zone_contents_t *zone,
 	if (!knot_rrset_empty(&rrset)) {
 		knot_rrset_t rrsigs = node_rrset(previous, KNOT_RRTYPE_RRSIG);
 		// NSEC proving that there is no node with the searched name
-		ret = ns_put_rr(resp, &rrset, &rrsigs, COMPR_HINT_NONE, 0, qdata);
+		ret = ns_put_rr(resp, &rrset, &rrsigs, KNOT_COMPR_HINT_NONE, 0, qdata);
 	}
 
 	return ret;
@@ -474,7 +490,7 @@ static int ns_put_nsec_nxdomain(const knot_dname_t *qname,
 	if (previous == NULL) {
 		previous = zone_contents_find_previous(zone, qname);
 		assert(previous != NULL);
-		while (previous->flags & NODE_FLAGS_NONAUTH) {
+		while (previous->flags != NODE_FLAGS_AUTH) {
 			previous = previous->prev;
 		}
 	}
@@ -494,7 +510,7 @@ dbg_ns_exec_verb(
 		return KNOT_EOK;
 	}
 
-	int ret = ns_put_rr(resp, &rrset, &rrsigs, COMPR_HINT_NONE, 0, qdata);
+	int ret = ns_put_rr(resp, &rrset, &rrsigs, KNOT_COMPR_HINT_NONE, 0, qdata);
 	if (ret != KNOT_EOK) {
 		dbg_ns("Failed to add NSEC for NXDOMAIN to response: %s\n",
 		       knot_strerror(ret));
@@ -513,7 +529,7 @@ dbg_ns_exec_verb(
 	}
 
 	const zone_node_t *prev_new = zone_contents_find_previous(zone, wildcard);
-	while (prev_new->flags & NODE_FLAGS_NONAUTH) {
+	while (prev_new->flags != NODE_FLAGS_AUTH) {
 		prev_new = prev_new->prev;
 	}
 
@@ -527,7 +543,7 @@ dbg_ns_exec_verb(
 			// bad zone, ignore
 			return KNOT_EOK;
 		}
-		ret = ns_put_rr(resp, &rrset, &rrsigs, COMPR_HINT_NONE, 0, qdata);
+		ret = ns_put_rr(resp, &rrset, &rrsigs, KNOT_COMPR_HINT_NONE, 0, qdata);
 		if (ret != KNOT_EOK) {
 			dbg_ns("Failed to add second NSEC for NXDOMAIN to "
 			       "response: %s\n", knot_strerror(ret));
@@ -677,7 +693,7 @@ static int ns_put_nsec_nsec3_nodata(const zone_node_t *node,
 		if (!knot_rrset_empty(&rrset)) {
 			dbg_ns_detail("Putting the RRSet to Authority\n");
 			knot_rrset_t rrsigs = node_rrset(node, KNOT_RRTYPE_RRSIG);
-			ret = ns_put_rr(resp, &rrset, &rrsigs, COMPR_HINT_NONE, 0, qdata);
+			ret = ns_put_rr(resp, &rrset, &rrsigs, KNOT_COMPR_HINT_NONE, 0, qdata);
 		}
 	}
 
@@ -749,7 +765,7 @@ int nsec_prove_dp_security(knot_pkt_t *pkt, struct query_data *qdata)
 	knot_rrset_t rrset = node_rrset(qdata->node, KNOT_RRTYPE_DS);
 	if (!knot_rrset_empty(&rrset)) {
 		knot_rrset_t rrsigs = node_rrset(qdata->node, KNOT_RRTYPE_RRSIG);
-		return ns_put_rr(pkt, &rrset, &rrsigs, COMPR_HINT_NONE, 0, qdata);
+		return ns_put_rr(pkt, &rrset, &rrsigs, KNOT_COMPR_HINT_NONE, 0, qdata);
 	}
 
 	/* DS doesn't exist => NODATA proof. */
@@ -772,7 +788,7 @@ int nsec_append_rrsigs(knot_pkt_t *pkt, struct query_data *qdata, bool optional)
 	struct rrsig_info *info = NULL;
 	WALK_LIST(info, qdata->rrsigs) {
 		knot_rrset_t *rrsig = &info->synth_rrsig;
-		uint16_t compr_hint = info->rrinfo->compress_ptr[COMPR_HINT_OWNER];
+		uint16_t compr_hint = info->rrinfo->compress_ptr[KNOT_COMPR_HINT_OWNER];
 		ret = knot_pkt_put(pkt, compr_hint, rrsig, flags);
 		if (ret != KNOT_EOK) {
 			break;
