@@ -1,7 +1,9 @@
 #include <stdio.h>
 #include <urcu.h>
 
+#include "dnssec/tsig.h"
 #include "knot/nameserver/process_query.h"
+#include "knot/nameserver/query_module.h"
 #include "knot/nameserver/chaos.h"
 #include "knot/nameserver/internet.h"
 #include "knot/nameserver/axfr.h"
@@ -13,20 +15,20 @@
 #include "knot/server/rrl.h"
 #include "knot/updates/acl.h"
 #include "knot/conf/conf.h"
-#include "libknot/tsig-op.h"
-#include "libknot/descriptor.h"
-#include "common/debug.h"
+#include "knot/common/debug.h"
+#include "libknot/libknot.h"
+#include "libknot/internal/macros.h"
 
 /*! \brief Accessor to query-specific data. */
 #define QUERY_DATA(ctx) ((struct query_data *)(ctx)->data)
 
 /*! \brief Reinitialize query data structure. */
-static void query_data_init(knot_process_t *ctx, void *module_param)
+static void query_data_init(knot_layer_t *ctx, void *module_param)
 {
 	/* Initialize persistent data. */
 	struct query_data *data = QUERY_DATA(ctx);
 	memset(data, 0, sizeof(struct query_data));
-	data->mm = &ctx->mm;
+	data->mm = ctx->mm;
 	data->param = (struct process_query_param*)module_param;
 
 	/* Initialize lists. */
@@ -34,21 +36,20 @@ static void query_data_init(knot_process_t *ctx, void *module_param)
 	init_list(&data->rrsigs);
 }
 
-static int process_query_begin(knot_process_t *ctx, void *module_param)
+static int process_query_begin(knot_layer_t *ctx, void *module_param)
 {
 	/* Initialize context. */
 	assert(ctx);
-	ctx->type = NS_PROC_QUERY_ID;
-	ctx->data = mm_alloc(&ctx->mm, sizeof(struct query_data));
+	ctx->data = mm_alloc(ctx->mm, sizeof(struct query_data));
 
 	/* Initialize persistent data. */
 	query_data_init(ctx, module_param);
 
 	/* Await packet. */
-	return NS_PROC_MORE;
+	return KNOT_STATE_CONSUME;
 }
 
-static int process_query_reset(knot_process_t *ctx)
+static int process_query_reset(knot_layer_t *ctx)
 {
 	assert(ctx);
 	struct query_data *qdata = QUERY_DATA(ctx);
@@ -57,7 +58,6 @@ static int process_query_reset(knot_process_t *ctx)
 	struct process_query_param *module_param = qdata->param;
 
 	/* Free allocated data. */
-	knot_pkt_free(&qdata->query);
 	ptrlist_free(&qdata->wildcards, qdata->mm);
 	nsec_clear_rrsigs(qdata);
 	knot_rrset_clear(&qdata->opt_rr, qdata->mm);
@@ -69,33 +69,31 @@ static int process_query_reset(knot_process_t *ctx)
 	query_data_init(ctx, module_param);
 
 	/* Await packet. */
-	return NS_PROC_MORE;
+	return KNOT_STATE_CONSUME;
 }
 
-static int process_query_finish(knot_process_t *ctx)
+static int process_query_finish(knot_layer_t *ctx)
 {
 	process_query_reset(ctx);
-	ctx->mm.free(ctx->data);
+	mm_free(ctx->mm, ctx->data);
 	ctx->data = NULL;
 
-	return NS_PROC_NOOP;
+	return KNOT_STATE_NOOP;
 }
 
-static int process_query_in(knot_pkt_t *pkt, knot_process_t *ctx)
+static int process_query_in(knot_layer_t *ctx, knot_pkt_t *pkt)
 {
 	assert(pkt && ctx);
 	struct query_data *qdata = QUERY_DATA(ctx);
 
 	/* Check if at least header is parsed. */
 	if (pkt->parsed < KNOT_WIRE_HEADER_SIZE) {
-		knot_pkt_free(&pkt);
-		return NS_PROC_NOOP; /* Ignore. */
+		return KNOT_STATE_NOOP; /* Ignore. */
 	}
 
 	/* Accept only queries. */
 	if (knot_wire_get_qr(pkt->wire)) {
-		knot_pkt_free(&pkt);
-		return NS_PROC_NOOP; /* Ignore. */
+		return KNOT_STATE_NOOP; /* Ignore. */
 	}
 
 	/* Store for processing. */
@@ -103,16 +101,16 @@ static int process_query_in(knot_pkt_t *pkt, knot_process_t *ctx)
 	qdata->packet_type = knot_pkt_type(pkt);
 
 	/* Declare having response. */
-	return NS_PROC_FULL;
+	return KNOT_STATE_PRODUCE;
 }
 
 /*!
  * \brief Create a response for a given query in the INTERNET class.
  */
-static int query_internet(knot_pkt_t *pkt, knot_process_t *ctx)
+static int query_internet(knot_pkt_t *pkt, knot_layer_t *ctx)
 {
 	struct query_data *data = QUERY_DATA(ctx);
-	int next_state = NS_PROC_FAIL;
+	int next_state = KNOT_STATE_FAIL;
 	dbg_ns("%s(%p, %p, pkt_type=%u)\n", __func__, pkt, ctx, data->packet_type);
 
 	switch(data->packet_type) {
@@ -134,7 +132,7 @@ static int query_internet(knot_pkt_t *pkt, knot_process_t *ctx)
 	default:
 		/* Nothing else is supported. */
 		data->rcode = KNOT_RCODE_NOTIMPL;
-		next_state = NS_PROC_FAIL;
+		next_state = KNOT_STATE_FAIL;
 		break;
 	}
 
@@ -144,7 +142,7 @@ static int query_internet(knot_pkt_t *pkt, knot_process_t *ctx)
 /*!
  * \brief Create a response for a given query in the CHAOS class.
  */
-static int query_chaos(knot_pkt_t *pkt, knot_process_t *ctx)
+static int query_chaos(knot_pkt_t *pkt, knot_layer_t *ctx)
 {
 	dbg_ns("%s(%p, %p)\n", __func__, pkt, ctx);
 	struct query_data *data = QUERY_DATA(ctx);
@@ -152,16 +150,16 @@ static int query_chaos(knot_pkt_t *pkt, knot_process_t *ctx)
 	/* Nothing except normal queries is supported. */
 	if (data->packet_type != KNOT_QUERY_NORMAL) {
 		data->rcode = KNOT_RCODE_NOTIMPL;
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	data->rcode = knot_chaos_answer(pkt);
 	if (data->rcode != KNOT_RCODE_NOERROR) {
 		dbg_ns("%s: failed with RCODE=%d\n", __func__, data->rcode);
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
-	return NS_PROC_DONE;
+	return KNOT_STATE_DONE;
 }
 
 /*! \brief Find zone for given question. */
@@ -222,7 +220,8 @@ static int answer_edns_init(const knot_pkt_t *query, knot_pkt_t *resp,
 	}
 
 	/* Initialize OPT record. */
-	int ret = knot_edns_init(&qdata->opt_rr, conf()->max_udp_payload, 0,
+	conf_val_t val = conf_get(conf(), C_SRV, C_MAX_UDP_PAYLOAD);
+	int ret = knot_edns_init(&qdata->opt_rr, conf_int(&val), 0,
 	                     KNOT_EDNS_VERSION, qdata->mm);
 	if (ret != KNOT_EOK) {
 		return ret;
@@ -230,7 +229,7 @@ static int answer_edns_init(const knot_pkt_t *query, knot_pkt_t *resp,
 
 	/* Check supported version. */
 	if (knot_edns_get_version(query->opt_rr) != KNOT_EDNS_VERSION) {
-		qdata->rcode_ext = KNOT_RCODE_BADVERS;
+		qdata->rcode = KNOT_RCODE_BADVERS;
 	}
 
 	/* Set DO bit if set (DNSSEC requested). */
@@ -239,12 +238,27 @@ static int answer_edns_init(const knot_pkt_t *query, knot_pkt_t *resp,
 	}
 
 	/* Append NSID if requested and available. */
-	if (knot_edns_has_nsid(query->opt_rr) && conf()->nsid_len > 0) {
-		ret = knot_edns_add_option(&qdata->opt_rr,
-		                           KNOT_EDNS_OPTION_NSID, conf()->nsid_len,
-		                           (const uint8_t*)conf()->nsid, qdata->mm);
-		if (ret != KNOT_EOK) {
-			return ret;
+	val = conf_get(conf(), C_SRV, C_NSID);
+	if (knot_edns_has_nsid(query->opt_rr)) {
+		conf_data(&val);
+		if (val.code != KNOT_EOK) {
+			ret = knot_edns_add_option(&qdata->opt_rr,
+			                           KNOT_EDNS_OPTION_NSID,
+			                           strlen(conf()->hostname),
+			                           (uint8_t *)conf()->hostname,
+			                           qdata->mm);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+		} else if (val.len > 0) {
+			ret = knot_edns_add_option(&qdata->opt_rr,
+			                           KNOT_EDNS_OPTION_NSID,
+			                           val.len,
+			                           val.data,
+			                           qdata->mm);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
 		}
 	}
 
@@ -257,25 +271,27 @@ static int answer_edns_put(knot_pkt_t *resp, struct query_data *qdata)
 		return KNOT_EOK;
 	}
 
-	/* Write back extended RCODE. */
-	if (qdata->rcode_ext != 0) {
-		knot_wire_set_rcode(resp->wire, KNOT_EDNS_RCODE_LO(qdata->rcode_ext));
-		knot_edns_set_ext_rcode(&qdata->opt_rr, KNOT_EDNS_RCODE_HI(qdata->rcode_ext));
-	}
-
 	/* Reclaim reserved size. */
 	int ret = knot_pkt_reclaim(resp, knot_edns_wire_size(&qdata->opt_rr));
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
+	uint8_t *wire_end = resp->wire + resp->size;
+
 	/* Write to packet. */
 	assert(resp->current == KNOT_ADDITIONAL);
-	return knot_pkt_put(resp, COMPR_HINT_NONE, &qdata->opt_rr, 0);
+	ret = knot_pkt_put(resp, KNOT_COMPR_HINT_NONE, &qdata->opt_rr, 0);
+	if (ret == KNOT_EOK) {
+		/* Save position of the OPT RR. */
+		qdata->opt_rr_pos = wire_end;
+	}
+
+	return ret;
 }
 
 /*! \brief Initialize response, sizes and find zone from which we're going to answer. */
-static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_process_t *ctx)
+static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_layer_t *ctx)
 {
 	struct query_data *qdata = QUERY_DATA(ctx);
 	server_t *server = qdata->param->server;
@@ -309,12 +325,7 @@ static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_proces
 
 	/* Setup EDNS. */
 	ret = answer_edns_init(query, resp, qdata);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
-	/* If Extended RCODE is set, do not continue with query processing. */
-	if (qdata->rcode_ext != 0) {
+	if (ret != KNOT_EOK || qdata->rcode != 0) {
 		return KNOT_ERROR;
 	}
 
@@ -323,8 +334,9 @@ static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_proces
 	if (has_limit) {
 		resp->max_size = KNOT_WIRE_MIN_PKTSIZE;
 		if (knot_pkt_has_edns(query)) {
+			conf_val_t val = conf_get(conf(), C_SRV, C_MAX_UDP_PAYLOAD);
 			uint16_t client = knot_edns_get_payload(query->opt_rr);
-			uint16_t server = conf()->max_udp_payload;
+			uint16_t server = conf_int(&val);
 			uint16_t transfer = MIN(client, server);
 			resp->max_size = MAX(resp->max_size, transfer);
 		}
@@ -335,7 +347,27 @@ static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_proces
 	return ret;
 }
 
-static int process_query_err(knot_pkt_t *pkt, knot_process_t *ctx)
+static int set_rcode_to_packet(knot_pkt_t *pkt, struct query_data *qdata)
+{
+	int ret = KNOT_EOK;
+	uint8_t ext_rcode = KNOT_EDNS_RCODE_HI(qdata->rcode);
+
+	if (ext_rcode != 0) {
+		/* No OPT RR and Ext RCODE results in SERVFAIL. */
+		if (qdata->opt_rr_pos == NULL) {
+			qdata->rcode = KNOT_RCODE_SERVFAIL;
+			ret = KNOT_ERROR;
+		} else {
+			knot_edns_set_ext_rcode_wire(qdata->opt_rr_pos, ext_rcode);
+		}
+	}
+
+	knot_wire_set_rcode(pkt->wire, KNOT_EDNS_RCODE_LO(qdata->rcode));
+
+	return ret;
+}
+
+static int process_query_err(knot_layer_t *ctx, knot_pkt_t *pkt)
 {
 	assert(pkt && ctx);
 	struct query_data *qdata = QUERY_DATA(ctx);
@@ -349,29 +381,30 @@ static int process_query_err(knot_pkt_t *pkt, knot_process_t *ctx)
 	/* Restore original QNAME. */
 	process_query_qname_case_restore(qdata, pkt);
 
-	/* Set RCODE. */
-	knot_wire_set_rcode(pkt->wire, qdata->rcode);
-
 	/* Add OPT and TSIG (best effort, send reply anyway if fails). */
 	if (pkt->current != KNOT_ADDITIONAL) {
 		knot_pkt_begin(pkt, KNOT_ADDITIONAL);
 	}
 
 	/* Put OPT RR to the additional section. */
-	if (answer_edns_reserve(pkt, qdata) == KNOT_EOK) {
+	int ret = answer_edns_reserve(pkt, qdata);
+	if (ret == KNOT_EOK) {
 		(void) answer_edns_put(pkt, qdata);
 	}
 
+	/* Set final RCODE to packet. */
+	(void) set_rcode_to_packet(pkt, qdata);
+
 	/* Transaction security (if applicable). */
 	(void) process_query_sign_response(pkt, qdata);
 
-	return NS_PROC_DONE;
+	return KNOT_STATE_DONE;
 }
 
 /*!
  * \brief Apply rate limit.
  */
-static int ratelimit_apply(int state, knot_pkt_t *pkt, knot_process_t *ctx)
+static int ratelimit_apply(int state, knot_pkt_t *pkt, knot_layer_t *ctx)
 {
 	/* Check if rate limiting applies. */
 	struct query_data *qdata = QUERY_DATA(ctx);
@@ -393,10 +426,11 @@ static int ratelimit_apply(int state, knot_pkt_t *pkt, knot_process_t *ctx)
 	}
 
 	/* Now it is slip or drop. */
-	if (rrl_slip_roll(conf()->rrl_slip)) {
+	conf_val_t val = conf_get(conf(), C_SRV, C_RATE_LIMIT_SLIP);
+	if (rrl_slip_roll(conf_int(&val))) {
 		/* Answer slips. */
-		if (process_query_err(pkt, ctx) != NS_PROC_DONE) {
-			return NS_PROC_FAIL;
+		if (process_query_err(ctx, pkt) != KNOT_EOK) {
+			return KNOT_STATE_FAIL;
 		}
 		knot_wire_set_tc(pkt->wire);
 	} else {
@@ -404,10 +438,10 @@ static int ratelimit_apply(int state, knot_pkt_t *pkt, knot_process_t *ctx)
 		pkt->size = 0;
 	}
 
-	return NS_PROC_DONE;
+	return KNOT_STATE_DONE;
 }
 
-static int process_query_out(knot_pkt_t *pkt, knot_process_t *ctx)
+static int process_query_out(knot_layer_t *ctx, knot_pkt_t *pkt)
 {
 	assert(pkt && ctx);
 	struct query_data *qdata = QUERY_DATA(ctx);
@@ -418,12 +452,12 @@ static int process_query_out(knot_pkt_t *pkt, knot_process_t *ctx)
 
 	/* Check parse state. */
 	knot_pkt_t *query = qdata->query;
-	int next_state = NS_PROC_DONE;
+	int next_state = KNOT_STATE_PRODUCE;
 	if (query->parsed < query->size) {
 		dbg_ns("%s: incompletely parsed query, FORMERR\n", __func__);
 		knot_pkt_clear(pkt);
 		qdata->rcode = KNOT_RCODE_FORMERR;
-		next_state = NS_PROC_FAIL;
+		next_state = KNOT_STATE_FAIL;
 		goto finish;
 	}
 
@@ -433,7 +467,7 @@ static int process_query_out(knot_pkt_t *pkt, knot_process_t *ctx)
 
 	int ret = prepare_answer(query, pkt, ctx);
 	if (ret != KNOT_EOK) {
-		next_state = NS_PROC_FAIL;
+		next_state = KNOT_STATE_FAIL;
 		goto finish;
 	}
 
@@ -445,26 +479,27 @@ static int process_query_out(knot_pkt_t *pkt, knot_process_t *ctx)
 	}
 
 	/* Answer based on qclass. */
-	switch (knot_pkt_qclass(pkt)) {
-	case KNOT_CLASS_CH:
-		next_state = query_chaos(pkt, ctx);
-		break;
-	case KNOT_CLASS_ANY:
-	case KNOT_CLASS_IN:
-		next_state = query_internet(pkt, ctx);
-		break;
-	default:
-		qdata->rcode = KNOT_RCODE_REFUSED;
-		next_state = NS_PROC_FAIL;
-		break;
+	if (next_state != KNOT_STATE_DONE) {
+		switch (knot_pkt_qclass(pkt)) {
+		case KNOT_CLASS_CH:
+			next_state = query_chaos(pkt, ctx);
+			break;
+		case KNOT_CLASS_ANY:
+		case KNOT_CLASS_IN:
+			next_state = query_internet(pkt, ctx);
+			break;
+		default:
+			qdata->rcode = KNOT_RCODE_REFUSED;
+			next_state = KNOT_STATE_FAIL;
+			break;
+		}
 	}
 
 	/*
 	 * Postprocessing.
 	 */
 
-
-	if (next_state == NS_PROC_DONE || next_state == NS_PROC_FULL) {
+	if (next_state == KNOT_STATE_DONE || next_state == KNOT_STATE_PRODUCE) {
 
 		/* Restore original QNAME. */
 		process_query_qname_case_restore(qdata, pkt);
@@ -476,23 +511,30 @@ static int process_query_out(knot_pkt_t *pkt, knot_process_t *ctx)
 		/* Put OPT RR to the additional section. */
 		ret = answer_edns_put(pkt, qdata);
 		if (ret != KNOT_EOK) {
-			next_state = NS_PROC_FAIL;
+			next_state = KNOT_STATE_FAIL;
 			goto finish;
 		}
 
 		/* Transaction security (if applicable). */
 		if (process_query_sign_response(pkt, qdata) != KNOT_EOK) {
-			next_state = NS_PROC_FAIL;
+			next_state = KNOT_STATE_FAIL;
 		}
 	}
 
 finish:
 	/* Default RCODE is SERVFAIL if not specified otherwise. */
-	if (next_state == NS_PROC_FAIL && qdata->rcode == KNOT_RCODE_NOERROR
-	    && qdata->rcode_ext == 0) {
+	if (next_state == KNOT_STATE_FAIL && qdata->rcode == KNOT_RCODE_NOERROR) {
 		qdata->rcode = KNOT_RCODE_SERVFAIL;
 	}
 
+	/* Store Extended RCODE - divide between header and OPT if possible. */
+	if (next_state != KNOT_STATE_FAIL) {
+		if (set_rcode_to_packet(pkt, qdata) != KNOT_EOK) {
+			next_state = KNOT_STATE_FAIL;
+		}
+	}
+	/* In case of NS_PROC_FAIL, RCODE is set in the error-processing function. */
+
 	/* Rate limits (if applicable). */
 	if (qdata->param->proc_flags & NS_QUERY_LIMIT_RATE) {
 		next_state = ratelimit_apply(next_state, pkt, ctx);
@@ -509,27 +551,27 @@ finish:
 	return next_state;
 }
 
-bool process_query_acl_check(list_t *acl, struct query_data *qdata)
+bool process_query_acl_check(const knot_dname_t *zone_name, acl_action_t action,
+                             struct query_data *qdata)
 {
 	knot_pkt_t *query = qdata->query;
 	const struct sockaddr_storage *query_source = qdata->param->remote;
-	const knot_dname_t *key_name = NULL;
-	knot_tsig_algorithm_t key_alg = KNOT_TSIG_ALG_NULL;
+	knot_tsig_key_t tsig = { 0 };
 
 	/* Skip if already checked and valid. */
-	if (qdata->sign.tsig_key != NULL) {
+	if (qdata->sign.tsig_key.name != NULL) {
 		return true;
 	}
 
 	/* Authenticate with NOKEY if the packet isn't signed. */
 	if (query->tsig_rr) {
-		key_name = query->tsig_rr->owner;
-		key_alg = tsig_rdata_alg(query->tsig_rr);
+		tsig.name = query->tsig_rr->owner;
+		tsig.algorithm = knot_tsig_rdata_alg(query->tsig_rr);
 	}
-	conf_iface_t *match = acl_find(acl, query_source, key_name);
 
-	/* Did not authenticate, no fitting rule found. */
-	if (match == NULL || (match->key && match->key->algorithm != key_alg)) {
+	/* Check if authenticated. */
+	conf_val_t acl = conf_zone_get(conf(), C_ACL, zone_name);
+	if (!acl_allowed(&acl, action, query_source, &tsig)) {
 		dbg_ns("%s: no ACL match => NOTAUTH\n", __func__);
 		qdata->rcode = KNOT_RCODE_NOTAUTH;
 		qdata->rcode_tsig = KNOT_TSIG_ERR_BADKEY;
@@ -537,7 +579,8 @@ bool process_query_acl_check(list_t *acl, struct query_data *qdata)
 	}
 
 	/* Remember used TSIG key. */
-	qdata->sign.tsig_key = match->key;
+	qdata->sign.tsig_key = tsig;
+
 	return true;
 }
 
@@ -553,13 +596,13 @@ int process_query_verify(struct query_data *qdata)
 
 	/* Keep digest for signing response. */
 	/*! \note This memory will be rewritten for multi-pkt answers. */
-	ctx->tsig_digest = (uint8_t *)tsig_rdata_mac(query->tsig_rr);
-	ctx->tsig_digestlen = tsig_rdata_mac_length(query->tsig_rr);
+	ctx->tsig_digest = (uint8_t *)knot_tsig_rdata_mac(query->tsig_rr);
+	ctx->tsig_digestlen = knot_tsig_rdata_mac_length(query->tsig_rr);
 
 	/* Checking query. */
 	process_query_qname_case_restore(qdata, query);
 	int ret = knot_tsig_server_check(query->tsig_rr, query->wire,
-	                                 query->size, ctx->tsig_key);
+	                                 query->size, &ctx->tsig_key);
 	process_query_qname_case_lower(query);
 
 	dbg_ns("%s: QUERY TSIG check result = %s\n", __func__, knot_strerror(ret));
@@ -580,7 +623,7 @@ int process_query_verify(struct query_data *qdata)
 	case KNOT_TSIG_EBADTIME:
 		qdata->rcode = KNOT_RCODE_NOTAUTH;
 		qdata->rcode_tsig = KNOT_TSIG_ERR_BADTIME;
-		ctx->tsig_time_signed = tsig_rdata_time_signed(query->tsig_rr);
+		ctx->tsig_time_signed = knot_tsig_rdata_time_signed(query->tsig_rr);
 		break;
 	case KNOT_EMALF:
 		qdata->rcode = KNOT_RCODE_FORMERR;
@@ -605,22 +648,22 @@ int process_query_sign_response(knot_pkt_t *pkt, struct query_data *qdata)
 	knot_sign_context_t *ctx = &qdata->sign;
 
 	/* KEY provided and verified TSIG or BADTIME allows signing. */
-	if (ctx->tsig_key != NULL && knot_tsig_can_sign(qdata->rcode_tsig)) {
+	if (ctx->tsig_key.name != NULL && knot_tsig_can_sign(qdata->rcode_tsig)) {
 
 		/* Sign query response. */
-		dbg_ns("%s: signing response using key %p\n", __func__, ctx->tsig_key);
-		size_t new_digest_len = knot_tsig_digest_length(ctx->tsig_key->algorithm);
+		dbg_ns("%s: signing response using key %p\n", __func__, &ctx->tsig_key);
+		size_t new_digest_len = dnssec_tsig_algorithm_size(ctx->tsig_key.algorithm);
 		if (ctx->pkt_count == 0) {
 			ret = knot_tsig_sign(pkt->wire, &pkt->size, pkt->max_size,
 			                     ctx->tsig_digest, ctx->tsig_digestlen,
 			                     ctx->tsig_digest, &new_digest_len,
-			                     ctx->tsig_key, qdata->rcode_tsig,
+			                     &ctx->tsig_key, qdata->rcode_tsig,
 			                     ctx->tsig_time_signed);
 		} else {
 			ret = knot_tsig_sign_next(pkt->wire, &pkt->size, pkt->max_size,
 			                          ctx->tsig_digest, ctx->tsig_digestlen,
 			                          ctx->tsig_digest, &new_digest_len,
-			                          ctx->tsig_key,
+			                          &ctx->tsig_key,
 			                          pkt->wire, pkt->size);
 		}
 		if (ret != KNOT_EOK) {
@@ -667,16 +710,15 @@ int process_query_qname_case_lower(knot_pkt_t *pkt)
 }
 
 /*! \brief Module implementation. */
-static const knot_process_module_t PROCESS_QUERY_MODULE = {
-	&process_query_begin,
-	&process_query_reset,
-	&process_query_finish,
-	&process_query_in,
-	&process_query_out,
-	&process_query_err
-};
-
-const knot_process_module_t *process_query_get_module(void)
+const knot_layer_api_t *process_query_layer(void)
 {
-	return &PROCESS_QUERY_MODULE;
+	static const knot_layer_api_t api = {
+		.begin   = &process_query_begin,
+		.reset   = &process_query_reset,
+		.finish  = &process_query_finish,
+		.consume = &process_query_in,
+		.produce = &process_query_out,
+		.fail    = &process_query_err
+	};
+	return &api;
 }
