@@ -26,29 +26,26 @@
 
 #pragma once
 
-#include "libknot/processing/process.h"
+#include "libknot/processing/layer.h"
 #include "knot/server/server.h"
 #include "knot/updates/acl.h"
 
 /* Query processing module implementation. */
-const knot_process_module_t *process_query_get_module(void);
-#define NS_PROC_QUERY process_query_get_module()
-#define NS_PROC_QUERY_ID 1
+const knot_layer_api_t *process_query_layer(void);
+#define NS_PROC_QUERY process_query_layer()
 
 /*! \brief Query processing logging common base. */
-#define NS_PROC_LOG(severity, log_type, remote, zone_str, what, msg, ...) do { \
+#define NS_PROC_LOG(severity, remote, zone_name, operation, msg, ...) do { \
 	char addr_str[SOCKADDR_STRLEN] = {0}; \
-	sockaddr_tostr(remote, addr_str, sizeof(addr_str)); \
-	log_msg(log_type, severity, what msg "\n", zone_str, addr_str, ##__VA_ARGS__); \
+	sockaddr_tostr(addr_str, sizeof(addr_str), remote); \
+	log_msg_zone(severity, zone_name, operation ", %s: " msg, \
+	             addr_str, ##__VA_ARGS__); \
 	} while (0)
 
 /*! \brief Query logging common base. */
-#define QUERY_LOG(severity, qdata, what, msg...) do {\
-	char *zone_str = knot_dname_to_str(knot_pkt_qname((qdata)->query)); \
-	NS_PROC_LOG(severity, LOG_SERVER, (qdata)->param->remote, zone_str, \
-	            what " of '%s' with '%s': ", msg); \
-	free(zone_str); \
-	} while(0)
+#define QUERY_LOG(severity, qdata, operation, msg, ...) \
+	NS_PROC_LOG(severity, (qdata)->param->remote, knot_pkt_qname((qdata)->query), \
+	            operation, msg, ##__VA_ARGS__);
 
 /* Query processing specific flags. */
 enum process_query_flag {
@@ -70,9 +67,8 @@ struct process_query_param {
 
 /*! \brief Query processing intermediate data. */
 struct query_data {
-	uint16_t rcode;       /*!< Resulting RCODE. */
+	uint16_t rcode;       /*!< Resulting RCODE (Whole extended RCODE). */
 	uint16_t rcode_tsig;  /*!< Resulting TSIG RCODE. */
-	uint8_t  rcode_ext;   /*!< Extended RCODE. */
 	uint16_t packet_type; /*!< Resolved packet type. */
 	knot_pkt_t *query;    /*!< Query to be solved. */
 	const zone_t *zone;   /*!< Zone from which is answered. */
@@ -86,6 +82,10 @@ struct query_data {
 	/* Original QNAME case. */
 	uint8_t orig_qname[KNOT_DNAME_MAXLEN];
 
+	/* EDNS */
+	knot_rrset_t opt_rr;
+	uint8_t *opt_rr_pos;  /*!< Place of the OPT RR in wire. */
+
 	/* Extensions. */
 	void *ext;
 	void (*ext_cleanup)(struct query_data*); /*!< Extensions cleanup callback. */
@@ -113,11 +113,13 @@ struct rrsig_info {
 /*!
  * \brief Check current query against ACL.
  *
- * \param acl
- * \param qdata
+ * \param zone_name  Current zone name.
+ * \param action     ACL action.
+ * \param qdata      Query data.
  * \return true if accepted, false if denied.
  */
-bool process_query_acl_check(list_t *acl, struct query_data *qdata);
+bool process_query_acl_check(const knot_dname_t *zone_name, acl_action_t action,
+                             struct query_data *qdata);
 
 /*!
  * \brief Verify current query transaction security and update query data.
@@ -131,4 +133,29 @@ bool process_query_acl_check(list_t *acl, struct query_data *qdata);
  */
 int process_query_verify(struct query_data *qdata);
 
+/*!
+ * \brief Sign current query using configured TSIG keys.
+ *
+ * \param pkt    Outgoing message.
+ * \param qdata  Query data.
+ *
+ * \retval KNOT_E*
+ */
+int process_query_sign_response(knot_pkt_t *pkt, struct query_data *qdata);
+
+/*!
+ * \brief Restore QNAME letter case.
+ *
+ * \param qdata  Query data.
+ * \param pkt    Incoming message.
+ */
+void process_query_qname_case_restore(struct query_data *qdata, knot_pkt_t *pkt);
+
+/*!
+ * \brief Convert QNAME to lowercase format for processing.
+ *
+ * \param pkt    Incoming message.
+ */
+int process_query_qname_case_lower(knot_pkt_t *pkt);
+
 /*! @} */
