@@ -1,31 +1,36 @@
 #include "knot/nameserver/query_module.h"
-#include "libknot/mempattern.h"
-#include "libknot/errcode.h"
-#include "common-knot/strlcpy.h"
+#include "libknot/libknot.h"
+#include "libknot/internal/strlcpy.h"
 
 /* Compiled-in module headers. */
 #include "knot/modules/synth_record.h"
+#include "knot/modules/dnsproxy.h"
+#ifdef HAVE_ROSEDB
+#include "knot/modules/rosedb.h"
+#endif
 #if USE_DNSTAP
 #include "knot/modules/dnstap.h"
 #endif
 
-/* Compiled-in module table. */
-struct compiled_module {
-	const char *name;
+typedef struct static_module {
+	const yp_name_t *name;
 	qmodule_load_t load;
 	qmodule_unload_t unload;
-};
+} static_module_t;
 
 /*! \note All modules should be dynamically loaded later on. */
-struct compiled_module MODULES[] = {
-        { "synth_record", &synth_record_load, &synth_record_unload },
-#if USE_DNSTAP
-        { "dnstap",       &dnstap_load,       &dnstap_unload }
+static_module_t MODULES[] = {
+        { C_MOD_SYNTH_RECORD, &synth_record_load, &synth_record_unload },
+        { C_MOD_DNSPROXY,     &dnsproxy_load,     &dnsproxy_unload },
+#ifdef HAVE_ROSEDB
+        { C_MOD_ROSEDB,       &rosedb_load,       &rosedb_unload },
 #endif
+#if USE_DNSTAP
+        { C_MOD_DNSTAP,       &dnstap_load,       &dnstap_unload },
+#endif
+        { NULL }
 };
 
-#define MODULE_COUNT sizeof(MODULES) / sizeof(MODULES[0])
-
 struct query_plan *query_plan_create(mm_ctx_t *mm)
 {
 	struct query_plan *plan = mm_alloc(mm, sizeof(struct query_plan));
@@ -57,7 +62,8 @@ void query_plan_free(struct query_plan *plan)
 	mm_free(plan->mm, plan);
 }
 
-static struct query_step *make_step(mm_ctx_t *mm, qmodule_process_t process, void *ctx)
+static struct query_step *make_step(mm_ctx_t *mm, qmodule_process_t process,
+                                    void *ctx)
 {
 	struct query_step *step = mm_alloc(mm, sizeof(struct query_step));
 	if (step == NULL) {
@@ -67,10 +73,12 @@ static struct query_step *make_step(mm_ctx_t *mm, qmodule_process_t process, voi
 	memset(step, 0, sizeof(struct query_step));
 	step->process = process;
 	step->ctx = ctx;
+
 	return step;
 }
 
-int query_plan_step(struct query_plan *plan, int stage, qmodule_process_t process, void *ctx)
+int query_plan_step(struct query_plan *plan, int stage, qmodule_process_t process,
+                    void *ctx)
 {
 	struct query_step *step = make_step(plan->mm, process, ctx);
 	if (step == NULL) {
@@ -78,43 +86,50 @@ int query_plan_step(struct query_plan *plan, int stage, qmodule_process_t proces
 	}
 
 	add_tail(&plan->stage[stage], &step->node);
+
 	return KNOT_EOK;
 }
 
-struct query_module *query_module_open(struct conf_t *config, const char *name,
-                                       const char *param, mm_ctx_t *mm)
+static static_module_t *find_module(const yp_name_t *name)
 {
-	/* Locate compiled-in modules. */
-	struct compiled_module *found = NULL;
-	for (unsigned i = 0; i < MODULE_COUNT; ++i) {
-		if (strcmp(MODULES[i].name, name) == 0) {
-			found = &MODULES[i];
+	/* Search for the module by name. */
+	static_module_t *module = NULL;
+	for (unsigned i = 0; MODULES[i].name != NULL; ++i) {
+		if (name[0] == MODULES[i].name[0] &&
+		    memcmp(name + 1, MODULES[i].name + 1, name[0]) == 0) {
+			module = &MODULES[i];
 			break;
 		}
 	}
 
-	/* Module not found. */
+	return module;
+}
+
+struct query_module *query_module_open(conf_t *config, conf_mod_id_t *mod_id,
+                                       mm_ctx_t *mm)
+{
+	if (config == NULL || mod_id == NULL) {
+		return NULL;
+	}
+
+	/* Locate the module. */
+	static_module_t *found = find_module(mod_id->name);
 	if (found == NULL) {
 		return NULL;
 	}
 
+	/* Create query module. */
 	struct query_module *module = mm_alloc(mm, sizeof(struct query_module));
 	if (module == NULL) {
 		return NULL;
 	}
-
-	size_t buflen = strlen(param) + 1;
 	memset(module, 0, sizeof(struct query_module));
+
 	module->mm = mm;
 	module->config = config;
+	module->id = mod_id;
 	module->load = found->load;
 	module->unload = found->unload;
-	module->param = mm_alloc(mm, buflen);
-	if (module->param == NULL) {
-		mm_free(mm, module);
-		return NULL;
-	}
-	strlcpy(module->param, param, buflen);
 
 	return module;
 }
@@ -125,7 +140,6 @@ void query_module_close(struct query_module *module)
 		return;
 	}
 
-	module->unload(module);
-	mm_free(module->mm, module->param);
+	conf_free_mod_id(module->id);
 	mm_free(module->mm, module);
 }
