@@ -18,7 +18,7 @@
 #include <stdint.h>
 #include <stdlib.h>
 
-#include "libknot/errcode.h"
+#include "libknot/libknot.h"
 #include "libknot/rrtype/tsig.h"
 #include "libknot/tsig-op.h"
 #include "knot/nameserver/tsig_ctx.h"
@@ -60,7 +60,7 @@ int tsig_sign_packet(tsig_ctx_t *ctx, knot_pkt_t *packet)
 
 	int ret = KNOT_ERROR;
 	if (ctx->digest_size == 0) {
-		ctx->digest_size = knot_tsig_digest_length(ctx->key->algorithm);
+		ctx->digest_size = dnssec_tsig_algorithm_size(ctx->key->algorithm);
 		ret = knot_tsig_sign(packet->wire, &packet->size, packet->max_size,
 		                     NULL, 0,
 		                     ctx->digest, &ctx->digest_size,
@@ -83,12 +83,12 @@ static int update_ctx_after_verify(tsig_ctx_t *ctx, knot_rrset_t *tsig_rr)
 	assert(ctx);
 	assert(tsig_rr);
 
-	if (ctx->digest_size != tsig_rdata_mac_length(tsig_rr)) {
+	if (ctx->digest_size != knot_tsig_rdata_mac_length(tsig_rr)) {
 		return KNOT_EMALF;
 	}
 
-	memcpy(ctx->digest, tsig_rdata_mac(tsig_rr), ctx->digest_size);
-	ctx->prev_signed_time = tsig_rdata_time_signed(tsig_rr);
+	memcpy(ctx->digest, knot_tsig_rdata_mac(tsig_rr), ctx->digest_size);
+	ctx->prev_signed_time = knot_tsig_rdata_time_signed(tsig_rr);
 	ctx->unsigned_count = 0;
 	ctx->buffer_used = 0;
 
