@@ -1,4 +1,4 @@
-/*  Copyright (C) 2013 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -12,34 +12,38 @@
 
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+*/
 
+#include <urcu.h>
+
+#include "dnssec/random.h"
 #include "knot/nameserver/update.h"
 #include "knot/nameserver/internet.h"
 #include "knot/nameserver/process_query.h"
 #include "knot/updates/apply.h"
 #include "knot/dnssec/zone-sign.h"
-#include "common/debug.h"
+#include "knot/common/debug.h"
+#include "libknot/internal/macros.h"
 #include "knot/dnssec/zone-events.h"
 #include "knot/updates/ddns.h"
 #include "knot/updates/zone-update.h"
+#include "libknot/libknot.h"
 #include "libknot/descriptor.h"
 #include "libknot/tsig-op.h"
 #include "knot/zone/zone.h"
 #include "knot/zone/events/events.h"
 #include "knot/server/tcp-handler.h"
 #include "knot/server/udp-handler.h"
-#include "knot/nameserver/requestor.h"
 #include "knot/nameserver/capture.h"
-#include "libknot/dnssec/random.h"
+#include "libknot/processing/requestor.h"
 
 /* UPDATE-specific logging (internal, expects 'qdata' variable set). */
-#define UPDATE_LOG(severity, msg...) \
-	QUERY_LOG(severity, qdata, "DDNS", msg)
+#define UPDATE_LOG(severity, msg, ...) \
+	QUERY_LOG(severity, qdata, "DDNS", msg, ##__VA_ARGS__)
 
 static void init_qdata_from_request(struct query_data *qdata,
                                     const zone_t *zone,
-                                    struct request_data *req,
+                                    struct knot_request *req,
                                     struct process_query_param *param)
 {
 	memset(qdata, 0, sizeof(*qdata));
@@ -76,13 +80,12 @@ static int sign_update(zone_t *zone, const zone_contents_t *old_contents,
 	uint32_t refresh_at = 0;
 	if (apex_rr_changed(old_contents, new_contents, KNOT_RRTYPE_DNSKEY) ||
 	    apex_rr_changed(old_contents, new_contents, KNOT_RRTYPE_NSEC3PARAM)) {
-		ret = knot_dnssec_zone_sign(new_contents, zone->conf,
-		                            sec_ch, KNOT_SOA_SERIAL_KEEP,
+		ret = knot_dnssec_zone_sign(new_contents, sec_ch,
+		                            ZONE_SIGN_KEEP_SOA_SERIAL,
 		                            &refresh_at);
 	} else {
 		// Sign the created changeset
-		ret = knot_dnssec_sign_changeset(new_contents, zone->conf,
-		                                 ddns_ch, sec_ch,
+		ret = knot_dnssec_sign_changeset(new_contents, ddns_ch, sec_ch,
 		                                 &refresh_at);
 	}
 	if (ret != KNOT_EOK) {
@@ -104,14 +107,14 @@ static int sign_update(zone_t *zone, const zone_contents_t *old_contents,
 
 	// Plan next zone resign.
 	const time_t resign_time = zone_events_get_time(zone, ZONE_EVENT_DNSSEC);
-	if (time(NULL) + refresh_at < resign_time) {
-		zone_events_schedule(zone, ZONE_EVENT_DNSSEC, refresh_at);
+	if (refresh_at < resign_time) {
+		zone_events_schedule_at(zone, ZONE_EVENT_DNSSEC, refresh_at);
 	}
 
 	return KNOT_EOK;
 }
 
-static int check_prereqs(struct request_data *request,
+static int check_prereqs(struct knot_request *request,
                          const zone_t *zone, zone_update_t *update,
                          struct query_data *qdata)
 {
@@ -128,7 +131,7 @@ static int check_prereqs(struct request_data *request,
 	return KNOT_EOK;
 }
 
-static int process_single_update(struct request_data *request,
+static int process_single_update(struct knot_request *request,
                                  const zone_t *zone, zone_update_t *update,
                                  struct query_data *qdata)
 {
@@ -147,7 +150,7 @@ static int process_single_update(struct request_data *request,
 
 static void set_rcodes(list_t *requests, const uint16_t rcode)
 {
-	struct request_data *req;
+	struct knot_request *req;
 	WALK_LIST(req, *requests) {
 		if (knot_wire_get_rcode(req->resp->wire) == KNOT_RCODE_NOERROR) {
 			knot_wire_set_rcode(req->resp->wire, rcode);
@@ -167,7 +170,7 @@ static int process_bulk(zone_t *zone, list_t *requests, changeset_t *ddns_ch)
 	zone_update_init(&zone_update, zone->contents, ddns_ch);
 
 	// Walk all the requests and process.
-	struct request_data *req;
+	struct knot_request *req;
 	WALK_LIST(req, *requests) {
 		// Init qdata structure for logging (unique per-request).
 		struct process_query_param param = { 0 };
@@ -235,9 +238,12 @@ static int process_normal(zone_t *zone, list_t *requests)
 	}
 	assert(new_contents);
 
+	conf_val_t val = conf_zone_get(conf(), C_DNSSEC_SIGNING, zone->name);
+	bool dnssec_enable = conf_bool(&val);
+
 	// Sign the update.
 	changeset_t sec_ch;
-	if (zone->conf->dnssec_enable) {
+	if (dnssec_enable) {
 		ret = changeset_init(&sec_ch, zone->name);
 		if (ret != KNOT_EOK) {
 			set_rcodes(requests, KNOT_RCODE_SERVFAIL);
@@ -260,19 +266,24 @@ static int process_normal(zone_t *zone, list_t *requests)
 		update_rollback(&ddns_ch);
 		update_free_zone(&new_contents);
 		changeset_clear(&ddns_ch);
-		if (zone->conf->dnssec_enable) {
+		if (dnssec_enable) {
 			changeset_clear(&sec_ch);
 		}
 		set_rcodes(requests, KNOT_RCODE_SERVFAIL);
 		return ret;
 	}
 
+	/* Temporarily unlock locked configuration. */
+	rcu_read_unlock();
+
 	// Switch zone contents.
 	zone_contents_t *old_contents = zone_switch_contents(zone, new_contents);
 	synchronize_rcu();
 
+	rcu_read_lock();
+
 	// Clear DNSSEC changes
-	if (zone->conf->dnssec_enable) {
+	if (dnssec_enable) {
 		update_cleanup(&sec_ch);
 		changeset_clear(&sec_ch);
 	}
@@ -284,14 +295,14 @@ static int process_normal(zone_t *zone, list_t *requests)
 	changeset_clear(&ddns_ch);
 
 	/* Sync zonefile immediately if configured. */
-	if (zone->conf->dbsync_timeout == 0) {
+	val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+	if (conf_int(&val) == 0) {
 		zone_events_schedule(zone, ZONE_EVENT_FLUSH, ZONE_EVENT_NOW);
 	}
 
 	return ret;
 }
 
-
 static int process_requests(zone_t *zone, list_t *requests)
 {
 	if (zone == NULL || requests == NULL) {
@@ -328,15 +339,8 @@ static int process_requests(zone_t *zone, list_t *requests)
 	return KNOT_EOK;
 }
 
-static int forward_request(zone_t *zone, struct request_data *request)
+static int forward_request(zone_t *zone, struct knot_request *request)
 {
-	/* Create requestor instance. */
-	struct requestor re;
-	requestor_init(&re, NS_PROC_CAPTURE, NULL);
-
-	/* Fetch primary master. */
-	const conf_iface_t *master = zone_master(zone);
-
 	/* Copy request and assign new ID. */
 	knot_pkt_t *query = knot_pkt_new(NULL, request->query->max_size, NULL);
 	int ret = knot_pkt_copy(query, request->query);
@@ -345,26 +349,57 @@ static int forward_request(zone_t *zone, struct request_data *request)
 		knot_wire_set_rcode(request->resp->wire, KNOT_RCODE_SERVFAIL);
 		return ret;
 	}
-	knot_wire_set_id(query->wire, knot_random_uint16_t());
+	knot_wire_set_id(query->wire, dnssec_random_uint16_t());
 	knot_tsig_append(query->wire, &query->size, query->max_size, query->tsig_rr);
 
-	/* Create a request. */
-	struct request *req = requestor_make(&re, master, query);
-	if (req == NULL) {
-		knot_pkt_free(&query);
-		return KNOT_ENOMEM;
+	/* Read the ddns master or the first master. */
+	conf_val_t remote = conf_zone_get(conf(), C_DDNS_MASTER, zone->name);
+	if (remote.code != KNOT_EOK) {
+		remote = conf_zone_get(conf(), C_MASTER, zone->name);
 	}
 
-	/* Enqueue and execute request. */
-	struct process_capture_param param;
-	param.sink = request->resp;
-	ret = requestor_enqueue(&re, req, &param);
-	if (ret == KNOT_EOK) {
-		struct timeval tv = { conf()->max_conn_reply, 0 };
-		ret = requestor_exec(&re, &tv);
-	}
+	/* Get the number of remote addresses. */
+	conf_val_t addr = conf_id_get(conf(), C_RMT, C_ADDR, &remote);
+	size_t addr_count = conf_val_count(&addr);
 
-	requestor_clear(&re);
+	/* Try all remote addresses to forward the request to. */
+	for (size_t i = 0; i < addr_count; i++) {
+		conf_remote_t master = conf_remote(conf(), &remote, i);
+
+		/* Create requestor instance. */
+		struct knot_requestor re;
+		knot_requestor_init(&re, NULL);
+
+		/* Prepare packet capture layer. */
+		struct capture_param param;
+		param.sink = request->resp;
+		knot_requestor_overlay(&re, LAYER_CAPTURE, &param);
+
+		/* Create a request. */
+		const struct sockaddr *dst = (const struct sockaddr *)&master.addr;
+		const struct sockaddr *src = (const struct sockaddr *)&master.via;
+		struct knot_request *req = knot_request_make(re.mm, dst, src, query, 0);
+		if (req == NULL) {
+			knot_pkt_free(&query);
+			return KNOT_ENOMEM;
+		}
+
+		/* Enqueue the request. */
+		ret = knot_requestor_enqueue(&re, req);
+		if (ret != KNOT_EOK) {
+			knot_requestor_clear(&re);
+			continue;
+		}
+
+		/* Execute the request. */
+		conf_val_t val = conf_get(conf(), C_SRV, C_TCP_REPLY_TIMEOUT);
+		struct timeval tv = { conf_int(&val), 0 };
+		ret = knot_requestor_exec(&re, &tv);
+		knot_requestor_clear(&re);
+		if (ret == KNOT_EOK) {
+			break;
+		}
+	}
 
 	/* Restore message ID and TSIG. */
 	knot_wire_set_id(request->resp->wire, knot_wire_get_id(request->query->wire));
@@ -385,16 +420,16 @@ static int forward_request(zone_t *zone, struct request_data *request)
 
 static void forward_requests(zone_t *zone, list_t *requests)
 {
-	struct request_data *req;
+	struct knot_request *req;
 	WALK_LIST(req, *requests) {
 		forward_request(zone, req);
 	}
 }
 
-static bool update_tsig_check(struct query_data *qdata, struct request_data *req)
+static bool update_tsig_check(struct query_data *qdata, struct knot_request *req)
 {
 	// Check that ACL is still valid.
-	if (!process_query_acl_check(&qdata->zone->conf->acl.update_in, qdata)) {
+	if (!process_query_acl_check(qdata->zone->name, ACL_ACTION_UPDATE, qdata)) {
 		UPDATE_LOG(LOG_WARNING, "ACL check failed");
 		knot_wire_set_rcode(req->resp->wire, qdata->rcode);
 		return false;
@@ -417,18 +452,10 @@ static bool update_tsig_check(struct query_data *qdata, struct request_data *req
 
 #undef UPDATE_LOG
 
-static void free_request(struct request_data *req)
-{
-	close(req->fd);
-	knot_pkt_free(&req->query);
-	knot_pkt_free(&req->resp);
-	free(req);
-}
-
-static void send_update_response(const zone_t *zone, struct request_data *req)
+static void send_update_response(const zone_t *zone, struct knot_request *req)
 {
 	if (req->resp) {
-		if (!zone_master(zone)) {
+		if (!zone_is_slave(zone)) {
 			// Sign the response with TSIG where applicable
 			struct query_data qdata;
 			init_qdata_from_request(&qdata, zone, req, NULL);
@@ -437,7 +464,10 @@ static void send_update_response(const zone_t *zone, struct request_data *req)
 		}
 
 		if (net_is_connected(req->fd)) {
-			tcp_send_msg(req->fd, req->resp->wire, req->resp->size);
+			conf_val_t val = conf_get(conf(), C_SRV, C_TCP_REPLY_TIMEOUT);
+			struct timeval timeout = { conf_int(&val), 0 };
+			tcp_send_msg(req->fd, req->resp->wire, req->resp->size,
+			             &timeout);
 		} else {
 			udp_send_msg(req->fd, req->resp->wire, req->resp->size,
 			             (struct sockaddr *)&req->remote);
@@ -445,9 +475,17 @@ static void send_update_response(const zone_t *zone, struct request_data *req)
 	}
 }
 
+static void free_request(struct knot_request *req)
+{
+	close(req->fd);
+	knot_pkt_free(&req->query);
+	knot_pkt_free(&req->resp);
+	free(req);
+}
+
 static void send_update_responses(const zone_t *zone, list_t *updates)
 {
-	struct request_data *req;
+	struct knot_request *req;
 	node_t *nxt = NULL;
 	WALK_LIST_DELSAFE(req, nxt, *updates) {
 		send_update_response(zone, req);
@@ -459,7 +497,7 @@ static void send_update_responses(const zone_t *zone, list_t *updates)
 static int init_update_responses(const zone_t *zone, list_t *updates,
                                  size_t *update_count)
 {
-	struct request_data *req = NULL;
+	struct knot_request *req = NULL;
 	node_t *nxt = NULL;
 	WALK_LIST_DELSAFE(req, nxt, *updates) {
 		req->resp = knot_pkt_new(NULL, KNOT_WIRE_MAX_PKTSIZE, NULL);
@@ -469,7 +507,7 @@ static int init_update_responses(const zone_t *zone, list_t *updates,
 
 		assert(req->query);
 		knot_pkt_init_response(req->resp, req->query);
-		if (zone_master(zone)) {
+		if (zone_is_slave(zone)) {
 			// Don't check TSIG for forwards.
 			continue;
 		}
@@ -483,7 +521,6 @@ static int init_update_responses(const zone_t *zone, list_t *updates,
 			// ACL/TSIG check failed, send response.
 			send_update_response(zone, req);
 			// Remove this request from processing list.
-			rem_node(&req->node);
 			free_request(req);
 			*update_count -= 1;
 		}
@@ -502,7 +539,7 @@ int update_query_process(knot_pkt_t *pkt, struct query_data *qdata)
 
 	/* Need valid transaction security. */
 	zone_t *zone = (zone_t *)qdata->zone;
-	NS_NEED_AUTH(&zone->conf->acl.update_in, qdata);
+	NS_NEED_AUTH(qdata, zone->name, ACL_ACTION_UPDATE);
 	/* Check expiration. */
 	NS_NEED_ZONE_CONTENTS(qdata, KNOT_RCODE_SERVFAIL);
 
@@ -511,12 +548,12 @@ int update_query_process(knot_pkt_t *pkt, struct query_data *qdata)
 	/* Store update into DDNS queue. */
 	int ret = zone_update_enqueue(zone, qdata->query, qdata->param);
 	if (ret != KNOT_EOK) {
-		return NS_PROC_FAIL;
+		return KNOT_STATE_FAIL;
 	}
 
 	/* No immediate response. */
 	pkt->size = 0;
-	return NS_PROC_DONE;
+	return KNOT_STATE_DONE;
 }
 
 int updates_execute(zone_t *zone)
@@ -528,22 +565,27 @@ int updates_execute(zone_t *zone)
 		return KNOT_EOK;
 	}
 
+	/* Block config changes. */
+	rcu_read_lock();
+
 	/* Init updates respones. */
 	int ret = init_update_responses(zone, &updates, &update_count);
 	if (ret != KNOT_EOK) {
 		/* Send what responses we can. */
 		set_rcodes(&updates, KNOT_RCODE_SERVFAIL);
 		send_update_responses(zone, &updates);
+		rcu_read_unlock();
 		return ret;
 	}
 
 	if (update_count == 0) {
 		/* All updates failed their ACL checks. */
+		rcu_read_unlock();
 		return KNOT_EOK;
 	}
 
 	/* Process update list - forward if zone has master, or execute. */
-	if (zone_master(zone)) {
+	if (zone_is_slave(zone)) {
 		log_zone_info(zone->name,
 		              "DDNS, forwarding %zu updates", update_count);
 		forward_requests(zone, &updates);
@@ -557,6 +599,6 @@ int updates_execute(zone_t *zone)
 	/* Send responses. */
 	send_update_responses(zone, &updates);
 
+	rcu_read_unlock();
 	return KNOT_EOK;
 }
-
