@@ -29,9 +29,10 @@
 #include <pthread_np.h>
 #endif /* HAVE_PTHREAD_NP_H */
 
-#include "knot/knot.h"
+#include "knot/common/debug.h"
+#include "knot/common/log.h"
 #include "knot/server/dthreads.h"
-#include "common/log.h"
+#include "libknot/libknot.h"
 
 /* BSD cpu set compatibility. */
 #if defined(HAVE_CPUSET_BSD)
@@ -124,17 +125,13 @@ static void *thread_ep(void *data)
 		return 0;
 	}
 
-	// Ignore specific signals (except SIGALRM)
-	sigset_t ignset;
-	sigemptyset(&ignset);
-	sigaddset(&ignset, SIGINT);
-	sigaddset(&ignset, SIGTERM);
-	sigaddset(&ignset, SIGHUP);
-	sigaddset(&ignset, SIGPIPE);
-	sigaddset(&ignset, SIGUSR1);
-	pthread_sigmask(SIG_BLOCK, &ignset, 0);
-	rcu_register_thread();
+	// Unblock SIGALRM
+	sigset_t mask;
+	(void)sigemptyset(&mask);
+	sigaddset(&mask, SIGALRM);
+	pthread_sigmask(SIG_UNBLOCK, &mask, NULL);
 
+	rcu_register_thread();
 	dbg_dt("dthreads: [%p] entered ep\n", thread);
 
 	/* Drop capabilities except FS access. */
@@ -633,7 +630,6 @@ int dt_stop(dt_unit_t *unit)
 	return KNOT_EOK;
 }
 
-
 int dt_setaffinity(dthread_t *thread, unsigned* cpu_id, size_t cpu_count)
 {
 	if (thread == NULL) {
@@ -809,7 +805,7 @@ int dt_unit_lock(dt_unit_t *unit)
 
 	/* Map errors. */
 	if (ret < 0) {
-		return knot_map_errno(EINVAL, EAGAIN);
+		return knot_map_errno();
 	}
 
 	return KNOT_EOK;
@@ -826,7 +822,7 @@ int dt_unit_unlock(dt_unit_t *unit)
 
 	/* Map errors. */
 	if (ret < 0) {
-		return knot_map_errno(EINVAL, EAGAIN);
+		return knot_map_errno();
 	}
 
 	return KNOT_EOK;
