@@ -20,15 +20,12 @@
 #include <sys/socket.h>
 #include <assert.h>
 
+#include "dnssec/random.h"
+#include "knot/common/debug.h"
 #include "knot/server/rrl.h"
-#include "knot/knot.h"
-#include "libknot/consts.h"
-#include "libknot/packet/wire.h"
-#include "common-knot/hattrie/murmurhash3.h"
-#include "libknot/dnssec/random.h"
-#include "libknot/descriptor.h"
-#include "common/errors.h"
 #include "knot/zone/zone.h"
+#include "libknot/libknot.h"
+#include "libknot/internal/trie/murmurhash3.h"
 
 /* Hopscotch defines. */
 #define HOP_LEN (sizeof(unsigned)*8)
@@ -59,7 +56,12 @@ enum {
 };
 
 /* Classification string. */
-const error_table_t rrl_clsstr_tbl[] = {
+struct cls_name {
+	int code;
+	const char *name;
+};
+
+static const struct cls_name rrl_cls_names[] = {
         {CLS_NORMAL,  "POSITIVE" },
         {CLS_ERROR,   "ERROR" },
         {CLS_NXDOMAIN,"NXDOMAIN"},
@@ -71,9 +73,16 @@ const error_table_t rrl_clsstr_tbl[] = {
         {CLS_NULL,    "NULL"},
         {CLS_NULL,    NULL}
 };
+
 static inline const char *rrl_clsstr(int code)
 {
-	return error_to_str(rrl_clsstr_tbl, code);
+	for (const struct cls_name *c = rrl_cls_names; c->name; c++) {
+		if (c->code == code) {
+			return c->name;
+		}
+	}
+
+	return "unknown class";
 }
 
 /* Bucket flags. */
@@ -279,7 +288,7 @@ static void rrl_log_state(const struct sockaddr_storage *ss, uint16_t flags, uin
 {
 #ifdef RRL_ENABLE_LOG
 	char addr_str[SOCKADDR_STRLEN] = {0};
-	sockaddr_tostr(ss, addr_str, sizeof(addr_str));
+	sockaddr_tostr(addr_str, sizeof(addr_str), ss);
 
 	const char *what = "leaves";
 	if (flags & RRL_BF_ELIMIT) {
@@ -491,7 +500,7 @@ bool rrl_slip_roll(int n_slip)
 	 * That represents a chance of 1/N that answer slips.
 	 * Therefore, on average, from 100 answers 100/N will slip. */
 	int threshold = RRL_SLIP_MAX / n_slip;
-	int roll = knot_random_uint16_t() % RRL_SLIP_MAX;
+	int roll = dnssec_random_uint16_t() % RRL_SLIP_MAX;
 	return (roll < threshold);
 }
 
@@ -521,7 +530,7 @@ int rrl_reseed(rrl_table_t *rrl)
 	}
 
 	memset(rrl->arr, 0, rrl->size * sizeof(rrl_item_t));
-	rrl->seed = knot_random_uint32_t();
+	rrl->seed = dnssec_random_uint32_t();
 	dbg_rrl("%s: reseed to '%u'\n", __func__, rrl->seed);
 
 	if (rrl->lk_count > 0) {
