@@ -29,26 +29,26 @@
 
 #pragma once
 
-#include "common-knot/evsched.h"
-#include "common-knot/lists.h"
-#include "common-knot/fdset.h"
-#include "common/namedb/namedb.h"
+#include "knot/common/evsched.h"
+#include "libknot/internal/lists.h"
+#include "knot/common/fdset.h"
+#include "libknot/internal/net.h"
+#include "libknot/internal/namedb/namedb.h"
 #include "knot/server/dthreads.h"
-#include "knot/server/net.h"
 #include "knot/server/rrl.h"
 #include "knot/worker/pool.h"
 #include "knot/zone/zonedb.h"
 
 /* Forwad declarations. */
-struct iface_t;
-struct server_t;
-struct conf_t;
+struct iface;
+struct server;
+struct conf;
 
 /*! \brief I/O handler structure.
   */
 typedef struct iohandler {
 	struct node        n;
-	struct server_t    *server; /*!< Reference to server */
+	struct server      *server; /*!< Reference to server */
 	dt_unit_t          *unit;   /*!< Threading unit */
 	unsigned           *thread_state; /*!< Thread state */
 	unsigned           *thread_id; /*!< Thread identifier. */
@@ -65,7 +65,7 @@ typedef enum {
 /*!
  * \brief Server interface structure.
  */
-typedef struct iface_t {
+typedef struct iface {
 	struct node n;
 	int fd[2];
 	struct sockaddr_storage addr;
@@ -89,14 +89,14 @@ typedef struct ifacelist {
  *
  * Keeps references to all important structures needed for operation.
  */
-typedef struct server_t {
+typedef struct server {
 
 	/*! \brief Server state tracking. */
 	volatile unsigned state;
 
 	/*! \brief Zone database. */
 	knot_zonedb_t *zone_db;
-	knot_namedb_t *timers_db;
+	namedb_t *timers_db;
 
 	/*! \brief I/O handlers. */
 	unsigned tu_size;
@@ -178,7 +178,7 @@ void server_stop(server_t *server);
  * \retval KNOT_EINVAL on invalid parameters.
  * \retval KNOT_ERROR unspecified error.
  */
-int server_reconfigure(const struct conf_t *conf, void *data);
+int server_reconfigure(conf_t *conf, void *data);
 
 /*!
  * \brief Reconfigure zone database.
@@ -187,7 +187,7 @@ int server_reconfigure(const struct conf_t *conf, void *data);
  *
  * \return KNOT_EOK on success or KNOT_ error
  */
-int server_update_zones(const struct conf_t *conf, void *data);
+int server_update_zones(conf_t *conf, void *data);
 
 /*!
  * \brief Update fdsets from current interfaces list.
