@@ -24,6 +24,7 @@
 #include <netinet/in.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <urcu.h>
 #ifdef HAVE_SYS_UIO_H			// struct iovec (OpenBSD)
 #include <sys/uio.h>
 #endif // HAVE_SYS_UIO_H
@@ -31,23 +32,26 @@
 #include <cap-ng.h>
 #endif /* HAVE_CAP_NG_H */
 
-#include "common-knot/sockaddr.h"
-#include "common-knot/fdset.h"
-#include "common/mempool.h"
-#include "knot/knot.h"
+#include "dnssec/random.h"
 #include "knot/server/tcp-handler.h"
-#include "libknot/packet/wire.h"
+#include "knot/common/debug.h"
+#include "knot/common/fdset.h"
+#include "knot/common/time.h"
 #include "knot/nameserver/process_query.h"
-#include "libknot/dnssec/crypto.h"
-#include "libknot/dnssec/random.h"
+#include "libknot/internal/mempool.h"
+#include "libknot/internal/macros.h"
+#include "libknot/internal/net.h"
+#include "libknot/internal/sockaddr.h"
+#include "libknot/processing/overlay.h"
 
 /*! \brief TCP context data. */
 typedef struct tcp_context {
-	knot_process_t query_ctx;   /*!< Query processing context. */
+	struct knot_overlay overlay;/*!< Query processing overlay. */
 	server_t *server;           /*!< Name server structure. */
 	struct iovec iov[2];        /*!< TX/RX buffers. */
 	unsigned client_threshold;  /*!< Index of first TCP client. */
 	timev_t last_poll_time;     /*!< Time of the last socket poll. */
+	timev_t throttle_end;       /*!< End of accept() throttling. */
 	fdset_t set;                /*!< Set of server/client sockets. */
 	unsigned thread_id;         /*!< Thread identifier. */
 } tcp_context_t;
@@ -55,12 +59,12 @@ typedef struct tcp_context {
 /*
  * Forward decls.
  */
-#define TCP_THROTTLE_LO 5 /*!< Minimum recovery time on errors. */
-#define TCP_THROTTLE_HI 50 /*!< Maximum recovery time on errors. */
+#define TCP_THROTTLE_LO 0 /*!< Minimum recovery time on errors. */
+#define TCP_THROTTLE_HI 2 /*!< Maximum recovery time on errors. */
 
 /*! \brief Calculate TCP throttle time (random). */
 static inline int tcp_throttle() {
-	return TCP_THROTTLE_LO + (knot_random_uint16_t() % TCP_THROTTLE_HI);
+	return TCP_THROTTLE_LO + (dnssec_random_uint16_t() % TCP_THROTTLE_HI);
 }
 
 /*! \brief Sweep TCP connection. */
@@ -68,23 +72,19 @@ static enum fdset_sweep_state tcp_sweep(fdset_t *set, int i, void *data)
 {
 	UNUSED(data);
 	assert(set && i < set->n && i >= 0);
-
 	int fd = set->pfd[i].fd;
 
+	/* Best-effort, name and shame. */
 	struct sockaddr_storage ss;
 	socklen_t len = sizeof(struct sockaddr_storage);
-	memset(&ss, 0, len);
-	if (getpeername(fd, (struct sockaddr*)&ss, &len) < 0) {
-		dbg_net("tcp: sweep getpeername() on invalid socket=%d\n", fd);
-		return FDSET_SWEEP;
+	if (getpeername(fd, (struct sockaddr*)&ss, &len) == 0) {
+		char addr_str[SOCKADDR_STRLEN] = {0};
+		sockaddr_tostr(addr_str, sizeof(addr_str), &ss);
+		log_notice("TCP, terminated inactive client, address '%s'", addr_str);
 	}
 
-	/* Translate */
-	char addr_str[SOCKADDR_STRLEN] = {0};
-	sockaddr_tostr(&ss, addr_str, sizeof(addr_str));
-
-	log_notice("connection terminated due to inactivity, address '%s'", addr_str);
 	close(fd);
+
 	return FDSET_SWEEP;
 }
 
@@ -112,41 +112,50 @@ static int tcp_handle(tcp_context_t *tcp, int fd,
 	}
 
 	/* Timeout. */
-	struct timeval tmout = { conf()->max_conn_reply, 0 };
+	rcu_read_lock();
+	conf_val_t val = conf_get(conf(), C_SRV, C_TCP_REPLY_TIMEOUT);
+	int64_t max_conn_reply = conf_int(&val);
+	rcu_read_unlock();
+	struct timeval tmout = { max_conn_reply, 0 };
 
 	/* Receive data. */
-	int ret = tcp_recv_msg(fd, rx->iov_base, rx->iov_len, &tmout);
+	struct timeval recv_tmout = tmout;
+	int ret = tcp_recv_msg(fd, rx->iov_base, rx->iov_len, &recv_tmout);
 	if (ret <= 0) {
 		dbg_net("tcp: client on fd=%d disconnected\n", fd);
 		if (ret == KNOT_EAGAIN) {
-			rcu_read_lock();
 			char addr_str[SOCKADDR_STRLEN] = {0};
-			sockaddr_tostr(&ss, addr_str, sizeof(addr_str));
-			log_warning("connection timed out, address '%s', "
-			            "timeout %d seconds",
-			            addr_str, conf()->max_conn_idle);
-			rcu_read_unlock();
+			sockaddr_tostr(addr_str, sizeof(addr_str), &ss);
+			log_warning("TCP, connection timed out, address '%s'",
+			            addr_str);
 		}
 		return KNOT_ECONNREFUSED;
 	} else {
 		rx->iov_len = ret;
 	}
 
-	/* Create query processing context. */
-	knot_process_begin(&tcp->query_ctx, &param, NS_PROC_QUERY);
+	/* Create packets. */
+	mm_ctx_t *mm = tcp->overlay.mm;
+	knot_pkt_t *ans = knot_pkt_new(tx->iov_base, tx->iov_len, mm);
+	knot_pkt_t *query = knot_pkt_new(rx->iov_base, rx->iov_len, mm);
+
+	/* Initialize processing overlay. */
+	knot_overlay_init(&tcp->overlay, mm);
+	knot_overlay_add(&tcp->overlay, NS_PROC_QUERY, &param);
 
 	/* Input packet. */
-	int state = knot_process_in(rx->iov_base, rx->iov_len, &tcp->query_ctx);
+	(void) knot_pkt_parse(query, 0);
+	int state = knot_overlay_consume(&tcp->overlay, query);
 
 	/* Resolve until NOOP or finished. */
 	ret = KNOT_EOK;
-	while (state & (NS_PROC_FULL|NS_PROC_FAIL)) {
-		uint16_t tx_len = tx->iov_len;
-		state = knot_process_out(tx->iov_base, &tx_len, &tcp->query_ctx);
+	while (state & (KNOT_STATE_PRODUCE|KNOT_STATE_FAIL)) {
+		state = knot_overlay_produce(&tcp->overlay, ans);
 
-		/* If it has response, send it. */
-		if (tx_len > 0) {
-			if (tcp_send_msg(fd, tx->iov_base, tx_len) != tx_len) {
+		/* Send, if response generation passed and wasn't ignored. */
+		if (ans->size > 0 && !(state & (KNOT_STATE_FAIL|KNOT_STATE_NOOP))) {
+			struct timeval send_tmout = tmout;
+			if (tcp_send_msg(fd, ans->wire, ans->size, &send_tmout) != ans->size) {
 				ret = KNOT_ECONNREFUSED;
 				break;
 			}
@@ -154,7 +163,12 @@ static int tcp_handle(tcp_context_t *tcp, int fd,
 	}
 
 	/* Reset after processing. */
-	knot_process_finish(&tcp->query_ctx);
+	knot_overlay_finish(&tcp->overlay);
+	knot_overlay_deinit(&tcp->overlay);
+
+	/* Cleanup. */
+	knot_pkt_free(&query);
+	knot_pkt_free(&ans);
 
 	return ret;
 }
@@ -168,29 +182,22 @@ int tcp_accept(int fd)
 	if (incoming < 0) {
 		int en = errno;
 		if (en != EINTR && en != EAGAIN) {
-			log_error("cannot accept connection (%d)", errno);
-			if (en == EMFILE || en == ENFILE ||
-			    en == ENOBUFS || en == ENOMEM) {
-				int throttle = tcp_throttle();
-				log_error("throttling TCP connection pool for "
-				          "%d seconds, too many allocated "
-				          "resources", throttle);
-				sleep(throttle);
-			}
-
+			return KNOT_EBUSY;
 		}
+		return KNOT_ERROR;
 	} else {
 		dbg_net("tcp: accepted connection fd=%d\n", incoming);
 		/* Set recv() timeout. */
 #ifdef SO_RCVTIMEO
 		struct timeval tv;
 		rcu_read_lock();
-		tv.tv_sec = conf()->max_conn_idle;
+		conf_val_t val = conf_get(conf(), C_SRV, C_TCP_IDLE_TIMEOUT);
+		tv.tv_sec = conf_int(&val);
 		rcu_read_unlock();
 		tv.tv_usec = 0;
 		if (setsockopt(incoming, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)) < 0) {
-			log_warning("cannot set up TCP connection watchdog "
-			            "timer, fd %d", incoming);
+			log_warning("TCP, failed to set up watchdog timer"
+			            ", fd %d", incoming);
 		}
 #endif
 	}
@@ -198,106 +205,6 @@ int tcp_accept(int fd)
 	return incoming;
 }
 
-
-/*! \brief Wait for data and return true if data arrived. */
-static int tcp_wait_for_data(int fd, struct timeval *timeout)
-{
-	fd_set set;
-	FD_ZERO(&set);
-	FD_SET(fd, &set);
-	return select(fd + 1, &set, NULL, NULL, timeout);
-}
-
-int tcp_recv_data(int fd, uint8_t *buf, int len, struct timeval *timeout)
-{
-	int ret = 0;
-	int rcvd = 0;
-	int flags = 0;
-
-#ifdef MSG_NOSIGNAL
-	flags |= MSG_NOSIGNAL;
-#endif
-
-	while (rcvd < len) {
-		/* Receive data. */
-		ret = recv(fd, buf + rcvd, len - rcvd, flags);
-		if (ret > 0) {
-			rcvd += ret;
-			continue;
-		}
-		/* Check for disconnected socket. */
-		if (ret == 0) {
-			return KNOT_ECONNREFUSED;
-		}
-
-		/* Check for no data available. */
-		if (errno == EAGAIN || errno == EINTR) {
-			/* Continue only if timeout didn't expire. */
-			ret = tcp_wait_for_data(fd, timeout);
-			if (ret) {
-				continue;
-			} else {
-				return KNOT_ETIMEOUT;
-			}
-		} else {
-			return KNOT_ECONN;
-		}
-	}
-
-	return rcvd;
-}
-
-int tcp_send_msg(int fd, const uint8_t *msg, size_t msglen)
-{
-	/* Create iovec for gathered write. */
-	struct iovec iov[2];
-	uint16_t pktsize = htons(msglen);
-	iov[0].iov_base = &pktsize;
-	iov[0].iov_len = sizeof(uint16_t);
-	iov[1].iov_base = (void *)msg;
-	iov[1].iov_len = msglen;
-
-	/* Send. */
-	int total_len = iov[0].iov_len + iov[1].iov_len;
-	int sent = writev(fd, iov, 2);
-	if (sent != total_len) {
-		return KNOT_ECONN;
-	}
-
-	return msglen; /* Do not count the size prefix. */
-}
-
-int tcp_recv_msg(int fd, uint8_t *buf, size_t len, struct timeval *timeout)
-{
-	if (buf == NULL || fd < 0) {
-		return KNOT_EINVAL;
-	}
-
-	/* Receive size. */
-	unsigned short pktsize = 0;
-	int ret = tcp_recv_data(fd, (uint8_t *)&pktsize, sizeof(pktsize), timeout);
-	if (ret != sizeof(pktsize)) {
-		return ret;
-	}
-
-	pktsize = ntohs(pktsize);
-	dbg_net("tcp: incoming packet size=%hu on fd=%d\n", pktsize, fd);
-
-	// Check packet size
-	if (len < pktsize) {
-		return KNOT_ENOMEM;
-	}
-
-	/* Receive payload. */
-	ret = tcp_recv_data(fd, buf, pktsize, timeout);
-	if (ret != pktsize) {
-		return ret;
-	}
-
-	dbg_net("tcp: received packet size=%d on fd=%d\n", ret, fd);
-	return ret;
-}
-
 static int tcp_event_accept(tcp_context_t *tcp, unsigned i)
 {
 	/* Accept client. */
@@ -313,11 +220,14 @@ static int tcp_event_accept(tcp_context_t *tcp, unsigned i)
 
 		/* Update watchdog timer. */
 		rcu_read_lock();
-		fdset_set_watchdog(&tcp->set, next_id, conf()->max_conn_hs);
+		conf_val_t val = conf_get(conf(), C_SRV, C_TCP_HSHAKE_TIMEOUT);
+		fdset_set_watchdog(&tcp->set, next_id, conf_int(&val));
 		rcu_read_unlock();
+
+		return KNOT_EOK;
 	}
 
-	return KNOT_EOK;
+	return client;
 }
 
 static int tcp_event_serve(tcp_context_t *tcp, unsigned i)
@@ -326,12 +236,13 @@ static int tcp_event_serve(tcp_context_t *tcp, unsigned i)
 	int ret = tcp_handle(tcp, fd, &tcp->iov[0], &tcp->iov[1]);
 
 	/* Flush per-query memory. */
-	mp_flush(tcp->query_ctx.mm.ctx);
+	mp_flush(tcp->overlay.mm->ctx);
 
 	if (ret == KNOT_EOK) {
 		/* Update socket activity timer. */
 		rcu_read_lock();
-		fdset_set_watchdog(&tcp->set, i, conf()->max_conn_idle);
+		conf_val_t val = conf_get(conf(), C_SRV, C_TCP_IDLE_TIMEOUT);
+		fdset_set_watchdog(&tcp->set, i, conf_int(&val));
 		rcu_read_unlock();
 	}
 
@@ -346,41 +257,48 @@ static int tcp_wait_for_events(tcp_context_t *tcp)
 
 	/* Mark the time of last poll call. */
 	time_now(&tcp->last_poll_time);
+	bool is_throttled = (tcp->last_poll_time.tv_sec < tcp->throttle_end.tv_sec);
+	if (!is_throttled) {
+		/* Configuration limit, infer maximal pool size. */
+		rcu_read_lock();
+		conf_val_t val = conf_get(conf(), C_SRV, C_MAX_TCP_CLIENTS);
+		unsigned max_per_set = MAX(conf_int(&val) / conf_tcp_threads(conf()), 1);
+		rcu_read_unlock();
+		/* Subtract master sockets check limits. */
+		is_throttled = (set->n - tcp->client_threshold) >= max_per_set;
+	}
 
 	/* Process events. */
 	unsigned i = 0;
 	while (nfds > 0 && i < set->n) {
-
-		/* Terminate faulty connections. */
+		bool should_close = false;
 		int fd = set->pfd[i].fd;
-
-		/* Active sockets. */
-		if (set->pfd[i].revents & POLLIN) {
-			--nfds; /* One less active event. */
-
-			/* Indexes <0, client_threshold) are master sockets. */
+		if (set->pfd[i].revents & (POLLERR|POLLHUP|POLLNVAL)) {
+			should_close = (i >= tcp->client_threshold);
+			--nfds;
+		} else if (set->pfd[i].revents & (POLLIN)) {
+			/* Master sockets */
 			if (i < tcp->client_threshold) {
-				/* Faulty master sockets shall be sorted later. */
-				(void) tcp_event_accept(tcp, i);
+				if (!is_throttled && tcp_event_accept(tcp, i) == KNOT_EBUSY) {
+					time_now(&tcp->throttle_end);
+					tcp->throttle_end.tv_sec += tcp_throttle();
+				}
+			/* Client sockets */
 			} else {
 				if (tcp_event_serve(tcp, i) != KNOT_EOK) {
-					fdset_remove(set, i);
-					close(fd);
-					continue; /* Stay on the same index. */
+					should_close = true;
 				}
 			}
-
+			--nfds;
 		}
 
-		if (set->pfd[i].revents & (POLLERR|POLLHUP|POLLNVAL)) {
-			--nfds; /* One less active event. */
+		/* Evaluate */
+		if (should_close) {
 			fdset_remove(set, i);
 			close(fd);
-			continue; /* Stay on the same index. */
+		} else {
+			++i;
 		}
-
-		/* Next socket. */
-		++i;
 	}
 
 	return nfds;
@@ -400,15 +318,18 @@ int tcp_master(dthread_t *thread)
 	tcp_context_t tcp;
 	memset(&tcp, 0, sizeof(tcp_context_t));
 
+	/* Create big enough memory cushion. */
+	mm_ctx_t mm;
+	mm_ctx_mempool(&mm, 16 * MM_DEFAULT_BLKSIZE);
+
 	/* Create TCP answering context. */
 	tcp.server = handler->server;
 	tcp.thread_id = handler->thread_id[dt_get_id(thread)];
-
-	/* Create big enough memory cushion. */
-	mm_ctx_mempool(&tcp.query_ctx.mm, 4 * sizeof(knot_pkt_t));
+	tcp.overlay.mm = &mm;
 
 	/* Prepare structures for bound sockets. */
-	fdset_init(&tcp.set, list_size(&conf()->ifaces) + CONFIG_XFERS);
+	conf_val_t val = conf_get(conf(), C_SRV, C_LISTEN);
+	fdset_init(&tcp.set, conf_val_count(&val) + CONF_XFERS);
 
 	/* Create iovec abstraction. */
 	for (unsigned i = 0; i < 2; ++i) {
@@ -428,7 +349,7 @@ int tcp_master(dthread_t *thread)
 	for(;;) {
 
 		/* Check handler state. */
-		if (knot_unlikely(*iostate & ServerReload)) {
+		if (unlikely(*iostate & ServerReload)) {
 			*iostate &= ~ServerReload;
 
 			/* Cancel client connections. */
@@ -464,15 +385,9 @@ int tcp_master(dthread_t *thread)
 finish:
 	free(tcp.iov[0].iov_base);
 	free(tcp.iov[1].iov_base);
-	mp_delete(tcp.query_ctx.mm.ctx);
+	mp_delete(mm.ctx);
 	fdset_clear(&tcp.set);
 	ref_release(ref);
 
 	return ret;
 }
-
-int tcp_master_destruct(dthread_t *thread)
-{
-	knot_crypto_cleanup_thread();
-	return KNOT_EOK;
-}
