@@ -29,6 +29,7 @@
 #include <errno.h>
 #include <limits.h>
 #include <sys/param.h>
+#include <urcu.h>
 #ifdef HAVE_SYS_UIO_H /* 'struct iovec' for OpenBSD */
 #include <sys/uio.h>
 #endif /* HAVE_SYS_UIO_H */
@@ -36,17 +37,14 @@
 #include <cap-ng.h>
 #endif /* HAVE_CAP_NG_H */
 
-#include "common-knot/sockaddr.h"
-#include "libknot/mempattern.h"
-#include "common/mempool.h"
-#include "knot/knot.h"
 #include "knot/server/udp-handler.h"
 #include "knot/server/server.h"
-#include "libknot/packet/wire.h"
-#include "libknot/consts.h"
-#include "libknot/packet/pkt.h"
-#include "libknot/dnssec/crypto.h"
-#include "libknot/processing/process.h"
+#include "libknot/internal/sockaddr.h"
+#include "libknot/internal/mempattern.h"
+#include "libknot/internal/mempool.h"
+#include "libknot/internal/macros.h"
+#include "libknot/libknot.h"
+#include "libknot/processing/overlay.h"
 
 /* Buffer identifiers. */
 enum {
@@ -57,9 +55,9 @@ enum {
 
 /*! \brief UDP context data. */
 typedef struct udp_context {
-	knot_process_t query_ctx; /*!< Query processing context. */
-	server_t *server;         /*!< Name server structure. */
-	unsigned thread_id;       /*!< Thread identifier. */
+	struct knot_overlay overlay; /*!< Query processing overlay. */
+	server_t *server;            /*!< Name server structure. */
+	unsigned thread_id;          /*!< Thread identifier. */
 } udp_context_t;
 
 /* FD_COPY macro compat. */
@@ -111,12 +109,6 @@ static inline void udp_pps_sample(unsigned n, unsigned thr_id) {}
 void udp_handle(udp_context_t *udp, int fd, struct sockaddr_storage *ss,
                 struct iovec *rx, struct iovec *tx)
 {
-#ifdef DEBUG_ENABLE_BRIEF
-	char addr_str[SOCKADDR_STRLEN] = {0};
-	sockaddr_tostr(ss, addr_str, sizeof(addr_str));
-	dbg_net("%s: received %zd bytes from '%s'\n", __func__, rx->iov_len, addr_str);
-#endif
-
 	/* Create query processing parameter. */
 	struct process_query_param param = {0};
 	param.remote = ss;
@@ -128,37 +120,42 @@ void udp_handle(udp_context_t *udp, int fd, struct sockaddr_storage *ss,
 	param.thread_id = udp->thread_id;
 
 	/* Rate limit is applied? */
-	if (knot_unlikely(udp->server->rrl != NULL) && udp->server->rrl->rate > 0) {
+	if (unlikely(udp->server->rrl != NULL) && udp->server->rrl->rate > 0) {
 		param.proc_flags |= NS_QUERY_LIMIT_RATE;
 	}
 
+	/* Create packets. */
+	mm_ctx_t *mm = udp->overlay.mm;
+	knot_pkt_t *query = knot_pkt_new(rx->iov_base, rx->iov_len, mm);
+	knot_pkt_t *ans = knot_pkt_new(tx->iov_base, tx->iov_len, mm);
+
 	/* Create query processing context. */
-	knot_process_begin(&udp->query_ctx, &param, NS_PROC_QUERY);
+	knot_overlay_init(&udp->overlay, mm);
+	knot_overlay_add(&udp->overlay, NS_PROC_QUERY, &param);
 
 	/* Input packet. */
-	int state = knot_process_in(rx->iov_base, rx->iov_len, &udp->query_ctx);
+	(void) knot_pkt_parse(query, 0);
+	int state = knot_overlay_consume(&udp->overlay, query);
 
 	/* Process answer. */
-	uint16_t tx_len = tx->iov_len;
-	if (state == NS_PROC_FULL) {
-		state = knot_process_out(tx->iov_base, &tx_len, &udp->query_ctx);
-	}
-
-	/* Process error response (if failed). */
-	if (state == NS_PROC_FAIL) {
-		tx_len = tx->iov_len; /* Reset size. */
-		state = knot_process_out(tx->iov_base, &tx_len, &udp->query_ctx);
+	while (state & (KNOT_STATE_PRODUCE|KNOT_STATE_FAIL)) {
+		state = knot_overlay_produce(&udp->overlay, ans);
 	}
 
 	/* Send response only if finished successfuly. */
-	if (state == NS_PROC_DONE) {
-		tx->iov_len = tx_len;
+	if (state == KNOT_STATE_DONE) {
+		tx->iov_len = ans->size;
 	} else {
 		tx->iov_len = 0;
 	}
 
-	/* Reset context. */
-	knot_process_finish(&udp->query_ctx);
+	/* Reset after processing. */
+	knot_overlay_finish(&udp->overlay);
+	knot_overlay_deinit(&udp->overlay);
+
+	/* Cleanup. */
+	knot_pkt_free(&query);
+	knot_pkt_free(&ans);
 }
 
 /* Check for sendmmsg syscall. */
@@ -444,18 +441,6 @@ void __attribute__ ((constructor)) udp_master_init()
 #endif /* HAVE_RECVMMSG */
 }
 
-
-int udp_send_msg(int fd, const uint8_t *msg, size_t msglen, struct sockaddr *addr)
-{
-	int addr_len = sockaddr_len((struct sockaddr_storage *)addr);
-	int ret = sendto(fd, msg, msglen, 0, addr, addr_len);
-	if (ret != msglen) {
-		return KNOT_ECONN;
-	}
-
-	return ret;
-}
-
 /*! \brief Release the reference on the interface list and clear watched fdset. */
 static void forget_ifaces(ifacelist_t *ifaces, fd_set *set, int maxfd)
 {
@@ -515,7 +500,9 @@ int udp_master(dthread_t *thread)
 	udp.thread_id = handler->thread_id[thr_id];
 
 	/* Create big enough memory cushion. */
-	mm_ctx_mempool(&udp.query_ctx.mm, 4 * sizeof(knot_pkt_t));
+	mm_ctx_t mm;
+	mm_ctx_mempool(&mm, 16 * MM_DEFAULT_BLKSIZE);
+	udp.overlay.mm = &mm;
 
 	/* Chose select as epoll/kqueue has larger overhead for a
 	 * single or handful of sockets. */
@@ -530,7 +517,7 @@ int udp_master(dthread_t *thread)
 	for (;;) {
 
 		/* Check handler state. */
-		if (knot_unlikely(*iostate & ServerReload)) {
+		if (unlikely(*iostate & ServerReload)) {
 			*iostate &= ~ServerReload;
 			udp.thread_id = handler->thread_id[thr_id];
 
@@ -560,7 +547,7 @@ int udp_master(dthread_t *thread)
 				if ((rcvd = _udp_recv(fd, rq)) > 0) {
 					_udp_handle(&udp, rq);
 					/* Flush allocated memory. */
-					mp_flush(udp.query_ctx.mm.ctx);
+					mp_flush(mm.ctx);
 					_udp_send(rq);
 					udp_pps_sample(rcvd, thr_id);
 				}
@@ -570,12 +557,6 @@ int udp_master(dthread_t *thread)
 
 	_udp_deinit(rq);
 	forget_ifaces(ref, &fds, maxfd);
-	mp_delete(udp.query_ctx.mm.ctx);
-	return KNOT_EOK;
-}
-
-int udp_master_destruct(dthread_t *thread)
-{
-	knot_crypto_cleanup_thread();
+	mp_delete(mm.ctx);
 	return KNOT_EOK;
 }
