@@ -17,14 +17,12 @@
 #include <assert.h>
 
 #include "knot/updates/apply.h"
-
-#include "knot/zone/zone.h"
-#include "libknot/common.h"
 #include "knot/updates/changesets.h"
+#include "knot/zone/zone.h"
 #include "knot/zone/zonefile.h"
-#include "common-knot/lists.h"
-#include "libknot/rrtype/soa.h"
-#include "libknot/rrtype/rrsig.h"
+#include "libknot/libknot.h"
+#include "libknot/internal/lists.h"
+#include "libknot/internal/macros.h"
 
 /* --------------------------- Update cleanup ------------------------------- */
 
@@ -245,7 +243,7 @@ static int apply_remove(zone_contents_t *contents, changeset_t *chset)
 {
 	changeset_iter_t itt;
 	changeset_iter_rem(&itt, chset, false);
-	
+
 	knot_rrset_t rr = changeset_iter_next(&itt);
 	while (!knot_rrset_empty(&rr)) {
 		// Find node for this owner
@@ -263,7 +261,7 @@ static int apply_remove(zone_contents_t *contents, changeset_t *chset)
 			changeset_iter_clear(&itt);
 			return ret;
 		}
-		
+
 		rr = changeset_iter_next(&itt);
 	}
 	changeset_iter_clear(&itt);
@@ -322,7 +320,7 @@ static int apply_add(zone_contents_t *contents, changeset_t *chset,
 {
 	changeset_iter_t itt;
 	changeset_iter_add(&itt, chset, false);
-	
+
 	knot_rrset_t rr = changeset_iter_next(&itt);
 	while(!knot_rrset_empty(&rr)) {
 		// Get or create node with this owner
@@ -340,7 +338,7 @@ static int apply_add(zone_contents_t *contents, changeset_t *chset,
 		rr = changeset_iter_next(&itt);
 	}
 	changeset_iter_clear(&itt);
-	
+
 	return KNOT_EOK;
 }
 
@@ -454,9 +452,8 @@ int apply_changesets(zone_t *zone, list_t *chsets, zone_contents_t **new_content
 	 * Apply the changesets.
 	 */
 	changeset_t *set = NULL;
-	const bool master = !zone_is_slave(zone);
 	WALK_LIST(set, *chsets) {
-		ret = apply_single(contents_copy, set, master);
+		ret = apply_single(contents_copy, set, !zone_is_slave(zone));
 		if (ret != KNOT_EOK) {
 			updates_rollback(chsets);
 			update_free_zone(&contents_copy);
@@ -494,24 +491,23 @@ int apply_changeset(zone_t *zone, changeset_t *change, zone_contents_t **new_con
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
-	
-	const bool master = !zone_is_slave(zone);
-	ret = apply_single(contents_copy, change, master);
+
+	ret = apply_single(contents_copy, change, !zone_is_slave(zone));
 	if (ret != KNOT_EOK) {
 		update_rollback(change);
 		update_free_zone(&contents_copy);
 		return ret;
 	}
-	
+
 	ret = finalize_updated_zone(contents_copy, true);
 	if (ret != KNOT_EOK) {
 		update_rollback(change);
 		update_free_zone(&contents_copy);
 		return ret;
 	}
-	
+
 	*new_contents = contents_copy;
-	
+
 	return KNOT_EOK;
 }
 
@@ -535,7 +531,7 @@ int apply_changesets_directly(zone_contents_t *contents, list_t *chsets)
 	if (ret != KNOT_EOK) {
 		updates_cleanup(chsets);
 	}
-	
+
 	return ret;
 }
 
@@ -544,20 +540,20 @@ int apply_changeset_directly(zone_contents_t *contents, changeset_t *ch)
 	if (contents == NULL || ch == NULL) {
 		return KNOT_EINVAL;
 	}
-	
+
 	const bool master = true; // Only DNSSEC changesets are applied directly.
 	int ret = apply_single(contents, ch, master);
 	if (ret != KNOT_EOK) {
 		update_cleanup(ch);
 		return ret;
 	}
-	
+
 	ret = finalize_updated_zone(contents, true);
 	if (ret != KNOT_EOK) {
 		update_cleanup(ch);
 		return ret;
 	}
-	
+
 	return KNOT_EOK;
 }
 
@@ -618,4 +614,3 @@ void update_free_zone(zone_contents_t **contents)
 	free(*contents);
 	*contents = NULL;
 }
-
