@@ -15,8 +15,8 @@
  */
 
 #include "knot/updates/zone-update.h"
-#include "common-knot/lists.h"
-#include "common/mempool.h"
+#include "libknot/internal/lists.h"
+#include "libknot/internal/mempool.h"
 
 static int add_to_node(zone_node_t *node, const zone_node_t *add_node,
                        mm_ctx_t *mm)
@@ -83,7 +83,7 @@ static int deep_copy_node_data(zone_node_t *node_copy, const zone_node_t *node,
 	// Clear space for RRs
 	node_copy->rrs = NULL;
 	node_copy->rrset_count = 0;
-	
+
 	for (uint16_t i = 0; i < node->rrset_count; ++i) {
 		knot_rrset_t rr = node_rrset_at(node, i);
 		int ret = node_add_rrset(node_copy, &rr, mm);
@@ -119,7 +119,7 @@ void zone_update_init(zone_update_t *update, const zone_contents_t *zone, change
 {
 	update->zone = zone;
 	update->change = change;
-	mm_ctx_mempool(&update->mm, 4096);
+	mm_ctx_mempool(&update->mm, MM_DEFAULT_BLKSIZE);
 }
 
 const zone_node_t *zone_update_get_node(zone_update_t *update, const knot_dname_t *dname)
