@@ -26,12 +26,12 @@
 
 #pragma once
 
-#include "common-knot/lists.h"
+#include "libknot/internal/lists.h"
 #include "libknot/rrtype/nsec3param.h"
 #include "knot/zone/node.h"
 #include "knot/zone/zone-tree.h"
 
-struct zone_t;
+struct zone;
 
 enum zone_contents_find_dname_result {
 	ZONE_NAME_FOUND = 1,
@@ -40,7 +40,7 @@ enum zone_contents_find_dname_result {
 
 /*----------------------------------------------------------------------------*/
 
-typedef struct zone_contents_t {
+typedef struct zone_contents {
 	zone_node_t *apex;       /*!< Apex node of the zone (holding SOA) */
 
 	zone_tree_t *nodes;
@@ -273,10 +273,6 @@ const knot_rdataset_t *zone_contents_soa(const zone_contents_t *zone);
  */
 uint32_t zone_contents_serial(const zone_contents_t *zone);
 
-
-/*! \brief Calculate next serial. */
-uint32_t zone_contents_next_serial(const zone_contents_t *zone, int policy);
-
 /*!
  * \brief Return true if zone is signed.
  */
