@@ -14,10 +14,16 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "libknot/rrtype/soa.h"
-#include "libknot/dnssec/random.h"
-#include "common-knot/trim.h"
-#include "common/mempool.h"
+#include <urcu.h>
+
+#include "dnssec/random.h"
+#include "libknot/libknot.h"
+#include "libknot/internal/mempool.h"
+#include "libknot/internal/macros.h"
+#include "libknot/processing/requestor.h"
+
+#include "knot/common/log.h"
+#include "knot/common/trim.h"
 #include "knot/server/udp-handler.h"
 #include "knot/server/tcp-handler.h"
 #include "knot/updates/changesets.h"
@@ -31,7 +37,6 @@
 #include "knot/nameserver/internet.h"
 #include "knot/nameserver/update.h"
 #include "knot/nameserver/notify.h"
-#include "knot/nameserver/requestor.h"
 #include "knot/nameserver/tsig_ctx.h"
 #include "knot/nameserver/process_answer.h"
 
@@ -41,8 +46,8 @@
 /* ------------------------- zone query requesting -------------------------- */
 
 /*! \brief Zone event logging. */
-#define ZONE_QUERY_LOG(severity, zone, remote, operation, msg...) \
-	NS_PROC_LOG(severity, &remote->addr, zone->name, operation, msg)
+#define ZONE_QUERY_LOG(severity, zone, remote, operation, msg, ...) \
+	NS_PROC_LOG(severity, &(remote)->addr, zone->name, operation, msg, ##__VA_ARGS__)
 
 /*! \brief Create zone query packet. */
 static knot_pkt_t *zone_query(const zone_t *zone, uint16_t pkt_type, mm_ctx_t *mm)
@@ -61,7 +66,7 @@ static knot_pkt_t *zone_query(const zone_t *zone, uint16_t pkt_type, mm_ctx_t *m
 		return NULL;
 	}
 
-	knot_wire_set_id(pkt->wire, knot_random_uint16_t());
+	knot_wire_set_id(pkt->wire, dnssec_random_uint16_t());
 	knot_wire_set_opcode(pkt->wire, opcode);
 	if (pkt_type == KNOT_QUERY_NOTIFY) {
 		knot_wire_set_aa(pkt->wire);
@@ -74,57 +79,23 @@ static knot_pkt_t *zone_query(const zone_t *zone, uint16_t pkt_type, mm_ctx_t *m
 	if (pkt_type == KNOT_QUERY_IXFR) {  /* RFC1995, SOA in AUTHORITY. */
 		knot_pkt_begin(pkt, KNOT_AUTHORITY);
 		knot_rrset_t soa_rr = node_rrset(contents->apex, KNOT_RRTYPE_SOA);
-		knot_pkt_put(pkt, COMPR_HINT_QNAME, &soa_rr, 0);
+		knot_pkt_put(pkt, KNOT_COMPR_HINT_QNAME, &soa_rr, 0);
 	} else if (pkt_type == KNOT_QUERY_NOTIFY) { /* RFC1996, SOA in ANSWER. */
 		knot_pkt_begin(pkt, KNOT_ANSWER);
 		knot_rrset_t soa_rr = node_rrset(contents->apex, KNOT_RRTYPE_SOA);
-		knot_pkt_put(pkt, COMPR_HINT_QNAME, &soa_rr, 0);
+		knot_pkt_put(pkt, KNOT_COMPR_HINT_QNAME, &soa_rr, 0);
 	}
 
 	return pkt;
 }
 
-static int prepare_edns(zone_t *zone, knot_pkt_t *pkt)
-{
-	/* Check if an extra EDNS option is configured. */
-	if (zone->conf->req_edns_data == NULL) {
-		return KNOT_EOK;
-	}
-
-	knot_rrset_t opt_rr;
-	int ret = knot_edns_init(&opt_rr, conf()->max_udp_payload, 0,
-	                         KNOT_EDNS_VERSION, &pkt->mm);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
-	ret = knot_edns_add_option(&opt_rr, zone->conf->req_edns_code,
-	                           zone->conf->req_edns_data_len,
-	                           (uint8_t *)zone->conf->req_edns_data,
-	                           &pkt->mm);
-	if (ret != KNOT_EOK) {
-		knot_rrset_clear(&opt_rr, &pkt->mm);
-		return ret;
-	}
-
-	knot_pkt_begin(pkt, KNOT_ADDITIONAL);
-
-	ret = knot_pkt_put(pkt, COMPR_HINT_NONE, &opt_rr, KNOT_PF_FREE);
-	if (ret != KNOT_EOK) {
-		knot_rrset_clear(&opt_rr, &pkt->mm);
-		return ret;
-	}
-
-	return KNOT_EOK;
-}
-
 /*!
  * \brief Create a zone event query, send it, wait for the response and process it.
  *
  * \note Everything in this function is executed synchronously, returns when
  *       the query processing is either complete or an error occurs.
  */
-static int zone_query_execute(zone_t *zone, uint16_t pkt_type, const conf_iface_t *remote)
+static int zone_query_execute(zone_t *zone, uint16_t pkt_type, const conf_remote_t *remote)
 {
 	/* Create a memory pool for this task. */
 	int ret = KNOT_EOK;
@@ -137,21 +108,20 @@ static int zone_query_execute(zone_t *zone, uint16_t pkt_type, const conf_iface_
 		return KNOT_ENOMEM;
 	}
 
-	/* Create requestor instance. */
-	struct requestor re;
-	requestor_init(&re, NS_PROC_ANSWER, &mm);
-
 	/* Answer processing parameters. */
 	struct process_answer_param param = { 0 };
 	param.zone = zone;
 	param.query = query;
-	param.remote = remote;
-	tsig_init(&param.tsig_ctx, remote->key);
+	param.remote = &remote->addr;
 
-	ret = prepare_edns(zone, query);
-	if (ret != KNOT_EOK) {
-		goto fail;
-	}
+	/* Create requestor instance. */
+	struct knot_requestor re;
+	knot_requestor_init(&re, &mm);
+	knot_requestor_overlay(&re, KNOT_STATE_ANSWER, &param);
+
+	const knot_tsig_key_t *key = remote->key.name != NULL ?
+	                             &remote->key : NULL;
+	tsig_init(&param.tsig_ctx, key);
 
 	ret = tsig_sign_packet(&param.tsig_ctx, query);
 	if (ret != KNOT_EOK) {
@@ -159,38 +129,41 @@ static int zone_query_execute(zone_t *zone, uint16_t pkt_type, const conf_iface_
 	}
 
 	/* Create a request. */
-	struct request *req = requestor_make(&re, remote, query);
+	const struct sockaddr *dst = (const struct sockaddr *)&remote->addr;
+	const struct sockaddr *src = (const struct sockaddr *)&remote->via;
+	struct knot_request *req = knot_request_make(re.mm, dst, src, query, 0);
 	if (req == NULL) {
 		ret = KNOT_ENOMEM;
 		goto fail;
 	}
 
 	/* Send the queries and process responses. */
-	ret = requestor_enqueue(&re, req, &param);
+	ret = knot_requestor_enqueue(&re, req);
 	if (ret == KNOT_EOK) {
-		struct timeval tv = { conf()->max_conn_reply, 0 };
-		ret = requestor_exec(&re, &tv);
+		conf_val_t val = conf_get(conf(), C_SRV, C_TCP_REPLY_TIMEOUT);
+		struct timeval tv = { conf_int(&val), 0 };
+		ret = knot_requestor_exec(&re, &tv);
 	}
 
 fail:
 	/* Cleanup. */
 	tsig_cleanup(&param.tsig_ctx);
-	requestor_clear(&re);
+	knot_requestor_clear(&re);
 	mp_delete(mm.ctx);
 
 	return ret;
 }
 
 /* @note Module specific, expects some variables set. */
-#define ZONE_XFER_LOG(severity, pkt_type, msg...) \
+#define ZONE_XFER_LOG(severity, pkt_type, msg, ...) \
 	if (pkt_type == KNOT_QUERY_AXFR) { \
-		ZONE_QUERY_LOG(severity, zone, master, "AXFR, incoming", msg); \
+		ZONE_QUERY_LOG(severity, zone, master, "AXFR, incoming", msg, ##__VA_ARGS__); \
 	} else { \
-		ZONE_QUERY_LOG(severity, zone, master, "IXFR, incoming", msg); \
+		ZONE_QUERY_LOG(severity, zone, master, "IXFR, incoming", msg, ##__VA_ARGS__); \
 	}
 
 /*! \brief Execute zone transfer request. */
-static int zone_query_transfer(zone_t *zone, const conf_iface_t *master, uint16_t pkt_type)
+static int zone_query_transfer(zone_t *zone, const conf_remote_t *master, uint16_t pkt_type)
 {
 	assert(zone);
 	assert(master);
@@ -204,7 +177,7 @@ static int zone_query_transfer(zone_t *zone, const conf_iface_t *master, uint16_
 		}
 
 		/* Log connection errors. */
-		ZONE_XFER_LOG(LOG_ERR, pkt_type, "failed (%s)", knot_strerror(ret));
+		ZONE_XFER_LOG(LOG_WARNING, pkt_type, "failed (%s)", knot_strerror(ret));
 	}
 
 	return ret;
@@ -249,7 +222,8 @@ static uint32_t soa_graceful_expire(const knot_rdataset_t *soa)
 {
 	// Allow for timeouts.  Otherwise zones with very short
 	// expiry may expire before the timeout is reached.
-	return knot_soa_expire(soa) + 2 * conf()->max_conn_idle;
+	conf_val_t val = conf_get(conf(), C_SRV, C_TCP_IDLE_TIMEOUT);
+	return knot_soa_expire(soa) + 2 * conf_int(&val);
 }
 
 /*! \brief Schedule expire event, unless it is already scheduled. */
@@ -269,24 +243,25 @@ int event_reload(zone_t *zone)
 	assert(zone);
 
 	/* Take zone file mtime and load it. */
-	time_t mtime = zonefile_mtime(zone->conf->file);
+	char *filename = conf_zonefile(conf(), zone->name);
+	time_t mtime = zonefile_mtime(filename);
+	free(filename);
 	uint32_t dnssec_refresh = time(NULL);
-	conf_zone_t *zone_config = zone->conf;
-	zone_contents_t *contents = zone_load_contents(zone_config);
+	zone_contents_t *contents = zone_load_contents(conf(), zone->name);
 	if (!contents) {
 		return KNOT_ERROR;
 	}
 
 	/* Store zonefile serial and apply changes from the journal. */
 	zone->zonefile_serial = zone_contents_serial(contents);
-	int result = zone_load_journal(zone, contents);
+	int result = zone_load_journal(conf(), zone, contents);
 	if (result != KNOT_EOK) {
 		goto fail;
 	}
 
 	/* Post load actions - calculate delta, sign with DNSSEC... */
 	/*! \todo issue #242 dnssec signing should occur in the special event */
-	result = zone_load_post(contents, zone, &dnssec_refresh);
+	result = zone_load_post(conf(), contents, zone, &dnssec_refresh);
 	if (result != KNOT_EOK) {
 		if (result == KNOT_ESPACE) {
 			log_zone_error(zone->name, "journal size is too small "
@@ -299,7 +274,7 @@ int event_reload(zone_t *zone)
 	}
 
 	/* Check zone contents consistency. */
-	result = zone_load_check(contents, zone_config);
+	result = zone_load_check(conf(), contents);
 	if (result != KNOT_EOK) {
 		goto fail;
 	}
@@ -307,7 +282,6 @@ int event_reload(zone_t *zone)
 	/* Everything went alright, switch the contents. */
 	zone->zonefile_mtime = mtime;
 	zone_contents_t *old = zone_switch_contents(zone, contents);
-	zone->flags &= ~ZONE_EXPIRED;
 	uint32_t old_serial = zone_contents_serial(old);
 	if (old != NULL) {
 		synchronize_rcu();
@@ -324,25 +298,30 @@ int event_reload(zone_t *zone)
 	}
 
 	/* Schedule zone resign. */
-	if (zone->conf->dnssec_enable) {
+	conf_val_t val = conf_zone_get(conf(), C_DNSSEC_SIGNING, zone->name);
+	if (conf_bool(&val)) {
 		schedule_dnssec(zone, dnssec_refresh);
 	}
 
 	/* Periodic execution. */
-	zone_events_schedule(zone, ZONE_EVENT_FLUSH, zone_config->dbsync_timeout);
+	val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+	int64_t sync_timeout = conf_int(&val);
+	if (sync_timeout >= 0) {
+		zone_events_schedule(zone, ZONE_EVENT_FLUSH, sync_timeout);
+	}
 
 	uint32_t current_serial = zone_contents_serial(zone->contents);
 	log_zone_info(zone->name, "loaded, serial %u -> %u",
 	              old_serial, current_serial);
 
-	return KNOT_EOK;
+	return zone_events_write_persistent(zone);
 
 fail:
 	zone_contents_deep_free(&contents);
 	return result;
 }
 
-static int try_refresh(zone_t *zone, const conf_iface_t *master, void *ctx)
+static int try_refresh(zone_t *zone, const conf_remote_t *master, void *ctx)
 {
 	assert(zone);
 	assert(master);
@@ -360,8 +339,8 @@ int event_refresh(zone_t *zone)
 {
 	assert(zone);
 
+	/* Ignore if not slave zone. */
 	if (!zone_is_slave(zone)) {
-		/* If not slave zone, ignore. */
 		return KNOT_EOK;
 	}
 
@@ -371,7 +350,7 @@ int event_refresh(zone_t *zone)
 		return KNOT_EOK;
 	}
 
-	int ret = zone_master_try(zone, try_refresh, NULL);
+	int ret = zone_master_try(zone, try_refresh, NULL, "refresh");
 	const knot_rdataset_t *soa = zone_soa(zone);
 	if (ret != KNOT_EOK) {
 		log_zone_error(zone->name, "refresh, failed (%s)",
@@ -384,14 +363,14 @@ int event_refresh(zone_t *zone)
 		zone_events_schedule(zone, ZONE_EVENT_REFRESH, knot_soa_refresh(soa));
 	}
 
-	return KNOT_EOK;
+	return zone_events_write_persistent(zone);
 }
 
 struct transfer_data {
 	uint16_t pkt_type;
 };
 
-static int try_transfer(zone_t *zone, const conf_iface_t *master, void *_data)
+static int try_xfer(zone_t *zone, const conf_remote_t *master, void *_data)
 {
 	assert(zone);
 	assert(master);
@@ -406,26 +385,29 @@ int event_xfer(zone_t *zone)
 {
 	assert(zone);
 
+	/* Ignore if not slave zone. */
 	if (!zone_is_slave(zone)) {
-		/* If not slave zone, ignore. */
 		return KNOT_EOK;
 	}
 
 	struct transfer_data data = { 0 };
+	const char *err_str = "";
 
 	/* Determine transfer type. */
 	bool is_bootstrap = zone_contents_is_empty(zone->contents);
 	if (is_bootstrap || zone->flags & ZONE_FORCE_AXFR) {
 		data.pkt_type = KNOT_QUERY_AXFR;
+		err_str = "AXFR, incoming";
 	} else {
 		data.pkt_type = KNOT_QUERY_IXFR;
+		err_str = "IXFR, incoming";
 	}
 
-	/* Execute zone transfer and clear master server preference. */
-	int ret = zone_master_try(zone, try_transfer, &data);
-	zone->preferred_master = NULL;
+	/* Execute zone transfer. */
+	int ret = zone_master_try(zone, try_xfer, &data, err_str);
+	zone_clear_preferred_master(zone);
 	if (ret != KNOT_EOK) {
-		log_zone_error(zone->name, "transfer, failed (%s)",
+		log_zone_error(zone->name, "%s, failed (%s)", err_str,
 		               knot_strerror(ret));
 		if (is_bootstrap) {
 			zone->bootstrap_retry = bootstrap_next(zone->bootstrap_retry);
@@ -446,10 +428,13 @@ int event_xfer(zone_t *zone)
 	zone_events_schedule(zone, ZONE_EVENT_REFRESH, knot_soa_refresh(soa));
 	zone_events_schedule(zone, ZONE_EVENT_NOTIFY,  ZONE_EVENT_NOW);
 	zone_events_cancel(zone, ZONE_EVENT_EXPIRE);
-	if (zone->conf->dbsync_timeout == 0) {
+	conf_val_t val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+	int64_t sync_timeout = conf_int(&val);
+	if (sync_timeout == 0) {
 		zone_events_schedule(zone, ZONE_EVENT_FLUSH, ZONE_EVENT_NOW);
-	} else if (!zone_events_is_scheduled(zone, ZONE_EVENT_FLUSH)) {
-		zone_events_schedule(zone, ZONE_EVENT_FLUSH, zone->conf->dbsync_timeout);
+	} else if (sync_timeout > 0 &&
+	           !zone_events_is_scheduled(zone, ZONE_EVENT_FLUSH)) {
+		zone_events_schedule(zone, ZONE_EVENT_FLUSH, sync_timeout);
 	}
 
 	/* Transfer cleanup. */
@@ -499,7 +484,6 @@ int event_expire(zone_t *zone)
 	/* Expire zonefile information. */
 	zone->zonefile_mtime = 0;
 	zone->zonefile_serial = 0;
-	zone->flags |= ZONE_EXPIRED;
 	zone_contents_deep_free(&expired);
 
 	log_zone_info(zone->name, "zone expired");
@@ -515,9 +499,10 @@ int event_flush(zone_t *zone)
 	assert(zone);
 
 	/* Reschedule. */
-	int next_timeout = zone->conf->dbsync_timeout;
-	if (next_timeout > 0) {
-		zone_events_schedule(zone, ZONE_EVENT_FLUSH, next_timeout);
+	conf_val_t val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+	int64_t sync_timeout = conf_int(&val);
+	if (sync_timeout > 0) {
+		zone_events_schedule(zone, ZONE_EVENT_FLUSH, sync_timeout);
 	}
 
 	/* Check zone contents. */
@@ -538,19 +523,27 @@ int event_notify(zone_t *zone)
 	}
 
 	/* Walk through configured remotes and send messages. */
-	conf_remote_t *remote = 0;
-	WALK_LIST(remote, zone->conf->acl.notify_out) {
-		conf_iface_t *iface = remote->remote;
+	conf_val_t notify = conf_zone_get(conf(), C_NOTIFY, zone->name);
+	while (notify.code == KNOT_EOK) {
+		conf_val_t addr = conf_id_get(conf(), C_RMT, C_ADDR, &notify);
+		size_t addr_count = conf_val_count(&addr);
 
-		int ret = zone_query_execute(zone, KNOT_QUERY_NOTIFY, iface);
-		if (ret == KNOT_EOK) {
-			ZONE_QUERY_LOG(LOG_INFO, zone, iface, "NOTIFY, outgoing",
-			               "serial %u",
-			               zone_contents_serial(zone->contents));
-		} else {
-			ZONE_QUERY_LOG(LOG_WARNING, zone, iface, "NOTIFY, outgoing",
-			               "failed (%s)", knot_strerror(ret));
+		for (int i = 0; i < addr_count; i++) {
+			conf_remote_t slave = conf_remote(conf(), &notify, i);
+			int ret = zone_query_execute(zone, KNOT_QUERY_NOTIFY, &slave);
+			if (ret == KNOT_EOK) {
+				ZONE_QUERY_LOG(LOG_INFO, zone, &slave,
+				               "NOTIFY, outgoing", "serial %u",
+				               zone_contents_serial(zone->contents));
+				break;
+			} else {
+				ZONE_QUERY_LOG(LOG_WARNING, zone, &slave,
+				               "NOTIFY, outgoing", "failed (%s)",
+				               knot_strerror(ret));
+			}
 		}
+
+		conf_val_next(&notify);
 	}
 
 	return KNOT_EOK;
@@ -567,24 +560,25 @@ int event_dnssec(zone_t *zone)
 	}
 
 	uint32_t refresh_at = time(NULL);
+	int sign_flags = 0;
+
 	if (zone->flags & ZONE_FORCE_RESIGN) {
 		log_zone_info(zone->name, "DNSSEC, dropping previous "
 		              "signatures, resigning zone");
-
 		zone->flags &= ~ZONE_FORCE_RESIGN;
-		ret = knot_dnssec_zone_sign_force(zone->contents, zone->conf,
-		                                  &ch, &refresh_at);
+		sign_flags = ZONE_SIGN_DROP_SIGNATURES;
 	} else {
 		log_zone_info(zone->name, "DNSSEC, signing zone");
-		ret = knot_dnssec_zone_sign(zone->contents, zone->conf,
-		                            &ch, KNOT_SOA_SERIAL_UPDATE,
-		                            &refresh_at);
+		sign_flags = 0;
 	}
+
+	ret = knot_dnssec_zone_sign(zone->contents, &ch, sign_flags, &refresh_at);
 	if (ret != KNOT_EOK) {
 		goto done;
 	}
 
-	if (!changeset_empty(&ch)) {
+	bool zone_changed = !changeset_empty(&ch);
+	if (zone_changed) {
 		/* Apply change. */
 		zone_contents_t *new_contents = NULL;
 		int ret = apply_changeset(zone, &ch, &new_contents);
@@ -606,7 +600,6 @@ int event_dnssec(zone_t *zone)
 
 		/* Switch zone contents. */
 		zone_contents_t *old_contents = zone_switch_contents(zone, new_contents);
-		zone->flags &= ~ZONE_EXPIRED;
 		synchronize_rcu();
 		update_free_zone(&old_contents);
 
@@ -616,9 +609,12 @@ int event_dnssec(zone_t *zone)
 	// Schedule dependent events.
 
 	schedule_dnssec(zone, refresh_at);
-	zone_events_schedule(zone, ZONE_EVENT_NOTIFY, ZONE_EVENT_NOW);
-	if (zone->conf->dbsync_timeout == 0) {
-		zone_events_schedule(zone, ZONE_EVENT_FLUSH, ZONE_EVENT_NOW);
+	if (zone_changed) {
+		zone_events_schedule(zone, ZONE_EVENT_NOTIFY, ZONE_EVENT_NOW);
+		conf_val_t val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+		if (conf_int(&val) == 0) {
+			zone_events_schedule(zone, ZONE_EVENT_FLUSH, ZONE_EVENT_NOW);
+		}
 	}
 
 done:
@@ -632,7 +628,7 @@ done:
 uint32_t bootstrap_next(uint32_t timer)
 {
 	timer *= 2;
-	timer += knot_random_uint32_t() % BOOTSTRAP_RETRY;
+	timer += dnssec_random_uint32_t() % BOOTSTRAP_RETRY;
 	if (timer > BOOTSTRAP_MAXTIME) {
 		timer = BOOTSTRAP_MAXTIME;
 	}
