@@ -15,9 +15,11 @@
 */
 
 #include "libknot/rrtype/soa.h"
+
 #include "knot/zone/events/replan.h"
 #include "knot/zone/events/handlers.h"
 #include "knot/zone/zone.h"
+#include "libknot/internal/macros.h"
 
 /* -- Zone event replanning functions --------------------------------------- */
 
@@ -74,7 +76,9 @@ static void replan_xfer(zone_t *zone, const zone_t *old_zone)
 /*!< \brief Replans flush event. */
 static void replan_flush(zone_t *zone, const zone_t *old_zone)
 {
-	if (zone->conf->dbsync_timeout <= 0) {
+	conf_val_t val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+	int64_t sync_timeout = conf_int(&val);
+	if (sync_timeout <= 0) {
 		// Immediate sync scheduled after events.
 		return;
 	}
@@ -82,19 +86,19 @@ static void replan_flush(zone_t *zone, const zone_t *old_zone)
 	const time_t flush_time = zone_events_get_time(old_zone, ZONE_EVENT_FLUSH);
 	if (flush_time <= ZONE_EVENT_NOW) {
 		// Not scheduled previously.
-		zone_events_schedule(zone, ZONE_EVENT_FLUSH, zone->conf->dbsync_timeout);
+		zone_events_schedule(zone, ZONE_EVENT_FLUSH, sync_timeout);
 		return;
 	}
 
 	// Pick time to schedule: either reuse or schedule sooner than old event.
-	const time_t schedule_at = MIN(time(NULL) + zone->conf->dbsync_timeout, flush_time);
+	const time_t schedule_at = MIN(time(NULL) + sync_timeout, flush_time);
 	zone_events_schedule_at(zone, ZONE_EVENT_FLUSH, schedule_at);
 }
 
 /*!< \brief Creates new DDNS q in the new zone - q contains references from the old zone. */
 static void duplicate_ddns_q(zone_t *zone, zone_t *old_zone)
 {
-	struct request_data *d, *nxt;
+	struct knot_request *d, *nxt;
 	WALK_LIST_DELSAFE(d, nxt, old_zone->ddns_queue) {
 		add_tail(&zone->ddns_queue, (node_t *)d);
 	}
@@ -107,7 +111,8 @@ static void duplicate_ddns_q(zone_t *zone, zone_t *old_zone)
 /*!< Replans DNSSEC event. Not whole resign needed, \todo #247 */
 static void replan_dnssec(zone_t *zone)
 {
-	if (zone->conf->dnssec_enable) {
+	conf_val_t val = conf_zone_get(conf(), C_DNSSEC_SIGNING, zone->name);
+	if (conf_bool(&val)) {
 		/* Keys could have changed, force resign. */
 		zone_events_schedule(zone, ZONE_EVENT_DNSSEC, ZONE_EVENT_NOW);
 	}
