@@ -21,21 +21,14 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include "knot/knot.h"
-#include "knot/other/debug.h"
-#include "libknot/libknot.h"
-#include "libknot/dnssec/key.h"
-#include "libknot/dnssec/rrset-sign.h"
-#include "libknot/rrtype/rrsig.h"
-#include "libknot/rrtype/soa.h"
-#include "libknot/rrtype/nsec.h"
-#include "libknot/rrtype/nsec3.h"
-#include "common/base32hex.h"
-#include "libknot/descriptor.h"
-#include "libknot/mempattern.h"
-#include "knot/dnssec/zone-nsec.h"
-
+#include "dnssec/keytag.h"
 #include "knot/zone/semantic-check.h"
+#include "knot/common/debug.h"
+#include "knot/dnssec/rrset-sign.h"
+#include "knot/dnssec/zone-nsec.h"
+#include "libknot/libknot.h"
+#include "libknot/internal/base32hex.h"
+#include "libknot/internal/mempattern.h"
 
 static char *error_messages[(-ZC_ERR_UNKNOWN) + 1] = {
 	[-ZC_ERR_MISSING_SOA] =
@@ -126,7 +119,7 @@ static char *error_messages[(-ZC_ERR_UNKNOWN) + 1] = {
 void err_handler_init(err_handler_t *h)
 {
 	memset(h, 0, sizeof(err_handler_t));
-	memset(h->errors, 0, sizeof(uint) * (-ZC_ERR_UNKNOWN + 1));
+	memset(h->errors, 0, sizeof(unsigned) * (-ZC_ERR_UNKNOWN + 1));
 	h->options.log_cname = 0;
 	h->options.log_glue = 0;
 	h->options.log_rrsigs = 0;
@@ -254,7 +247,7 @@ static int check_dnskey_rdata(const knot_rrset_t *rrset, size_t rdata_pos)
 	const uint16_t mask = 1 << 8; //0b0000000100000000;
 
 	const knot_rdata_t *rr_data = knot_rdataset_at(&rrset->rrs, rdata_pos);
-	uint16_t flags = knot_wire_read_u16(knot_rdata_data(rr_data));
+	uint16_t flags = wire_read_u16(knot_rdata_data(rr_data));
 	if (flags & mask) {
 		return KNOT_EOK;
 	} else {
@@ -365,16 +358,20 @@ static int check_rrsig_rdata(err_handler_t *handler,
 	uint16_t key_tag_rrsig = knot_rrsig_key_tag(rrsig, rr_pos);
 	for (uint16_t i = 0; i < dnskey_rrset->rrs.rr_count &&
 	     !match; ++i) {
-		uint8_t dnskey_alg =
-			knot_dnskey_alg(&dnskey_rrset->rrs, i);
+		uint8_t dnskey_alg = knot_dnskey_alg(&dnskey_rrset->rrs, i);
 		if (rrsig_alg != dnskey_alg) {
 			continue;
 		}
 
 		/* Calculate keytag. */
-		const knot_rdata_t *rr_data = knot_rdataset_at(&dnskey_rrset->rrs, i);
-		uint16_t dnskey_key_tag =
-			knot_keytag(knot_rdata_data(rr_data), knot_rdata_rdlen(rr_data));
+		const knot_rdata_t *dnskey_rr = knot_rdataset_at(&dnskey_rrset->rrs, i);
+		dnssec_binary_t rdata = {
+			.size = knot_rdata_rdlen(dnskey_rr),
+			.data = knot_rdata_data(dnskey_rr)
+		};
+		uint16_t dnskey_key_tag = 0;
+		dnssec_keytag(&rdata, &dnskey_key_tag);
+
 		if (key_tag_rrsig != dnskey_key_tag) {
 			continue;
 		}
@@ -529,7 +526,6 @@ static int rdata_nsec_to_type_array(const knot_rdataset_t *rrs, uint16_t type,
 		uint8_t *bitmap =
 			malloc(sizeof(uint8_t) * (bitmap_size));
 		if (bitmap == NULL) {
-			ERR_ALLOC_FAILED;
 			free(*array);
 			return KNOT_ENOMEM;
 		}
@@ -546,7 +542,6 @@ static int rdata_nsec_to_type_array(const knot_rdataset_t *rrs, uint16_t type,
 						    sizeof(uint16_t) *
 						    *count);
 				if (tmp == NULL) {
-					ERR_ALLOC_FAILED;
 					free(bitmap);
 					free(*array);
 					return KNOT_ENOMEM;
