@@ -16,7 +16,7 @@
 
 #pragma once
 
-#include "common/namedb/namedb.h"
+#include "libknot/internal/namedb/namedb.h"
 #include "knot/zone/zone.h"
 #include "knot/zone/zonedb.h"
 
@@ -28,14 +28,14 @@
  *
  * \return KNOT_E*
  */
-int open_timers_db(const char *storage, knot_namedb_t **timer_db);
+int open_timers_db(const char *storage, namedb_t **timer_db);
 
 /*!
  * \brief Closes zone timers db.
  *
  * \param timer_db  Timer database.
  */
-void close_timers_db(knot_namedb_t *timer_db);
+void close_timers_db(namedb_t *timer_db);
 
 /*!
  * \brief Reads zone timers from timers db.
@@ -50,17 +50,17 @@ void close_timers_db(knot_namedb_t *timer_db);
  *
  * \return KNOT_E*
  */
-int read_zone_timers(knot_namedb_t *timer_db, const zone_t *zone, time_t *timers);
+int read_zone_timers(namedb_t *timer_db, const zone_t *zone, time_t *timers);
 
 /*!
- * \brief Writes all zone timers to timers db.
+ * \brief Writes zone timers to timers db.
  *
  * \param timer_db  Timer database.
- * \param zone_db   Zone database.
+ * \param zone      Zone to store timers for.
  *
  * \return KNOT_E*
  */
-int write_timer_db(knot_namedb_t *timer_db, knot_zonedb_t *zone_db);
+int write_zone_timers(namedb_t *timer_db, zone_t *zone);
 
 /*!
  * \brief Removes stale zones info from timers db.
@@ -69,4 +69,4 @@ int write_timer_db(knot_namedb_t *timer_db, knot_zonedb_t *zone_db);
  * \param zone_db   Current zone database.
  * \return KNOT_EOK or an error
  */
-int sweep_timer_db(knot_namedb_t *timer_db, knot_zonedb_t *zone_db);
+int sweep_timer_db(namedb_t *timer_db, knot_zonedb_t *zone_db);
