@@ -20,8 +20,9 @@
 
 #include "knot/zone/zone-tree.h"
 #include "knot/zone/node.h"
-#include "common/debug.h"
-#include "common-knot/hattrie/hat-trie.h"
+#include "knot/common/debug.h"
+#include "libknot/internal/trie/hat-trie.h"
+#include "libknot/internal/macros.h"
 
 /*----------------------------------------------------------------------------*/
 /* API functions                                                              */
@@ -248,7 +249,6 @@ int zone_tree_remove(zone_tree_t *tree,
 		*removed = (zone_node_t *)(*rval);
 	}
 
-
 	hattrie_del(tree, (char*)lf+1, *lf);
 	return KNOT_EOK;
 }
