@@ -18,26 +18,34 @@
 #include <assert.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <urcu.h>
 
+#include "dnssec/random.h"
 #include "libknot/descriptor.h"
-#include "common-knot/evsched.h"
-#include "common-knot/lists.h"
-#include "common-knot/trim.h"
+#include "knot/common/evsched.h"
+#include "libknot/internal/lists.h"
+#include "knot/common/log.h"
+#include "knot/common/trim.h"
 #include "knot/zone/node.h"
+#include "knot/zone/serial.h"
 #include "knot/zone/zone.h"
 #include "knot/zone/zonefile.h"
 #include "knot/zone/contents.h"
+#include "knot/updates/acl.h"
 #include "knot/updates/apply.h"
-#include "knot/nameserver/requestor.h"
-#include "libknot/common.h"
+#include "libknot/processing/requestor.h"
+#include "knot/nameserver/process_query.h"
+#include "libknot/libknot.h"
 #include "libknot/dname.h"
-#include "libknot/dnssec/random.h"
-#include "libknot/util/utils.h"
+#include "libknot/internal/utils.h"
+#include "libknot/internal/mem.h"
 #include "libknot/rrtype/soa.h"
 
+#define JOURNAL_SUFFIX	".diff.db"
+
 static void free_ddns_queue(zone_t *z)
 {
-	struct request_data *n = NULL;
+	struct knot_request *n = NULL;
 	node_t *nxt = NULL;
 	WALK_LIST_DELSAFE(n, nxt, z->ddns_queue) {
 		close(n->fd);
@@ -47,30 +55,20 @@ static void free_ddns_queue(zone_t *z)
 	}
 }
 
-zone_t* zone_new(conf_zone_t *conf)
+zone_t* zone_new(const knot_dname_t *name)
 {
-	if (!conf) {
-		return NULL;
-	}
-
 	zone_t *zone = malloc(sizeof(zone_t));
 	if (zone == NULL) {
-		ERR_ALLOC_FAILED;
 		return NULL;
 	}
 	memset(zone, 0, sizeof(zone_t));
 
-	zone->name = knot_dname_from_str_alloc(conf->name);
-	knot_dname_to_lower(zone->name);
+	zone->name = knot_dname_copy(name, NULL);
 	if (zone->name == NULL) {
 		free(zone);
-		ERR_ALLOC_FAILED;
 		return NULL;
 	}
 
-	// Configuration
-	zone->conf = conf;
-
 	// DDNS
 	pthread_mutex_init(&zone->ddns_lock, NULL);
 	zone->ddns_queue_size = 0;
@@ -79,6 +77,9 @@ zone_t* zone_new(conf_zone_t *conf)
 	// Journal lock
 	pthread_mutex_init(&zone->journal_lock, NULL);
 
+	// Preferred master lock
+	pthread_mutex_init(&zone->preferred_lock, NULL);
+
 	// Initialize events
 	zone_events_init(zone);
 
@@ -101,12 +102,18 @@ void zone_free(zone_t **zone_ptr)
 	pthread_mutex_destroy(&zone->ddns_lock);
 	pthread_mutex_destroy(&zone->journal_lock);
 
-	/* Free assigned config. */
-	conf_free_zone(zone->conf);
+	/* Free preferred master. */
+	pthread_mutex_destroy(&zone->preferred_lock);
+	free(zone->preferred_master);
 
 	/* Free zone contents. */
 	zone_contents_deep_free(&zone->contents);
 
+	if (zone->query_plan != NULL) {
+		conf_deactivate_modules(conf(), &zone->query_modules,
+		                        zone->query_plan);
+	}
+
 	free(zone);
 	*zone_ptr = NULL;
 }
@@ -116,21 +123,25 @@ int zone_change_store(zone_t *zone, changeset_t *change)
 	assert(zone);
 	assert(change);
 
-	conf_zone_t *conf = zone->conf;
+	conf_val_t val = conf_zone_get(conf(), C_MAX_JOURNAL_SIZE, zone->name);
+	int64_t ixfr_fslimit = conf_int(&val);
+	char *journal_file = conf_journalfile(conf(), zone->name);
 
 	pthread_mutex_lock(&zone->journal_lock);
-	int ret = journal_store_changeset(change, conf->ixfr_db, conf->ixfr_fslimit);
+	int ret = journal_store_changeset(change, journal_file, ixfr_fslimit);
 	if (ret == KNOT_EBUSY) {
 		log_zone_notice(zone->name, "journal is full, flushing");
 
 		/* Transaction rolled back, journal released, we may flush. */
 		ret = zone_flush_journal(zone);
 		if (ret == KNOT_EOK) {
-			ret = journal_store_changeset(change, conf->ixfr_db, conf->ixfr_fslimit);
+			ret = journal_store_changeset(change, journal_file, ixfr_fslimit);
 		}
 	}
 	pthread_mutex_unlock(&zone->journal_lock);
 
+	free(journal_file);
+
 	return ret;
 }
 
@@ -139,22 +150,26 @@ int zone_changes_store(zone_t *zone, list_t *chgs)
 	assert(zone);
 	assert(chgs);
 
-	conf_zone_t *conf = zone->conf;
+	conf_val_t val = conf_zone_get(conf(), C_MAX_JOURNAL_SIZE, zone->name);
+	int64_t ixfr_fslimit = conf_int(&val);
+	char *journal_file = conf_journalfile(conf(), zone->name);
 
 	pthread_mutex_lock(&zone->journal_lock);
-	int ret = journal_store_changesets(chgs, conf->ixfr_db, conf->ixfr_fslimit);
-
+	int ret = journal_store_changesets(chgs, journal_file, ixfr_fslimit);
 	if (ret == KNOT_EBUSY) {
 		log_zone_notice(zone->name, "journal is full, flushing");
 
 		/* Transaction rolled back, journal released, we may flush. */
 		ret = zone_flush_journal(zone);
 		if (ret == KNOT_EOK) {
-			ret = journal_store_changesets(chgs, conf->ixfr_db, conf->ixfr_fslimit);
+			ret = journal_store_changesets(chgs, journal_file, ixfr_fslimit);
 		}
+
 	}
 	pthread_mutex_unlock(&zone->journal_lock);
 
+	free(journal_file);
+
 	return ret;
 }
 
@@ -171,59 +186,87 @@ zone_contents_t *zone_switch_contents(zone_t *zone, zone_contents_t *new_content
 	return old_contents;
 }
 
-const conf_iface_t *zone_master(const zone_t *zone)
-{
-	if (zone == NULL) {
-		return NULL;
-	}
-
-	if (EMPTY_LIST(zone->conf->acl.xfr_in)) {
-		return NULL;
-	}
-
-	conf_remote_t *master = HEAD(zone->conf->acl.xfr_in);
-	return master->remote;
-}
-
 bool zone_is_slave(const zone_t *zone)
 {
-	return zone && !EMPTY_LIST(zone->conf->acl.xfr_in);
+	if (zone == NULL) {
+		return false;
+	}
+
+	conf_val_t val = conf_zone_get(conf(), C_MASTER, zone->name);
+	return conf_val_count(&val) > 0 ? true : false;
+}
+
+void zone_set_preferred_master(zone_t *zone, const struct sockaddr_storage *addr)
+{
+	if (zone == NULL || addr == NULL) {
+		return;
+	}
+
+	pthread_mutex_lock(&zone->preferred_lock);
+	free(zone->preferred_master);
+	zone->preferred_master = malloc(sizeof(struct sockaddr_storage));
+	*zone->preferred_master = *addr;
+	pthread_mutex_unlock(&zone->preferred_lock);
+}
+
+void zone_clear_preferred_master(zone_t *zone)
+{
+	if (zone == NULL) {
+		return;
+	}
+
+	pthread_mutex_lock(&zone->preferred_lock);
+	free(zone->preferred_master);
+	zone->preferred_master = NULL;
+	pthread_mutex_unlock(&zone->preferred_lock);
 }
 
 /*!
- * \brief Get zone preferred master while checking it's existence.
- *
- * It is possible that the preferred master will change during the function
- * execution. We can ignore this race. In the worst case, the preferred master
- * won't be found and we will try all servers in default order.
+ * \brief Get preferred zone master while checking its existence.
  */
-static const conf_remote_t *preferred_master(const zone_t *zone)
+int static preferred_master(zone_t *zone, conf_remote_t *master)
 {
-	if (!zone->preferred_master) {
-		return NULL;
+	pthread_mutex_lock(&zone->preferred_lock);
+
+	if (zone->preferred_master == NULL) {
+		pthread_mutex_unlock(&zone->preferred_lock);
+		return KNOT_ENOENT;
 	}
 
-	conf_remote_t *master = NULL;
-	WALK_LIST(master, zone->conf->acl.xfr_in) {
-		if (master->remote == zone->preferred_master) {
-			return master;
+	conf_val_t masters = conf_zone_get(conf(), C_MASTER, zone->name);
+	while (masters.code == KNOT_EOK) {
+		conf_val_t addr = conf_id_get(conf(), C_RMT, C_ADDR, &masters);
+		size_t addr_count = conf_val_count(&addr);
+
+		for (size_t i = 0; i < addr_count; i++) {
+			conf_remote_t remote = conf_remote(conf(), &masters, i);
+			if (netblock_match(&remote.addr, zone->preferred_master, -1)) {
+				*master = remote;
+				pthread_mutex_unlock(&zone->preferred_lock);
+				return KNOT_EOK;
+			}
 		}
+
+		conf_val_next(&masters);
 	}
 
-	return NULL;
+	pthread_mutex_unlock(&zone->preferred_lock);
+
+	return KNOT_ENOENT;
 }
 
-int zone_master_try(zone_t *zone, zone_master_cb callback, void *callback_data)
+int zone_master_try(zone_t *zone, zone_master_cb callback, void *callback_data,
+                    const char *err_str)
 {
-	if (!zone || EMPTY_LIST(zone->conf->acl.xfr_in)) {
+	if (zone == NULL || callback == NULL) {
 		return KNOT_EINVAL;
 	}
 
 	/* Try the preferred server. */
 
-	const conf_remote_t *preferred = preferred_master(zone);
-	if (preferred) {
-		int ret = callback(zone, preferred->remote, callback_data);
+	conf_remote_t preferred = { { AF_UNSPEC } };
+	if (preferred_master(zone, &preferred) == KNOT_EOK) {
+		int ret = callback(zone, &preferred, callback_data);
 		if (ret == KNOT_EOK) {
 			return ret;
 		}
@@ -231,19 +274,37 @@ int zone_master_try(zone_t *zone, zone_master_cb callback, void *callback_data)
 
 	/* Try all the other servers. */
 
-	conf_remote_t *master = NULL;
-	WALK_LIST(master, zone->conf->acl.xfr_in) {
-		if (master == preferred) {
-			continue;
+	bool success = false;
+
+	conf_val_t masters = conf_zone_get(conf(), C_MASTER, zone->name);
+	while (masters.code == KNOT_EOK) {
+		conf_val_t addr = conf_id_get(conf(), C_RMT, C_ADDR, &masters);
+		size_t addr_count = conf_val_count(&addr);
+
+		for (size_t i = 0; i < addr_count; i++) {
+			conf_remote_t master = conf_remote(conf(), &masters, i);
+			if (preferred.addr.ss_family != AF_UNSPEC &&
+			    netblock_match(&master.addr, &preferred.addr, -1)) {
+				preferred.addr.ss_family = AF_UNSPEC;
+				continue;
+			}
+			int ret = callback(zone, &master, callback_data);
+			if (ret == KNOT_EOK) {
+				success = true;
+				break;
+			}
 		}
 
-		int ret = callback(zone, master->remote, callback_data);
-		if (ret == KNOT_EOK) {
-			return KNOT_EOK;
+		if (!success) {
+			assert(err_str);
+			log_zone_warning(zone->name, "%s, remote '%s' not available",
+			                 err_str, conf_str(&masters));
 		}
+
+		conf_val_next(&masters);
 	}
 
-	return KNOT_ENOMASTER;
+	return success ? KNOT_EOK : KNOT_ENOMASTER;
 }
 
 int zone_flush_journal(zone_t *zone)
@@ -254,6 +315,13 @@ int zone_flush_journal(zone_t *zone)
 		return KNOT_EINVAL;
 	}
 
+	/* Check for disabled zonefile synchronization. */
+	conf_val_t val = conf_zone_get(conf(), C_ZONEFILE_SYNC, zone->name);
+	if (conf_int(&val) < 0 && (zone->flags & ZONE_FORCE_FLUSH) == 0) {
+		return KNOT_EOK;
+	}
+	zone->flags &= ~ZONE_FORCE_FLUSH;
+
 	/* Check for difference against zonefile serial. */
 	zone_contents_t *contents = zone->contents;
 	uint32_t serial_to = zone_contents_serial(contents);
@@ -261,30 +329,39 @@ int zone_flush_journal(zone_t *zone)
 		return KNOT_EOK; /* No differences. */
 	}
 
+	char *zonefile = conf_zonefile(conf(), zone->name);
+
 	/* Synchronize journal. */
-	conf_zone_t *conf = zone->conf;
-	int ret = zonefile_write(conf->file, contents);
+	int ret = zonefile_write(zonefile, contents);
 	if (ret == KNOT_EOK) {
 		log_zone_info(zone->name, "zone file updated, serial %u -> %u",
 		              zone->zonefile_serial, serial_to);
 	} else {
 		log_zone_warning(zone->name, "failed to update zone file (%s)",
 		                 knot_strerror(ret));
+		free(zonefile);
 		return ret;
 	}
 
 	/* Update zone version. */
 	struct stat st;
-	if (stat(zone->conf->file, &st) < 0) {
+	if (stat(zonefile, &st) < 0) {
 		log_zone_warning(zone->name, "failed to update zone file (%s)",
 		                 knot_strerror(KNOT_EACCES));
+		free(zonefile);
 		return KNOT_EACCES;
 	}
 
+	free(zonefile);
+
+	char *journal_file = conf_journalfile(conf(), zone->name);
+
 	/* Update zone file serial and journal. */
 	zone->zonefile_mtime = st.st_mtime;
 	zone->zonefile_serial = serial_to;
-	journal_mark_synced(zone->conf->ixfr_db);
+	journal_mark_synced(journal_file);
+
+	free(journal_file);
 
 	/* Trim extra heap. */
 	mem_trim();
@@ -296,11 +373,11 @@ int zone_update_enqueue(zone_t *zone, knot_pkt_t *pkt, struct process_query_para
 {
 
 	/* Create serialized request. */
-	struct request_data *req = malloc(sizeof(struct request_data));
+	struct knot_request *req = malloc(sizeof(struct knot_request));
 	if (req == NULL) {
 		return KNOT_ENOMEM;
 	}
-	memset(req, 0, sizeof(struct request_data));
+	memset(req, 0, sizeof(struct knot_request));
 
 	/* Copy socket and request. */
 	req->fd = dup(param->socket);
@@ -358,11 +435,15 @@ bool zone_transfer_needed(const zone_t *zone, const knot_pkt_t *pkt)
 	}
 
 	const knot_pktsection_t *answer = knot_pkt_section(pkt, KNOT_ANSWER);
-	const knot_rrset_t soa = answer->rr[0];
-	if (soa.type != KNOT_RRTYPE_SOA) {
+	if (answer->count < 1) {
 		return false;
 	}
 
-	return knot_serial_compare(zone_contents_serial(zone->contents),
-	                           knot_soa_serial(&soa.rrs)) < 0;
+	const knot_rrset_t *soa = knot_pkt_rr(answer, 0);
+	if (soa->type != KNOT_RRTYPE_SOA) {
+		return false;
+	}
+
+	return serial_compare(zone_contents_serial(zone->contents),
+	                           knot_soa_serial(&soa->rrs)) < 0;
 }
