@@ -28,14 +28,15 @@
 #include <stdbool.h>
 #include <stdint.h>
 
-#include "common-knot/evsched.h"
-#include "common-knot/ref.h"
+#include "knot/common/evsched.h"
+#include "knot/common/ref.h"
 #include "knot/conf/conf.h"
 #include "knot/server/journal.h"
 #include "knot/updates/acl.h"
 #include "knot/zone/events/events.h"
 #include "knot/zone/contents.h"
 #include "libknot/dname.h"
+#include "libknot/packet/pkt.h"
 
 struct process_query_param;
 
@@ -45,35 +46,40 @@ struct process_query_param;
 typedef enum zone_flag_t {
 	ZONE_FORCE_AXFR   = 1 << 0, /* Force AXFR as next transfer. */
 	ZONE_FORCE_RESIGN = 1 << 1, /* Force zone resign. */
-	ZONE_EXPIRED      = 1 << 2, /* Zone is expired. */
+	ZONE_FORCE_FLUSH  = 1 << 2  /* Force zone flush. */
 } zone_flag_t;
 
 /*!
  * \brief Structure for holding DNS zone.
  */
-typedef struct zone_t
+typedef struct zone
 {
 	knot_dname_t *name;
 	zone_contents_t *contents;
-	conf_zone_t *conf;
 	zone_flag_t flags;
 
 	/*! \brief DDNS queue and lock. */
 	pthread_mutex_t ddns_lock;
 	size_t ddns_queue_size;
 	list_t ddns_queue;
-	
+
 	/*! \brief Journal access lock. */
 	pthread_mutex_t journal_lock;
 
 	/*! \brief Zone events. */
 	zone_events_t events;     /*!< Zone events timers. */
-	uint32_t bootstrap_retry; /*!< AXFR/IN bootstrap retry. */
 	time_t zonefile_mtime;
+	uint32_t bootstrap_retry; /*!< AXFR/IN bootstrap retry. */
 	uint32_t zonefile_serial;
 
+	/*! \brief Preferred master lock. */
+	pthread_mutex_t preferred_lock;
 	/*! \brief Preferred master for remote operation. */
-	const conf_iface_t *preferred_master;
+	struct sockaddr_storage *preferred_master;
+
+	/*! \brief Query modules. */
+	list_t query_modules;
+	struct query_plan *query_plan;
 } zone_t;
 
 /*----------------------------------------------------------------------------*/
@@ -81,11 +87,11 @@ typedef struct zone_t
 /*!
  * \brief Creates new zone with emtpy zone content.
  *
- * \param conf  Zone configuration.
+ * \param name  Zone name.
  *
  * \return The initialized zone structure or NULL if an error occured.
  */
-zone_t *zone_new(conf_zone_t *conf);
+zone_t* zone_new(const knot_dname_t *name);
 
 /*!
  * \brief Deallocates the zone structure.
@@ -106,13 +112,17 @@ int zone_change_store(zone_t *zone, changeset_t *change);
 /*!
  * \brief Atomically switch the content of the zone.
  */
-zone_contents_t *zone_switch_contents(zone_t *zone,
-					   zone_contents_t *new_contents);
+zone_contents_t *zone_switch_contents(zone_t *zone, zone_contents_t *new_contents);
 
-/*! \brief Check if zone has master. */
+/*! \brief Checks if the zone is slave. */
 bool zone_is_slave(const zone_t *zone);
 
-typedef int (*zone_master_cb)(zone_t *zone, const conf_iface_t *remote, void *data);
+/*! \brief Sets the address as a preferred master address. */
+void zone_set_preferred_master(zone_t *zone, const struct sockaddr_storage *addr);
+/*! \brief Clears the current preferred master address. */
+void zone_clear_preferred_master(zone_t *zone);
+
+typedef int (*zone_master_cb)(zone_t *zone, const conf_remote_t *remote, void *data);
 
 /*!
  * \brief Perform an action with a first working master server.
@@ -123,10 +133,8 @@ typedef int (*zone_master_cb)(zone_t *zone, const conf_iface_t *remote, void *da
  *
  * \return Error code from the last callback.
  */
-int zone_master_try(zone_t *zone, zone_master_cb callback, void *callback_data);
-
-/*! \brief Return zone master remote. */
-const conf_iface_t *zone_master(const zone_t *zone);
+int zone_master_try(zone_t *zone, zone_master_cb callback, void *callback_data,
+                    const char *err_str);
 
 /*! \brief Synchronize zone file with journal. */
 int zone_flush_journal(zone_t *zone);
