@@ -21,16 +21,15 @@
 
 #include "knot/zone/zonedb.h"
 #include "knot/server/server.h"
-#include "libknot/common.h"
 #include "knot/zone/zone.h"
 #include "knot/zone/zonedb.h"
 #include "libknot/dname.h"
 #include "libknot/packet/wire.h"
 #include "knot/zone/node.h"
-#include "common/debug.h"
-#include "libknot/mempattern.h"
-#include "common/mempool.h"
-
+#include "knot/common/debug.h"
+#include "libknot/internal/mempattern.h"
+#include "libknot/internal/mempool.h"
+#include "libknot/internal/macros.h"
 
 /*----------------------------------------------------------------------------*/
 /* Non-API functions                                                          */
@@ -39,14 +38,17 @@
 /*! \brief Discard zone in zone database. */
 static void discard_zone(zone_t *zone)
 {
-	/* Flush bootstrapped zones. */
-	if (zone->zonefile_mtime == 0) {
+	char *journal_file = conf_journalfile(conf(), zone->name);
 
+	/* Flush if bootstrapped or if the journal doesn't exist. */
+	if (zone->zonefile_mtime == 0 || !journal_exists(journal_file)) {
 		pthread_mutex_lock(&zone->journal_lock);
 		zone_flush_journal(zone);
 		pthread_mutex_unlock(&zone->journal_lock);
-
 	}
+
+	free(journal_file);
+
 	zone_free(&zone);
 }
 
