@@ -33,7 +33,7 @@
 #include "knot/zone/zone.h"
 #include "knot/zone/contents.h"
 #include "libknot/dname.h"
-#include "common-knot/hhash.h"
+#include "libknot/internal/hhash.h"
 
 /*
  * Zone DB represents a list of managed zones.
@@ -148,5 +148,4 @@ void knot_zonedb_free(knot_zonedb_t **db);
  */
 void knot_zonedb_deep_free(knot_zonedb_t **db);
 
-
 /*! @} */
