@@ -26,21 +26,22 @@
 #include <unistd.h>
 #include <inttypes.h>
 
-#include "common-knot/strlcat.h"
-#include "common-knot/strlcpy.h"
-#include "libknot/common.h"
+#include "libknot/libknot.h"
+#include "libknot/internal/strlcat.h"
+#include "libknot/internal/strlcpy.h"
+#include "libknot/internal/macros.h"
 #include "knot/zone/semantic-check.h"
 #include "knot/zone/contents.h"
 #include "knot/dnssec/zone-nsec.h"
-#include "knot/other/debug.h"
+#include "knot/common/debug.h"
 #include "knot/zone/zonefile.h"
 #include "libknot/rdata.h"
 #include "knot/zone/zone-dump.h"
 #include "libknot/rrtype/naptr.h"
 
-#define ERROR(zone, fmt...) log_zone_error(zone, "zone loader, " fmt)
-#define WARNING(zone, fmt...) log_zone_warning(zone, "zone loader, " fmt)
-#define INFO(zone, fmt...) log_zone_info(zone, "zone loader, " fmt)
+#define ERROR(zone, fmt, ...) log_zone_error(zone, "zone loader, " fmt, ##__VA_ARGS__)
+#define WARNING(zone, fmt, ...) log_zone_warning(zone, "zone loader, " fmt, ##__VA_ARGS__)
+#define INFO(zone, fmt, ...) log_zone_info(zone, "zone loader, " fmt, ##__VA_ARGS__)
 
 void process_error(zs_scanner_t *s)
 {
@@ -186,23 +187,8 @@ static void scanner_process(zs_scanner_t *scanner)
 	knot_rdataset_clear(&rr.rrs, NULL);
 }
 
-static zone_contents_t *create_zone_from_name(const char *origin)
-{
-	if (origin == NULL) {
-		return NULL;
-	}
-	knot_dname_t *owner = knot_dname_from_str_alloc(origin);
-	if (owner == NULL) {
-		return NULL;
-	}
-	knot_dname_to_lower(owner);
-	zone_contents_t *z = zone_contents_new(owner);
-	knot_dname_free(&owner, NULL);
-	return z;
-}
-
-int zonefile_open(zloader_t *loader, const char *source, const char *origin,
-		  bool semantic_checks)
+int zonefile_open(zloader_t *loader, const char *source,
+                  const knot_dname_t *origin, bool semantic_checks)
 {
 	if (!loader) {
 		return KNOT_EINVAL;
@@ -216,29 +202,36 @@ int zonefile_open(zloader_t *loader, const char *source, const char *origin,
 	/* Create context. */
 	zcreator_t *zc = malloc(sizeof(zcreator_t));
 	if (zc == NULL) {
-		ERR_ALLOC_FAILED;
 		return KNOT_ENOMEM;
 	}
 	memset(zc, 0, sizeof(zcreator_t));
 
-	zc->z = create_zone_from_name(origin);
+	zc->z = zone_contents_new(origin);
 	if (zc->z == NULL) {
 		free(zc);
 		return KNOT_ENOMEM;
 	}
 
+	/* Prepare textual owner for zone scanner. */
+	char *origin_str = knot_dname_to_str_alloc(origin);
+	if (origin_str == NULL) {
+		free(zc);
+		return KNOT_ENOMEM;
+	}
+
 	/* Create file loader. */
 	memset(loader, 0, sizeof(zloader_t));
-	loader->scanner = zs_scanner_create(origin, KNOT_CLASS_IN, 3600,
+	loader->scanner = zs_scanner_create(origin_str, KNOT_CLASS_IN, 3600,
 	                                    scanner_process, process_error,
 	                                    zc);
 	if (loader->scanner == NULL) {
+		free(origin_str);
 		free(zc);
 		return KNOT_ERROR;
 	}
 
 	loader->source = strdup(source);
-	loader->origin = strdup(origin);
+	loader->origin = origin_str;
 	loader->creator = zc;
 	loader->semantic_checks = semantic_checks;
 
@@ -372,7 +365,7 @@ int zonefile_write(const char *path, zone_contents_t *zone)
 	FILE *f = fdopen(fd, "w");
 	if (f == NULL) {
 		WARNING(zname, "failed to open zone, file '%s' (%s)",
-		        new_fname, knot_strerror(knot_errno_to_error(errno)));
+		        new_fname, knot_strerror(-errno));
 		unlink(new_fname);
 		free(new_fname);
 		return KNOT_ERROR;
