@@ -19,15 +19,17 @@
 #include <stdbool.h>
 
 #include "libknot/packet/pkt.h"
-
-#include "common/debug.h"
-#include "common/log.h"
-#include "libknot/common.h"
 #include "libknot/descriptor.h"
-#include "libknot/packet/wire.h"
+#include "libknot/errcode.h"
 #include "libknot/rrtype/tsig.h"
 #include "libknot/tsig-op.h"
+#include "libknot/packet/wire.h"
 #include "libknot/packet/rrset-wire.h"
+#include "libknot/internal/macros.h"
+
+/*! \brief Packet RR array growth step. */
+#define NEXT_RR_ALIGN 16
+#define NEXT_RR_COUNT(count) (((count) / NEXT_RR_ALIGN + 1) * NEXT_RR_ALIGN)
 
 /*! \brief Scan packet for RRSet existence. */
 static bool pkt_contains(const knot_pkt_t *packet,
@@ -120,27 +122,59 @@ static void pkt_rr_wirecount_add(knot_pkt_t *pkt, knot_section_t section_id,
 {
 	assert(pkt);
 	switch (section_id) {
-	case KNOT_ANSWER:     return knot_wire_add_ancount(pkt->wire, val);
-	case KNOT_AUTHORITY:  return knot_wire_add_nscount(pkt->wire, val);
-	case KNOT_ADDITIONAL: return knot_wire_add_arcount(pkt->wire, val);
+	case KNOT_ANSWER:     knot_wire_add_ancount(pkt->wire, val); break;
+	case KNOT_AUTHORITY:  knot_wire_add_nscount(pkt->wire, val); break;
+	case KNOT_ADDITIONAL: knot_wire_add_arcount(pkt->wire, val); break;
 	}
 }
 
+/*! \brief Reserve enough space in the RR arrays. */
+static int pkt_rr_array_alloc(knot_pkt_t *pkt, uint16_t count)
+{
+	/* Enough space. */
+	if (pkt->rrset_allocd >= count) {
+		return KNOT_EOK;
+	}
+
+	/* Allocate rr_info and rr fields to next size. */
+	size_t next_size = NEXT_RR_COUNT(count);
+	knot_rrinfo_t *rr_info = mm_alloc(&pkt->mm, sizeof(knot_rrinfo_t) * next_size);
+	if (rr_info == NULL) {
+		return KNOT_ENOMEM;
+	}
+
+	knot_rrset_t *rr = mm_alloc(&pkt->mm, sizeof(knot_rrset_t) * next_size);
+	if (rr == NULL) {
+		mm_free(&pkt->mm, rr_info);
+		return KNOT_ENOMEM;
+	}
+
+	/* Copy the old data. */
+	memcpy(rr_info, pkt->rr_info, pkt->rrset_allocd * sizeof(knot_rrinfo_t));
+	memcpy(rr, pkt->rr, pkt->rrset_allocd * sizeof(knot_rrset_t));
+
+	/* Reassign and free old data. */
+	mm_free(&pkt->mm, pkt->rr);
+	mm_free(&pkt->mm, pkt->rr_info);
+	pkt->rr = rr;
+	pkt->rr_info = rr_info;
+	pkt->rrset_allocd = next_size;
+
+	return KNOT_EOK;
+}
+
 /*! \brief Clear the packet and switch wireformat pointers (possibly allocate new). */
-static int pkt_reset(knot_pkt_t *pkt, void *wire, uint16_t len)
+static int pkt_init(knot_pkt_t *pkt, void *wire, uint16_t len, mm_ctx_t *mm)
 {
 	assert(pkt);
 
-	/* Free allocated data. */
-	pkt_free_data(pkt);
+	memset(pkt, 0, sizeof(knot_pkt_t));
 
-	/* NULL everything up to 'sections' (not the large data fields). */
-	int ret = KNOT_EOK;
-	mm_ctx_t mm = pkt->mm;
-	memset(pkt, 0, offsetof(knot_pkt_t, rr_info));
-	pkt->mm = mm;
+	/* No data to free, set memory context. */
+	memcpy(&pkt->mm, mm, sizeof(mm_ctx_t));
 
 	/* Initialize wire. */
+	int ret = KNOT_EOK;
 	if (wire == NULL) {
 		ret = pkt_wire_alloc(pkt, len);
 	} else {
@@ -150,10 +184,18 @@ static int pkt_reset(knot_pkt_t *pkt, void *wire, uint16_t len)
 	return ret;
 }
 
+/*! \brief Reset packet parse state. */
+static int pkt_reset_sections(knot_pkt_t *pkt)
+{
+	pkt->parsed  = 0;
+	pkt->current = KNOT_ANSWER;
+	memset(pkt->sections, 0, sizeof(pkt->sections));
+	return knot_pkt_begin(pkt, KNOT_ANSWER);
+}
+
 /*! \brief Clear packet payload and free allocated data. */
 static void pkt_clear_payload(knot_pkt_t *pkt)
 {
-	dbg_packet("%s(%p)\n", __func__, pkt);
 	assert(pkt);
 
 	/* Keep question. */
@@ -166,10 +208,8 @@ static void pkt_clear_payload(knot_pkt_t *pkt)
 	/* Free RRSets if applicable. */
 	pkt_free_data(pkt);
 
-	/* Reset section. */
-	pkt->current = KNOT_ANSWER;
-	pkt->sections[pkt->current].rr = pkt->rr;
-	pkt->sections[pkt->current].rrinfo = pkt->rr_info;
+	/* Reset sections. */
+	pkt_reset_sections(pkt);
 }
 
 /*! \brief Allocate new packet using memory context. */
@@ -177,26 +217,23 @@ static knot_pkt_t *pkt_new_mm(void *wire, uint16_t len, mm_ctx_t *mm)
 {
 	assert(mm);
 
-	knot_pkt_t *pkt = mm->alloc(mm->ctx, sizeof(knot_pkt_t));
+	knot_pkt_t *pkt = mm_alloc(mm, sizeof(knot_pkt_t));
 	if (pkt == NULL) {
 		return NULL;
 	}
 
-	/* No data to free, set memory context. */
-	pkt->rrset_count = 0;
-	memcpy(&pkt->mm, mm, sizeof(mm_ctx_t));
-	if (pkt_reset(pkt, wire, len) != KNOT_EOK) {
-		mm->free(pkt);
+	if (pkt_init(pkt, wire, len, mm) != KNOT_EOK) {
+		mm_free(mm, pkt);
 		return NULL;
 	}
 
 	return pkt;
 }
 
+_public_
 knot_pkt_t *knot_pkt_new(void *wire, uint16_t len, mm_ctx_t *mm)
 {
 	/* Default memory allocator if NULL. */
-	dbg_packet("%s(%p, %hu, %p)\n", __func__, wire, len, mm);
 	mm_ctx_t _mm;
 	if (mm == NULL) {
 		mm_ctx_init(&_mm);
@@ -206,9 +243,9 @@ knot_pkt_t *knot_pkt_new(void *wire, uint16_t len, mm_ctx_t *mm)
 	return pkt_new_mm(wire, len, mm);
 }
 
+_public_
 int knot_pkt_copy(knot_pkt_t *dst, const knot_pkt_t *src)
 {
-	dbg_packet("%s(%p, %p)\n", __func__, dst, src);
 	if (dst == NULL || src == NULL) {
 		return KNOT_EINVAL;
 	}
@@ -229,13 +266,19 @@ int knot_pkt_copy(knot_pkt_t *dst, const knot_pkt_t *src)
 		}
 	}
 
+	/* Invalidate arrays . */
+	dst->rr = NULL;
+	dst->rr_info = NULL;
+	dst->rrset_count = 0;
+	dst->rrset_allocd = 0;
+
 	/* @note This could be done more effectively if needed. */
 	return knot_pkt_parse(dst, 0);
 }
 
+_public_
 int knot_pkt_init_response(knot_pkt_t *pkt, const knot_pkt_t *query)
 {
-	dbg_packet("%s(%p, %p)\n", __func__, pkt, query);
 	if (pkt == NULL || query == NULL) {
 		return KNOT_EINVAL;
 	}
@@ -243,7 +286,6 @@ int knot_pkt_init_response(knot_pkt_t *pkt, const knot_pkt_t *query)
 	/* Header + question size. */
 	size_t base_size = KNOT_WIRE_HEADER_SIZE + knot_pkt_question_size(query);
 	if (base_size > pkt->max_size) {
-		dbg_packet("%s: can't fit HEADER + QUESTION\n", __func__);
 		return KNOT_ESPACE;
 	}
 
@@ -263,9 +305,9 @@ int knot_pkt_init_response(knot_pkt_t *pkt, const knot_pkt_t *query)
 	return KNOT_EOK;
 }
 
+_public_
 void knot_pkt_clear(knot_pkt_t *pkt)
 {
-	dbg_packet("%s(%p)\n", __func__, pkt);
 	if (pkt == NULL) {
 		return;
 	}
@@ -278,9 +320,9 @@ void knot_pkt_clear(knot_pkt_t *pkt)
 	memset(pkt->wire, 0, pkt->size);
 }
 
+_public_
 void knot_pkt_free(knot_pkt_t **pkt)
 {
-	dbg_packet("%s(%p)\n", __func__, pkt);
 	if (pkt == NULL || *pkt == NULL) {
 		return;
 	}
@@ -288,19 +330,22 @@ void knot_pkt_free(knot_pkt_t **pkt)
 	/* Free temporary RRSets. */
 	pkt_free_data(*pkt);
 
+	/* Free RR/RR info arrays. */
+	mm_free(&(*pkt)->mm, (*pkt)->rr);
+	mm_free(&(*pkt)->mm, (*pkt)->rr_info);
+
 	// free the space for wireformat
 	if ((*pkt)->flags & KNOT_PF_FREE) {
 		(*pkt)->mm.free((*pkt)->wire);
 	}
 
-	dbg_packet("Freeing packet structure\n");
 	(*pkt)->mm.free(*pkt);
 	*pkt = NULL;
 }
 
+_public_
 int knot_pkt_reserve(knot_pkt_t *pkt, uint16_t size)
 {
-	dbg_packet("%s(%p, %hu)\n", __func__, pkt, size);
 	if (pkt == NULL) {
 		return KNOT_EINVAL;
 	}
@@ -314,9 +359,9 @@ int knot_pkt_reserve(knot_pkt_t *pkt, uint16_t size)
 	}
 }
 
+_public_
 int knot_pkt_reclaim(knot_pkt_t *pkt, uint16_t size)
 {
-	dbg_packet("%s(%p, %hu)\n", __func__, pkt, size);
 	if (pkt == NULL) {
 		return KNOT_EINVAL;
 	}
@@ -330,9 +375,9 @@ int knot_pkt_reclaim(knot_pkt_t *pkt, uint16_t size)
 
 }
 
+_public_
 uint16_t knot_pkt_type(const knot_pkt_t *pkt)
 {
-	dbg_packet("%s(%p)\n", __func__, pkt);
 	if (pkt == NULL) {
 		return 0;
 	}
@@ -364,10 +409,9 @@ uint16_t knot_pkt_type(const knot_pkt_t *pkt)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 uint16_t knot_pkt_question_size(const knot_pkt_t *pkt)
 {
-	dbg_packet("%s(%p)\n", __func__, pkt);
 	if (pkt == NULL || pkt->qname_size == 0) {
 		return 0;
 	}
@@ -376,10 +420,9 @@ uint16_t knot_pkt_question_size(const knot_pkt_t *pkt)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 const knot_dname_t *knot_pkt_qname(const knot_pkt_t *pkt)
 {
-	dbg_packet("%s(%p)\n", __func__, pkt);
 	if (pkt == NULL || pkt->qname_size == 0) {
 		return NULL;
 	}
@@ -388,31 +431,30 @@ const knot_dname_t *knot_pkt_qname(const knot_pkt_t *pkt)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 uint16_t knot_pkt_qtype(const knot_pkt_t *pkt)
 {
-	dbg_packet("%s(%p)\n", __func__, pkt);
 	if (pkt == NULL || pkt->qname_size == 0) {
 		return 0;
 	}
 
 	unsigned off = KNOT_WIRE_HEADER_SIZE + pkt->qname_size;
-	return knot_wire_read_u16(pkt->wire + off);
+	return wire_read_u16(pkt->wire + off);
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 uint16_t knot_pkt_qclass(const knot_pkt_t *pkt)
 {
-	dbg_packet("%s(%p)\n", __func__, pkt);
 	if (pkt == NULL || pkt->qname_size == 0) {
 		return 0;
 	}
 
 	unsigned off = KNOT_WIRE_HEADER_SIZE + pkt->qname_size + sizeof(uint16_t);
-	return knot_wire_read_u16(pkt->wire + off);
+	return wire_read_u16(pkt->wire + off);
 }
 
+_public_
 int knot_pkt_begin(knot_pkt_t *pkt, knot_section_t section_id)
 {
 	if (pkt == NULL) {
@@ -420,19 +462,19 @@ int knot_pkt_begin(knot_pkt_t *pkt, knot_section_t section_id)
 	}
 
 	/* Cannot step to lower section. */
-	dbg_packet("%s(%p, %u)\n", __func__, pkt, section_id);
 	assert(section_id >= pkt->current);
 	pkt->current = section_id;
 
 	/* Remember watermark. */
-	pkt->sections[section_id].rr = pkt->rr + pkt->rrset_count;
-	pkt->sections[section_id].rrinfo = pkt->rr_info + pkt->rrset_count;
+	pkt->sections[section_id].pkt = pkt;
+	pkt->sections[section_id].pos = pkt->rrset_count;
+
 	return KNOT_EOK;
 }
 
+_public_
 int knot_pkt_put_question(knot_pkt_t *pkt, const knot_dname_t *qname, uint16_t qclass, uint16_t qtype)
 {
-	dbg_packet("%s(%p, %p, %hu, %hu)\n", __func__, pkt, qname, qclass, qtype);
 	if (pkt == NULL || qname == NULL) {
 		return KNOT_EINVAL;
 	}
@@ -449,14 +491,15 @@ int knot_pkt_put_question(knot_pkt_t *pkt, const knot_dname_t *qname, uint16_t q
 
 	/* Check size limits. */
 	size_t question_len = 2 * sizeof(uint16_t) + qname_len;
-	if (qname_len < 0 || pkt->size + question_len > pkt->max_size)
+	if (qname_len < 0 || pkt->size + question_len > pkt->max_size) {
 		return KNOT_ESPACE;
+	}
 
 	/* Copy QTYPE & QCLASS */
 	dst += qname_len;
-	knot_wire_write_u16(dst, qtype);
+	wire_write_u16(dst, qtype);
 	dst += sizeof(uint16_t);
-	knot_wire_write_u16(dst, qclass);
+	wire_write_u16(dst, qclass);
 
 	/* Update question count and sizes. */
 	knot_wire_set_qdcount(pkt->wire, 1);
@@ -467,20 +510,26 @@ int knot_pkt_put_question(knot_pkt_t *pkt, const knot_dname_t *qname, uint16_t q
 	return knot_pkt_begin(pkt, KNOT_ANSWER);
 }
 
+_public_
 int knot_pkt_put(knot_pkt_t *pkt, uint16_t compr_hint, const knot_rrset_t *rr,
                  uint16_t flags)
 {
-	dbg_packet("%s(%p, %u, %p, %u)\n", __func__, pkt, compr_hint, rr, flags);
 	if (pkt == NULL || rr == NULL) {
 		return KNOT_EINVAL;
 	}
 
+	/* Reserve memory for RR descriptors. */
+	int ret = pkt_rr_array_alloc(pkt, pkt->rrset_count + 1);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
 	knot_rrinfo_t *rrinfo = &pkt->rr_info[pkt->rrset_count];
 	memset(rrinfo, 0, sizeof(knot_rrinfo_t));
 	rrinfo->pos = pkt->size;
 	rrinfo->flags = flags;
 	rrinfo->compress_ptr[0] = compr_hint;
-	pkt->rr[pkt->rrset_count] = *rr;
+	memcpy(pkt->rr + pkt->rrset_count, rr, sizeof(knot_rrset_t));
 
 	/* Check for double insertion. */
 	if ((flags & KNOT_PF_CHECKDUP) &&
@@ -500,13 +549,10 @@ int knot_pkt_put(knot_pkt_t *pkt, uint16_t compr_hint, const knot_rrset_t *rr,
 	                                        compr.wire);
 
 	/* Write RRSet to wireformat. */
-	int ret = knot_rrset_to_wire(rr, pos, maxlen, &compr);
+	ret = knot_rrset_to_wire(rr, pos, maxlen, &compr);
 	if (ret < 0) {
-		dbg_packet("%s: rr_to_wire = %s\n,", __func__, knot_strerror(ret));
-
 		/* Truncate packet if required. */
 		if (ret == KNOT_ESPACE && !(flags & KNOT_PF_NOTRUNC)) {
-				dbg_packet("%s: set TC=1\n", __func__);
 				knot_wire_set_tc(pkt->wire);
 		}
 		return ret;
@@ -527,15 +573,13 @@ int knot_pkt_put(knot_pkt_t *pkt, uint16_t compr_hint, const knot_rrset_t *rr,
 		pkt_rr_wirecount_add(pkt, pkt->current, rr_added);
 	}
 
-	dbg_packet("%s: added %u RRs (@%zu, len=%zu), pktsize=%zu\n",
-	           __func__, rr_added, pkt->size - len, len, pkt->size);
-
 	return KNOT_EOK;
 }
 
-const knot_pktsection_t *knot_pkt_section(const knot_pkt_t *pkt, knot_section_t section_id)
+_public_
+const knot_pktsection_t *knot_pkt_section(const knot_pkt_t *pkt,
+                                          knot_section_t section_id)
 {
-	dbg_packet("%s(%p, %u)\n", __func__, pkt, section_id);
 	if (pkt == NULL) {
 		return NULL;
 	}
@@ -543,13 +587,26 @@ const knot_pktsection_t *knot_pkt_section(const knot_pkt_t *pkt, knot_section_t
 	return &pkt->sections[section_id];
 }
 
+_public_
+const knot_rrset_t *knot_pkt_rr(const knot_pktsection_t *section, uint16_t i)
+{
+	if (section == NULL) {
+		return NULL;
+	}
+
+	return section->pkt->rr + section->pos + i;
+}
+
+_public_
 int knot_pkt_parse(knot_pkt_t *pkt, unsigned flags)
 {
-	dbg_packet("%s(%p, %u)\n", __func__, pkt, flags);
 	if (pkt == NULL) {
 		return KNOT_EINVAL;
 	}
 
+	/* Reset parse state. */
+	pkt_reset_sections(pkt);
+
 	int ret = knot_pkt_parse_question(pkt);
 	if (ret == KNOT_EOK) {
 		ret = knot_pkt_parse_payload(pkt, flags);
@@ -558,16 +615,15 @@ int knot_pkt_parse(knot_pkt_t *pkt, unsigned flags)
 	return ret;
 }
 
+_public_
 int knot_pkt_parse_question(knot_pkt_t *pkt)
 {
-	dbg_packet("%s(%p)\n", __func__, pkt);
 	if (pkt == NULL) {
 		return KNOT_EINVAL;
 	}
 
 	/* Check at least header size. */
 	if (pkt->size < KNOT_WIRE_HEADER_SIZE) {
-		dbg_packet("%s: smaller than DNS header, NOREPLY\n", __func__);
 		return KNOT_EMALF;
 	}
 
@@ -577,7 +633,6 @@ int knot_pkt_parse_question(knot_pkt_t *pkt)
 	/* Check QD count. */
 	uint16_t qd = knot_wire_get_qdcount(pkt->wire);
 	if (qd > 1) {
-		dbg_packet("%s: QD(%u) > 1, FORMERR\n", __func__, qd);
 		return KNOT_EMALF;
 	}
 
@@ -598,7 +653,6 @@ int knot_pkt_parse_question(knot_pkt_t *pkt)
 	/* Check QCLASS/QTYPE size. */
 	uint16_t question_size = len + 2 * sizeof(uint16_t); /* QCLASS + QTYPE */
 	if (pkt->parsed + question_size > pkt->size) {
-		dbg_packet("%s: missing QCLASS/QTYPE, FORMERR\n", __func__);
 		return KNOT_EMALF;
 	}
 
@@ -611,31 +665,31 @@ int knot_pkt_parse_question(knot_pkt_t *pkt)
 /* \note Private for check_rr_constraints(). */
 #define CHECK_AR_CONSTRAINTS(pkt, rr, var, check_func) \
 	if ((pkt)->current != KNOT_ADDITIONAL) { \
-		dbg_packet("%s: RRTYPE%u not in AR\n", __func__, rr->type); \
 		return KNOT_EMALF; \
 	} else if ((pkt)->var != NULL) { \
-		dbg_packet("%s: found 2nd RRTYPE%u\n", __func__, rr->type); \
 		return KNOT_EMALF; \
 	} else if (!check_func(rr)) { \
-		dbg_packet("%s: bad RRTYPE%u RDATA\n", __func__, rr->type); \
 		return KNOT_EMALF; \
 	} else { \
 		(pkt)->var = rr; \
 	}
 
-/*! \brief Check constraints (position, uniqueness, validity) for special types (TSIG, OPT). */
-static int check_rr_constraints(knot_pkt_t *pkt, knot_rrset_t *rr, size_t rr_size, unsigned flags)
+/*! \brief Check constraints (position, uniqueness, validity) for special types
+ *         (TSIG, OPT).
+ */
+static int check_rr_constraints(knot_pkt_t *pkt, knot_rrset_t *rr, size_t rr_size,
+                                unsigned flags)
 {
 	/* Check RR constraints. */
 	switch(rr->type) {
 	case KNOT_RRTYPE_TSIG:
-		CHECK_AR_CONSTRAINTS(pkt, rr, tsig_rr, tsig_rdata_is_ok);
+		CHECK_AR_CONSTRAINTS(pkt, rr, tsig_rr, knot_tsig_rdata_is_ok);
 
 		/* Strip TSIG RR from wireformat and decrease ARCOUNT. */
 		if (!(flags & KNOT_PF_KEEPWIRE)) {
 			pkt->parsed -= rr_size;
 			pkt->size -= rr_size;
-			knot_wire_set_id(pkt->wire, tsig_rdata_orig_id(rr));
+			knot_wire_set_id(pkt->wire, knot_tsig_rdata_orig_id(rr));
 			knot_wire_set_arcount(pkt->wire, knot_wire_get_arcount(pkt->wire) - 1);
 		}
 		break;
@@ -651,20 +705,24 @@ static int check_rr_constraints(knot_pkt_t *pkt, knot_rrset_t *rr, size_t rr_siz
 
 #undef CHECK_AR_RECORD
 
+_public_
 int knot_pkt_parse_rr(knot_pkt_t *pkt, unsigned flags)
 {
-	dbg_packet("%s(%p, %u)\n", __func__, pkt, flags);
 	if (pkt == NULL) {
 		return KNOT_EINVAL;
 	}
 
 	if (pkt->parsed >= pkt->size) {
-		dbg_packet("%s: parsed %zu/%zu data\n", __func__, pkt->parsed, pkt->size);
 		return KNOT_EFEWDATA;
 	}
 
+	/* Reserve memory for RR descriptors. */
+	int ret = pkt_rr_array_alloc(pkt, pkt->rrset_count + 1);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
 	/* Initialize RR info. */
-	int ret = KNOT_EOK;
 	memset(&pkt->rr_info[pkt->rrset_count], 0, sizeof(knot_rrinfo_t));
 	pkt->rr_info[pkt->rrset_count].pos = pkt->parsed;
 	pkt->rr_info[pkt->rrset_count].flags = KNOT_PF_FREE;
@@ -672,10 +730,9 @@ int knot_pkt_parse_rr(knot_pkt_t *pkt, unsigned flags)
 	/* Parse wire format. */
 	size_t rr_size = pkt->parsed;
 	knot_rrset_t *rr = &pkt->rr[pkt->rrset_count];
-	ret = knot_rrset_rr_from_wire(pkt->wire, &pkt->parsed, pkt->max_size,
+	ret = knot_rrset_rr_from_wire(pkt->wire, &pkt->parsed, pkt->size,
 	                              &pkt->mm, rr);
 	if (ret != KNOT_EOK) {
-		dbg_packet("%s: failed to parse RR\n", __func__);
 		return ret;
 	}
 
@@ -690,9 +747,9 @@ int knot_pkt_parse_rr(knot_pkt_t *pkt, unsigned flags)
 	return check_rr_constraints(pkt, rr, rr_size, flags);
 }
 
+_public_
 int knot_pkt_parse_section(knot_pkt_t *pkt, unsigned flags)
 {
-	dbg_packet("%s(%p, %u)\n", __func__, pkt, flags);
 	if (pkt == NULL) {
 		return KNOT_EINVAL;
 	}
@@ -705,8 +762,6 @@ int knot_pkt_parse_section(knot_pkt_t *pkt, unsigned flags)
 	for (rr_parsed = 0; rr_parsed < rr_count; ++rr_parsed) {
 		ret = knot_pkt_parse_rr(pkt, flags);
 		if (ret != KNOT_EOK) {
-			dbg_packet("%s: failed to parse RR %u/%u = %d\n",
-			           __func__, rr_parsed, rr_count, ret);
 			return ret;
 		}
 	}
@@ -714,9 +769,9 @@ int knot_pkt_parse_section(knot_pkt_t *pkt, unsigned flags)
 	return KNOT_EOK;
 }
 
+_public_
 int knot_pkt_parse_payload(knot_pkt_t *pkt, unsigned flags)
 {
-	dbg_packet("%s(%p, %u)\n", __func__, pkt, flags);
 	if (pkt == NULL) {
 		return KNOT_EINVAL;
 	}
@@ -724,19 +779,22 @@ int knot_pkt_parse_payload(knot_pkt_t *pkt, unsigned flags)
 	assert(pkt->wire != NULL);
 	assert(pkt->size > 0);
 
-	int ret = KNOT_ERROR;
+	/* Reserve memory in advance to avoid resizing. */
+	size_t rr_count = knot_wire_get_ancount(pkt->wire) +
+	                  knot_wire_get_nscount(pkt->wire) +
+	                  knot_wire_get_arcount(pkt->wire);
+	int ret = pkt_rr_array_alloc(pkt, rr_count);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
 
 	for (knot_section_t i = KNOT_ANSWER; i <= KNOT_ADDITIONAL; ++i) {
 		ret = knot_pkt_begin(pkt, i);
 		if (ret != KNOT_EOK) {
-			dbg_packet("%s: failed to begin section %u = %d\n",
-			           __func__, i, ret);
 			return ret;
 		}
 		ret = knot_pkt_parse_section(pkt, flags);
 		if (ret != KNOT_EOK) {
-			dbg_packet("%s: failed to parse section %u = %d\n",
-			           __func__, i, ret);
 			return ret;
 		}
 	}
@@ -744,22 +802,21 @@ int knot_pkt_parse_payload(knot_pkt_t *pkt, unsigned flags)
 	/* TSIG must be last record of AR if present. */
 	const knot_pktsection_t *ar = knot_pkt_section(pkt, KNOT_ADDITIONAL);
 	if (pkt->tsig_rr != NULL) {
-		if (ar->count > 0 && pkt->tsig_rr->rrs.data != ar->rr[ar->count - 1].rrs.data) {
-			dbg_packet("%s: TSIG not last RR in AR.\n", __func__);
+		const knot_rrset_t *last_rr = knot_pkt_rr(ar, ar->count - 1);
+		if (ar->count > 0 && pkt->tsig_rr->rrs.data != last_rr->rrs.data) {
 			return KNOT_EMALF;
 		}
 	}
 
 	/* Check for trailing garbage. */
 	if (pkt->parsed < pkt->size) {
-		dbg_packet("%s: %zu bytes of trailing garbage\n",
-		           __func__, pkt->size - pkt->parsed);
 		return KNOT_EMALF;
 	}
 
 	return KNOT_EOK;
 }
 
+_public_
 uint16_t knot_pkt_get_ext_rcode(const knot_pkt_t *pkt)
 {
 	if (pkt == NULL) {
