@@ -21,15 +21,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "libknot/common.h"
+#include "libknot/packet/rrset-wire.h"
 #include "libknot/consts.h"
 #include "libknot/descriptor.h"
 #include "libknot/dname.h"
 #include "libknot/packet/pkt.h"
-#include "libknot/packet/rrset-wire.h"
 #include "libknot/packet/wire.h"
 #include "libknot/rrset.h"
 #include "libknot/rrtype/naptr.h"
+#include "libknot/internal/macros.h"
+
+#define RR_HEADER_SIZE 10
 
 /*!
  * \brief Get maximal size of a domain name in a wire with given capacity.
@@ -168,7 +170,7 @@ static int write_rdata_block(const uint8_t **src, size_t *src_avail,
  */
 static int rdata_traverse(const uint8_t **src, size_t *src_avail,
                           uint8_t **dst, size_t *dst_avail,
-                          const rdata_descriptor_t *desc,
+                          const knot_rdata_descriptor_t *desc,
                           dname_config_t *dname_cfg)
 {
 	for (int i = 0; desc->block_types[i] != KNOT_RDATA_WF_END; i++) {
@@ -183,6 +185,93 @@ static int rdata_traverse(const uint8_t **src, size_t *src_avail,
 	return KNOT_EOK;
 }
 
+/*!
+ * \brief Compute total length of RDATA blocks.
+ */
+static int rdata_len_block(const uint8_t **src, size_t *src_avail,
+                           const uint8_t *pkt_wire, int block_type)
+{
+	int ret, compr_size;
+
+	switch (block_type) {
+	case KNOT_RDATA_WF_COMPRESSIBLE_DNAME:
+	case KNOT_RDATA_WF_DECOMPRESSIBLE_DNAME:
+	case KNOT_RDATA_WF_FIXED_DNAME:
+		compr_size = knot_dname_wire_check(*src, *src + *src_avail,
+		                                   pkt_wire);
+		if (compr_size < 0) {
+			return compr_size;
+		}
+		if (compr_size == 0) {
+			return KNOT_EMALF;
+		}
+
+		ret = knot_dname_realsize(*src, pkt_wire);
+		if (ret < 0) {
+			return ret;
+		}
+
+		*src += compr_size;
+		*src_avail -= compr_size;
+		break;
+	case KNOT_RDATA_WF_NAPTR_HEADER:
+		ret = knot_naptr_header_size(*src, *src + *src_avail);
+		if (ret < 0) {
+			return ret;
+		}
+
+		*src += ret;
+		*src_avail -= ret;
+		break;
+	case KNOT_RDATA_WF_REMAINDER:
+		ret = *src_avail;
+		*src += ret;
+		*src_avail -= ret;
+		break;
+	default:
+		/* Fixed size block */
+		assert(block_type > 0);
+		ret = block_type;
+		if (*src_avail < ret) {
+			return KNOT_EMALF;
+		}
+
+		*src += ret;
+		*src_avail -= ret;
+		break;
+	}
+
+	return ret;
+}
+
+/*!
+ * \brief Compute total length of RDATA blocks.
+ */
+static int rdata_len(const uint8_t **src, size_t *src_avail,
+                     const uint8_t *pkt_wire,
+                     const knot_rdata_descriptor_t *desc)
+{
+	int _len = 0;
+	const uint8_t *_src = *src;
+	size_t _src_avail = *src_avail;
+
+	for (int i = 0; desc->block_types[i] != KNOT_RDATA_WF_END; i++) {
+		int block_type = desc->block_types[i];
+		int ret = rdata_len_block(&_src, &_src_avail, pkt_wire, block_type);
+		if (ret < 0) {
+			return ret;
+		}
+		_len += ret;
+	}
+
+	if (_src_avail > 0) {
+		/* Trailing data in message. */
+		return KNOT_EMALF;
+	}
+
+	return _len;
+}
+
 /*- RRSet to wire -----------------------------------------------------------*/
 
 /*!
@@ -195,12 +284,7 @@ static int write_owner(const knot_rrset_t *rrset, uint8_t **dst, size_t *dst_ava
 	assert(dst && *dst);
 	assert(dst_avail);
 
-	/* Check for zero label owner (don't compress). */
-
-	uint16_t owner_pointer = 0;
-	if (*rrset->owner != '\0') {
-		owner_pointer = compr_get_ptr(compr, COMPR_HINT_OWNER);
-	}
+	uint16_t owner_pointer = compr_get_ptr(compr, KNOT_COMPR_HINT_OWNER);
 
 	/* Check size */
 
@@ -226,7 +310,7 @@ static int write_owner(const knot_rrset_t *rrset, uint8_t **dst, size_t *dst_ava
 			return written;
 		}
 
-		compr_set_ptr(compr, COMPR_HINT_OWNER, *dst, written);
+		compr_set_ptr(compr, KNOT_COMPR_HINT_OWNER, *dst, written);
 		owner_size = written;
 	}
 
@@ -264,11 +348,11 @@ static int write_fixed_header(const knot_rrset_t *rrset, uint16_t rrset_index,
 	uint32_t ttl = knot_rdata_ttl(knot_rdataset_at(&rrset->rrs, rrset_index));
 	uint8_t *write = *dst;
 
-	knot_wire_write_u16(write, rrset->type);
+	wire_write_u16(write, rrset->type);
 	write += sizeof(uint16_t);
-	knot_wire_write_u16(write, rrset->rclass);
+	wire_write_u16(write, rrset->rclass);
 	write += sizeof(uint16_t);
-	knot_wire_write_u32(write, ttl);
+	wire_write_u32(write, ttl);
 	write += sizeof(uint32_t);
 
 	assert(write == *dst + size);
@@ -309,6 +393,7 @@ static int compress_rdata_dname(const uint8_t **src, size_t *src_avail,
 	int written = knot_compr_put_dname(dname, *dst, dname_max(*dst_avail),
 	                                   put_compr);
 	if (written < 0) {
+		assert(written == KNOT_ESPACE);
 		return written;
 	}
 
@@ -358,14 +443,14 @@ static int write_rdata(const knot_rrset_t *rrset, uint16_t rrset_index,
 	dname_config_t dname_cfg = {
 		.write_cb = compress_rdata_dname,
 		.compr = compr,
-		.hint = COMPR_HINT_RDATA + rrset_index
+		.hint = KNOT_COMPR_HINT_RDATA + rrset_index
 	};
 
 	const uint8_t *src = knot_rdata_data(rdata);
 	size_t src_avail = knot_rdata_rdlen(rdata);
 	if (src_avail > 0) {
 		/* Only write non-empty data. */
-		const rdata_descriptor_t *desc =
+		const knot_rdata_descriptor_t *desc =
 			knot_get_rdata_descriptor(rrset->type);
 		int ret = rdata_traverse(&src, &src_avail, dst, dst_avail,
 		                         desc, &dname_cfg);
@@ -382,7 +467,7 @@ static int write_rdata(const knot_rrset_t *rrset, uint16_t rrset_index,
 	/* Write final RDLENGTH */
 
 	size_t rdlength = *dst - wire_rdata_begin;
-	knot_wire_write_u16(wire_rdlength, rdlength);
+	wire_write_u16(wire_rdlength, rdlength);
 
 	return KNOT_EOK;
 }
@@ -411,6 +496,7 @@ static int write_rr(const knot_rrset_t *rrset, uint16_t rrset_index,
 /*!
  * \brief Write RR Set content to a wire.
  */
+_public_
 int knot_rrset_to_wire(const knot_rrset_t *rrset, uint8_t *wire, uint16_t max_size,
                        knot_compr_t *compr)
 {
@@ -428,14 +514,13 @@ int knot_rrset_to_wire(const knot_rrset_t *rrset, uint8_t *wire, uint16_t max_si
 		}
 	}
 
-	return write - wire;
+	size_t written = write - wire;
+
+	return written;
 }
 
 /*- RRSet from wire ---------------------------------------------------------*/
 
-#define RR_HEADER_SIZE 10
-#define MAX_RDLENGTH 65535
-
 /*!
  * \brief Parse header of one RR from packet wireformat.
  */
@@ -459,13 +544,13 @@ static int parse_header(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
 		return KNOT_EMALF;
 	}
 
-	uint16_t type = knot_wire_read_u16(pkt_wire + *pos);
+	uint16_t type = wire_read_u16(pkt_wire + *pos);
 	*pos += sizeof(uint16_t);
-	uint16_t rclass = knot_wire_read_u16(pkt_wire + *pos);
+	uint16_t rclass = wire_read_u16(pkt_wire + *pos);
 	*pos += sizeof(uint16_t);
-	*ttl = knot_wire_read_u32(pkt_wire + *pos);
+	*ttl = wire_read_u32(pkt_wire + *pos);
 	*pos += sizeof(uint32_t);
-	*rdlen = knot_wire_read_u16(pkt_wire + *pos);
+	*rdlen = wire_read_u16(pkt_wire + *pos);
 	*pos += sizeof(uint16_t);
 
 	if (pkt_size - *pos < *rdlen) {
@@ -497,13 +582,13 @@ static int decompress_rdata_dname(const uint8_t **src, size_t *src_avail,
 	if (compr_size <= 0) {
 		return compr_size;
 	}
-	
+
 	int decompr_size = knot_dname_unpack(*dst, *src, *dst_avail,
 	                                     dname_cfg->pkt_wire);
 	if (decompr_size <= 0) {
 		return decompr_size;
 	}
-	
+
 	/* Update buffers */
 	*dst += decompr_size;
 	*dst_avail -= decompr_size;
@@ -514,7 +599,8 @@ static int decompress_rdata_dname(const uint8_t **src, size_t *src_avail,
 	return KNOT_EOK;
 }
 
-static bool allow_zero_rdata(const knot_rrset_t *rr, const rdata_descriptor_t *desc)
+static bool allow_zero_rdata(const knot_rrset_t *rr,
+                             const knot_rdata_descriptor_t *desc)
 {
 	return rr->rclass != KNOT_CLASS_IN ||  // NONE and ANY for DDNS
 	       rr->type == KNOT_RRTYPE_APL ||  // APL RR type
@@ -536,7 +622,7 @@ static int parse_rdata(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
 		return KNOT_EMALF;
 	}
 
-	const rdata_descriptor_t *desc = knot_get_rdata_descriptor(rrset->type);
+	const knot_rdata_descriptor_t *desc = knot_get_rdata_descriptor(rrset->type);
 	if (desc->type_name == NULL) {
 		desc = knot_get_obsolete_rdata_descriptor(rrset->type);
 	}
@@ -554,9 +640,26 @@ static int parse_rdata(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
 	const uint8_t *src = pkt_wire + *pos;
 	size_t src_avail = rdlength;
 
-	const size_t buffer_size = rdlength + KNOT_MAX_RDATA_DNAMES * KNOT_DNAME_MAXLEN;
-	uint8_t rdata_buffer[buffer_size];
-	uint8_t *dst = rdata_buffer;
+	int buffer_size = rdata_len(&src, &src_avail, pkt_wire, desc);
+	if (buffer_size < 0) {
+		return buffer_size;
+	}
+	
+	if (buffer_size > MAX_RDLENGTH) {
+		/* DNAME compression caused RDATA overflow. */
+		return KNOT_EMALF;
+	}
+
+	knot_rdataset_t *rrs = &rrset->rrs;
+	int ret = knot_rdataset_reserve(rrs, buffer_size, mm);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	knot_rdata_t *rr = knot_rdataset_at(rrs, rrs->rr_count - 1);
+	assert(rr);
+	knot_rdata_set_ttl(rr, ttl);
+	uint8_t *dst = knot_rdata_data(rr);
 	size_t dst_avail = buffer_size;
 
 	/* Parse RDATA */
@@ -566,36 +669,28 @@ static int parse_rdata(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
 		.pkt_wire = pkt_wire
 	};
 
-	int ret = rdata_traverse(&src, &src_avail, &dst, &dst_avail, desc, &dname_cfg);
+	ret = rdata_traverse(&src, &src_avail, &dst, &dst_avail, desc, &dname_cfg);
 	if (ret != KNOT_EOK) {
+		knot_rdataset_unreserve(rrs, mm);
 		return ret;
 	}
 
-	if (src_avail > 0) {
-		/* Trailing data in message. */
-		return KNOT_EMALF;
+	ret = knot_rdataset_sort_at(rrs, rrs->rr_count - 1, mm);
+	if (ret != KNOT_EOK) {
+		knot_rdataset_unreserve(rrs, mm);
+		return ret;
 	}
+	
+	/* Update position pointer. */
+	*pos += rdlength;
 
-	const size_t written = buffer_size - dst_avail;
-	if (written > MAX_RDLENGTH) {
-		/* DNAME compression caused RDATA overflow. */
-		return KNOT_EMALF;
-	}
-	
-	ret = knot_rrset_add_rdata(rrset, rdata_buffer, written, ttl, mm);
-	if (ret == KNOT_EOK) {
-		/* Update position pointer. */
-		*pos += rdlength;
-	}
-	
-	return ret;
+	return KNOT_EOK;
 }
 
-/*!
-* \brief Creates one RR from a wire.
- */
+_public_
 int knot_rrset_rr_from_wire(const uint8_t *pkt_wire, size_t *pos,
-                            size_t pkt_size, mm_ctx_t *mm, knot_rrset_t *rrset)
+                            size_t pkt_size, mm_ctx_t *mm,
+                            knot_rrset_t *rrset)
 {
 	if (!pkt_wire || !pos || !rrset || *pos > pkt_size) {
 		return KNOT_EINVAL;
