@@ -20,7 +20,8 @@
 #include <stdlib.h>
 
 #include "libknot/rdataset.h"
-#include "libknot/common.h"
+#include "libknot/errcode.h"
+#include "libknot/internal/macros.h"
 
 static knot_rdata_t *rr_seek(knot_rdata_t *d, size_t pos)
 {
@@ -69,7 +70,6 @@ static int add_rr_at(knot_rdataset_t *rrs, const knot_rdata_t *rr, size_t pos,
 	if (tmp) {
 		rrs->data = tmp;
 	} else {
-		ERR_ALLOC_FAILED;
 		return KNOT_ENOMEM;
 	}
 
@@ -112,8 +112,8 @@ static int remove_rr_at(knot_rdataset_t *rrs, size_t pos, mm_ctx_t *mm)
 	size_t total_size = knot_rdataset_size(rrs);
 	uint16_t old_size = knot_rdata_rdlen(old_rr);
 
-	void *old_threshold = old_rr + knot_rdata_array_size(old_size);
-	void *last_threshold = last_rr + knot_rdata_array_size(knot_rdata_rdlen(last_rr));
+	uint8_t *old_threshold = old_rr + knot_rdata_array_size(old_size);
+	uint8_t *last_threshold = last_rr + knot_rdata_array_size(knot_rdata_rdlen(last_rr));
 	// Move RDATA
 	memmove(old_rr, old_threshold,
 	        last_threshold - old_threshold);
@@ -124,7 +124,6 @@ static int remove_rr_at(knot_rdataset_t *rrs, size_t pos, mm_ctx_t *mm)
 		                       total_size - (knot_rdata_array_size(old_size)),
 		                       total_size);
 		if (tmp == NULL) {
-			ERR_ALLOC_FAILED;
 			return KNOT_ENOMEM;
 		} else {
 			rrs->data = tmp;
@@ -139,6 +138,7 @@ static int remove_rr_at(knot_rdataset_t *rrs, size_t pos, mm_ctx_t *mm)
 	return KNOT_EOK;
 }
 
+_public_
 void knot_rdataset_init(knot_rdataset_t *rrs)
 {
 	if (rrs) {
@@ -147,6 +147,7 @@ void knot_rdataset_init(knot_rdataset_t *rrs)
 	}
 }
 
+_public_
 void knot_rdataset_clear(knot_rdataset_t *rrs, mm_ctx_t *mm)
 {
 	if (rrs) {
@@ -155,6 +156,7 @@ void knot_rdataset_clear(knot_rdataset_t *rrs, mm_ctx_t *mm)
 	}
 }
 
+_public_
 int knot_rdataset_copy(knot_rdataset_t *dst, const knot_rdataset_t *src, mm_ctx_t *mm)
 {
 	if (dst == NULL || src == NULL) {
@@ -165,7 +167,6 @@ int knot_rdataset_copy(knot_rdataset_t *dst, const knot_rdataset_t *src, mm_ctx_
 	size_t src_size = knot_rdataset_size(src);
 	dst->data = mm_alloc(mm, src_size);
 	if (dst->data == NULL) {
-		ERR_ALLOC_FAILED;
 		return KNOT_ENOMEM;
 	}
 
@@ -173,6 +174,7 @@ int knot_rdataset_copy(knot_rdataset_t *dst, const knot_rdataset_t *src, mm_ctx_
 	return KNOT_EOK;
 }
 
+_public_
 knot_rdata_t *knot_rdataset_at(const knot_rdataset_t *rrs, size_t pos)
 {
 	if (rrs == NULL || pos >= rrs->rr_count) {
@@ -182,6 +184,7 @@ knot_rdata_t *knot_rdataset_at(const knot_rdataset_t *rrs, size_t pos)
 	return rr_seek(rrs->data, pos);
 }
 
+_public_
 size_t knot_rdataset_size(const knot_rdataset_t *rrs)
 {
 	if (rrs == NULL) {
@@ -198,6 +201,7 @@ size_t knot_rdataset_size(const knot_rdataset_t *rrs)
 	return total_size;
 }
 
+_public_
 int knot_rdataset_add(knot_rdataset_t *rrs, const knot_rdata_t *rr, mm_ctx_t *mm)
 {
 	if (rrs == NULL || rr == NULL) {
@@ -220,6 +224,39 @@ int knot_rdataset_add(knot_rdataset_t *rrs, const knot_rdata_t *rr, mm_ctx_t *mm
 	return add_rr_at(rrs, rr, rrs->rr_count, mm);
 }
 
+_public_
+int knot_rdataset_reserve(knot_rdataset_t *rrs, size_t size, mm_ctx_t *mm)
+{
+	if (rrs == NULL || size > MAX_RDLENGTH) {
+		return KNOT_EINVAL;
+	}
+
+	size_t total_size = knot_rdataset_size(rrs);
+	size_t new_size = total_size + knot_rdata_array_size(size);
+
+	uint8_t *tmp = mm_realloc(mm, rrs->data, new_size, total_size);
+	if (tmp == NULL) {
+		return KNOT_ENOMEM;
+	}
+
+	rrs->data = tmp;
+	rrs->rr_count++;
+
+	// We have to initialise the 'size' field in the reserved space.
+	knot_rdata_t *rr = knot_rdataset_at(rrs, rrs->rr_count - 1);
+	assert(rr);
+	knot_rdata_set_rdlen(rr, size);
+
+	return KNOT_EOK;
+}
+
+_public_
+int knot_rdataset_unreserve(knot_rdataset_t *rrs, mm_ctx_t *mm)
+{
+	return remove_rr_at(rrs, rrs->rr_count - 1, mm);
+}
+
+_public_
 bool knot_rdataset_eq(const knot_rdataset_t *rrs1, const knot_rdataset_t *rrs2)
 {
 	if (rrs1->rr_count != rrs2->rr_count) {
@@ -237,7 +274,9 @@ bool knot_rdataset_eq(const knot_rdataset_t *rrs1, const knot_rdataset_t *rrs2)
 	return true;
 }
 
-bool knot_rdataset_member(const knot_rdataset_t *rrs, const knot_rdata_t *rr, bool cmp_ttl)
+_public_
+bool knot_rdataset_member(const knot_rdataset_t *rrs, const knot_rdata_t *rr,
+			  bool cmp_ttl)
 {
 	for (uint16_t i = 0; i < rrs->rr_count; ++i) {
 		const knot_rdata_t *cmp_rr = knot_rdataset_at(rrs, i);
@@ -259,6 +298,7 @@ bool knot_rdataset_member(const knot_rdataset_t *rrs, const knot_rdata_t *rr, bo
 	return false;
 }
 
+_public_
 int knot_rdataset_merge(knot_rdataset_t *rrs1, const knot_rdataset_t *rrs2, mm_ctx_t *mm)
 {
 	if (rrs1 == NULL || rrs2 == NULL) {
@@ -276,6 +316,7 @@ int knot_rdataset_merge(knot_rdataset_t *rrs1, const knot_rdataset_t *rrs2, mm_c
 	return KNOT_EOK;
 }
 
+_public_
 int knot_rdataset_intersect(const knot_rdataset_t *a, const knot_rdataset_t *b,
                        knot_rdataset_t *out, mm_ctx_t *mm)
 {
@@ -300,7 +341,9 @@ int knot_rdataset_intersect(const knot_rdataset_t *a, const knot_rdataset_t *b,
 	return KNOT_EOK;
 }
 
-int knot_rdataset_subtract(knot_rdataset_t *from, const knot_rdataset_t *what, mm_ctx_t *mm)
+_public_
+int knot_rdataset_subtract(knot_rdataset_t *from, const knot_rdataset_t *what,
+			   mm_ctx_t *mm)
 {
 	if (from == NULL || what == NULL || from->data == what->data) {
 		return KNOT_EINVAL;
@@ -319,3 +362,54 @@ int knot_rdataset_subtract(knot_rdataset_t *from, const knot_rdataset_t *what, m
 
 	return KNOT_EOK;
 }
+
+_public_
+int knot_rdataset_sort_at(knot_rdataset_t *rrs, size_t pos, mm_ctx_t *mm)
+{
+	if (rrs == NULL || rrs->rr_count == 0) {
+		return KNOT_EINVAL;
+	}
+
+	knot_rdata_t *rr = knot_rdataset_at(rrs, pos);
+	assert(rr);
+
+	knot_rdata_t *earlier_rr = NULL;
+	for (uint16_t i = 0; i < rrs->rr_count; ++i) {
+		if (i == pos) {
+			// It already is at the position
+			return KNOT_EOK;
+		}
+		earlier_rr = knot_rdataset_at(rrs, i);
+		int cmp = knot_rdata_cmp(earlier_rr, rr);
+		if (cmp == 0) {
+			// Duplication - we need to remove this RR
+			return remove_rr_at(rrs, pos, mm);
+		} else if (cmp > 0) {
+			// Found position to move
+			break;
+		}
+	}
+
+	// RDATA have to be rearanged.
+	knot_rdata_t *last_rr = knot_rdataset_at(rrs, pos - 1);
+	assert(last_rr);
+	assert(earlier_rr);
+
+	// Save the RR to be moved
+	const uint16_t size = knot_rdata_rdlen(rr);
+	const uint32_t ttl = knot_rdata_ttl(rr);
+	const uint8_t *rdata = knot_rdata_data(rr);
+
+	knot_rdata_t tmp_rr[knot_rdata_array_size(size)];
+	knot_rdata_init(tmp_rr, size, rdata, ttl);
+
+	// Move the array or just part of it
+	knot_rdata_t *earlier_rr_moved = earlier_rr + knot_rdata_array_size(size);
+	size_t last_rr_size = knot_rdata_array_size(knot_rdata_rdlen(last_rr));
+	memmove(earlier_rr_moved, earlier_rr, (last_rr + last_rr_size) - earlier_rr);
+
+	// Set new RR
+	knot_rdata_init(earlier_rr, size, knot_rdata_data(tmp_rr), ttl);
+
+	return KNOT_EOK;
+}
