@@ -30,9 +30,9 @@
 #include <stdint.h>
 #include <stdbool.h>
 
-#include "libknot/mempattern.h"
 #include "libknot/dname.h"
 #include "libknot/rdataset.h"
+#include "libknot/internal/mempattern.h"
 
 /*!
  * \brief Structure for representing RRSet.
@@ -50,6 +50,7 @@ struct knot_rrset {
 
 typedef struct knot_rrset knot_rrset_t;
 
+/*! \todo Documentation */
 typedef enum {
 	KNOT_RRSET_COMPARE_PTR,
 	KNOT_RRSET_COMPARE_HEADER,
@@ -129,7 +130,7 @@ void knot_rrset_clear(knot_rrset_t *rrset, mm_ctx_t *mm);
  * \param rrset  RRSet to add the RDATA to.
  * \param rdata  RDATA to add to the RRSet.
  * \param size   Size of RDATA.
- * \param ttl   TTL for RR.
+ * \param ttl    TTL for RR.
  * \param mm     Memory context.
  *
  * \return KNOT_E*
