@@ -17,13 +17,13 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <assert.h>
-
 #include <string.h>
+
 #include "libknot/rrtype/opt.h"
-#include "libknot/common.h"
 #include "libknot/descriptor.h"
-#include "common/debug.h"
-#include "common-knot/sockaddr.h"
+#include "libknot/errcode.h"
+#include "libknot/internal/macros.h"
+#include "libknot/internal/sockaddr.h"
 
 /*! \brief Some implementation-related constants. */
 enum knot_edns_private_consts {
@@ -46,7 +46,7 @@ enum knot_edns_private_consts {
 };
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 int knot_edns_init(knot_rrset_t *opt_rr, uint16_t max_pld,
                   uint8_t ext_rcode, uint8_t ver, mm_ctx_t *mm)
 {
@@ -72,9 +72,8 @@ int knot_edns_init(knot_rrset_t *opt_rr, uint16_t max_pld,
 	return ret;
 }
 
-
 /*----------------------------------------------------------------------------*/
-
+_public_
 size_t knot_edns_wire_size(knot_rrset_t *opt_rr)
 {
 	if (opt_rr == NULL) {
@@ -88,7 +87,7 @@ size_t knot_edns_wire_size(knot_rrset_t *opt_rr)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 uint16_t knot_edns_get_payload(const knot_rrset_t *opt_rr)
 {
 	assert(opt_rr != NULL);
@@ -96,7 +95,7 @@ uint16_t knot_edns_get_payload(const knot_rrset_t *opt_rr)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 void knot_edns_set_payload(knot_rrset_t *opt_rr, uint16_t payload)
 {
 	assert(opt_rr != NULL);
@@ -104,7 +103,7 @@ void knot_edns_set_payload(knot_rrset_t *opt_rr, uint16_t payload)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 uint8_t knot_edns_get_ext_rcode(const knot_rrset_t *opt_rr)
 {
 	assert(opt_rr != NULL);
@@ -112,7 +111,7 @@ uint8_t knot_edns_get_ext_rcode(const knot_rrset_t *opt_rr)
 	uint32_t ttl = 0;
 	uint8_t *ttl_ptr = (uint8_t *)&ttl;
 	// TTL is stored in machine byte order. Convert it to wire order first.
-	knot_wire_write_u32(ttl_ptr, knot_rrset_ttl(opt_rr));
+	wire_write_u32(ttl_ptr, knot_rrset_ttl(opt_rr));
 
 	uint8_t rcode;
 	memcpy(&rcode, ttl_ptr + EDNS_OFFSET_RCODE, sizeof(uint8_t));
@@ -128,17 +127,17 @@ static void set_value_to_ttl(knot_rrset_t *opt_rr, size_t offset, uint8_t value)
 	uint8_t *ttl_ptr = (uint8_t *)&ttl;
 
 	// TTL is stored in machine byte order. Convert it to wire order first.
-	knot_wire_write_u32(ttl_ptr, knot_rrset_ttl(opt_rr));
+	wire_write_u32(ttl_ptr, knot_rrset_ttl(opt_rr));
 	// Set the Extended RCODE in the converted TTL
 	memcpy(ttl_ptr + offset, &value, sizeof(uint8_t));
 	// Convert it back to machine byte order
-	uint32_t ttl_local = knot_wire_read_u32(ttl_ptr);
+	uint32_t ttl_local = wire_read_u32(ttl_ptr);
 	// Store the TTL to the RDATA
 	knot_rdata_set_ttl(knot_rdataset_at(&opt_rr->rrs, 0), ttl_local);
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 void knot_edns_set_ext_rcode(knot_rrset_t *opt_rr, uint8_t ext_rcode)
 {
 	assert(opt_rr != NULL);
@@ -146,7 +145,7 @@ void knot_edns_set_ext_rcode(knot_rrset_t *opt_rr, uint8_t ext_rcode)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 uint8_t knot_edns_get_version(const knot_rrset_t *opt_rr)
 {
 	assert(opt_rr != NULL);
@@ -154,7 +153,7 @@ uint8_t knot_edns_get_version(const knot_rrset_t *opt_rr)
 	uint32_t ttl = 0;
 	uint8_t *ttl_ptr = (uint8_t *)&ttl;
 	// TTL is stored in machine byte order. Convert it to wire order first.
-	knot_wire_write_u32(ttl_ptr, knot_rrset_ttl(opt_rr));
+	wire_write_u32(ttl_ptr, knot_rrset_ttl(opt_rr));
 
 	uint8_t version;
 	memcpy(&version, ttl_ptr + EDNS_OFFSET_VERSION, sizeof(uint8_t));
@@ -163,7 +162,7 @@ uint8_t knot_edns_get_version(const knot_rrset_t *opt_rr)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 void knot_edns_set_version(knot_rrset_t *opt_rr, uint8_t version)
 {
 	assert(opt_rr != NULL);
@@ -171,7 +170,7 @@ void knot_edns_set_version(knot_rrset_t *opt_rr, uint8_t version)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 bool knot_edns_do(const knot_rrset_t *opt_rr)
 {
 	assert(opt_rr != NULL);
@@ -180,7 +179,7 @@ bool knot_edns_do(const knot_rrset_t *opt_rr)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 void knot_edns_set_do(knot_rrset_t *opt_rr)
 {
 	assert(opt_rr != NULL);
@@ -212,12 +211,12 @@ static uint8_t *find_option(knot_rdata_t *rdata, uint16_t opt_code)
 
 	int i = 0;
 	while (i + KNOT_EDNS_OPTION_HDRLEN <= rdlength) {
-		uint16_t code = knot_wire_read_u16(data + i);
+		uint16_t code = wire_read_u16(data + i);
 		if (opt_code == code) {
 			pos = data + i;
 			break;
 		}
-		uint16_t opt_len = knot_wire_read_u16(data + i
+		uint16_t opt_len = wire_read_u16(data + i
 		                                      + sizeof(uint16_t));
 		i += (KNOT_EDNS_OPTION_HDRLEN + opt_len);
 	}
@@ -226,7 +225,7 @@ static uint8_t *find_option(knot_rdata_t *rdata, uint16_t opt_code)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 int knot_edns_add_option(knot_rrset_t *opt_rr, uint16_t code,
                          uint16_t length, const uint8_t *data, mm_ctx_t *mm)
 {
@@ -248,14 +247,10 @@ int knot_edns_add_option(knot_rrset_t *opt_rr, uint16_t code,
 
 	uint8_t new_data[new_data_len];
 
-	dbg_edns_verb("EDNS: Adding option. Code: %u, length: %u, data:\n",
-	              code, length);
-	dbg_edns_hex_verb((char *)data, length);
-
 	memcpy(new_data, old_data, old_data_len);
 	// write length and code in wireformat (convert endian)
-	knot_wire_write_u16(new_data + old_data_len, code);
-	knot_wire_write_u16(new_data + old_data_len + sizeof(uint16_t), length);
+	wire_write_u16(new_data + old_data_len, code);
+	wire_write_u16(new_data + old_data_len + sizeof(uint16_t), length);
 	// write the option data
 	memcpy(new_data + old_data_len + KNOT_EDNS_OPTION_HDRLEN, data, length);
 
@@ -267,7 +262,7 @@ int knot_edns_add_option(knot_rrset_t *opt_rr, uint16_t code,
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 bool knot_edns_has_option(const knot_rrset_t *opt_rr, uint16_t code)
 {
 	assert(opt_rr != NULL);
@@ -281,14 +276,14 @@ bool knot_edns_has_option(const knot_rrset_t *opt_rr, uint16_t code)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 bool knot_edns_has_nsid(const knot_rrset_t *opt_rr)
 {
 	return knot_edns_has_option(opt_rr, KNOT_EDNS_OPTION_NSID);
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 bool knot_edns_check_record(knot_rrset_t *opt_rr)
 {
 	if (opt_rr->rrs.rr_count != 1) {
@@ -306,7 +301,7 @@ bool knot_edns_check_record(knot_rrset_t *opt_rr)
 
 	/* RFC2671 4.4: {uint16_t code, uint16_t len, data} */
 	while (pos + KNOT_EDNS_OPTION_HDRLEN <= rdlength) {
-		uint16_t opt_len = knot_wire_read_u16(data + pos
+		uint16_t opt_len = wire_read_u16(data + pos
 		                                      + sizeof(uint16_t));
 		pos += KNOT_EDNS_OPTION_HDRLEN + opt_len;
 	}
@@ -318,7 +313,7 @@ bool knot_edns_check_record(knot_rrset_t *opt_rr)
 }
 
 /*----------------------------------------------------------------------------*/
-
+_public_
 int knot_edns_client_subnet_create(const knot_addr_family_t family,
                                    const uint8_t *addr,
                                    const uint16_t addr_len,
@@ -343,7 +338,7 @@ int knot_edns_client_subnet_create(const knot_addr_family_t family,
 		return KNOT_EINVAL;
 	}
 
-	knot_wire_write_u16(data + EDNS_OFFSET_CLIENT_SUBNET_FAMILY, family);
+	wire_write_u16(data + EDNS_OFFSET_CLIENT_SUBNET_FAMILY, family);
 	data[EDNS_OFFSET_CLIENT_SUBNET_SRC_MASK] = src_mask;
 	data[EDNS_OFFSET_CLIENT_SUBNET_DST_MASK] = dst_mask;
 	memcpy(data + EDNS_OFFSET_CLIENT_SUBNET_ADDR, addr, addr_prefix_len);
@@ -359,6 +354,7 @@ int knot_edns_client_subnet_create(const knot_addr_family_t family,
 	return KNOT_EOK;
 }
 
+_public_
 int knot_edns_client_subnet_parse(const uint8_t *data,
                                   const uint16_t data_len,
                                   knot_addr_family_t *family,
@@ -377,7 +373,7 @@ int knot_edns_client_subnet_parse(const uint8_t *data,
 		return KNOT_ESPACE;
 	}
 
-	*family = knot_wire_read_u16(data + EDNS_OFFSET_CLIENT_SUBNET_FAMILY);
+	*family = wire_read_u16(data + EDNS_OFFSET_CLIENT_SUBNET_FAMILY);
 	*src_mask = data[EDNS_OFFSET_CLIENT_SUBNET_SRC_MASK];
 	*dst_mask = data[EDNS_OFFSET_CLIENT_SUBNET_DST_MASK];
 	memcpy(addr, data + EDNS_OFFSET_CLIENT_SUBNET_ADDR, rest);
