@@ -14,22 +14,21 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <assert.h>
+#include <inttypes.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdint.h>
-#include <inttypes.h>
-#include <assert.h>
 #include <time.h>
 
+#include "dnssec/tsig.h"
 #include "libknot/rrtype/tsig.h"
-
-#include "common/debug.h"
-#include "common/log.h"
-#include "libknot/common.h"
-#include "libknot/util/utils.h"
-#include "libknot/rrset.h"
-#include "libknot/dname.h"
 #include "libknot/consts.h"
+#include "libknot/dname.h"
+#include "libknot/errcode.h"
+#include "libknot/rrset.h"
+#include "libknot/internal/macros.h"
+#include "libknot/internal/utils.h"
 
 /*! \brief TSIG field offsets. */
 typedef enum tsig_off_t {
@@ -57,21 +56,20 @@ typedef enum tsig_off_t {
  * \param nb Required number of bytes after the offset (for boundaries check).
  * \return pointer to field on wire or NULL.
  */
-static uint8_t* tsig_rdata_seek(const knot_rrset_t *rr, tsig_off_t id, size_t nb)
+static uint8_t* rdata_seek(const knot_rrset_t *rr, tsig_off_t id, size_t nb)
 {
 	const knot_rdata_t *rr_data = knot_rdataset_at(&rr->rrs, 0);
-	uint8_t *rd = knot_rdata_data(rr_data);
-	if (rd == NULL) {
+	if (!rr_data || knot_rdata_rdlen(rr_data) == 0) {
 		return NULL;
 	}
 
+	uint8_t *rd = knot_rdata_data(rr_data);
+	assert(rd);
+
 	/* TSIG RR names should be already sanitized on parse. */
 	int alg_len = knot_dname_size(rd);
 	uint16_t lim = knot_rdata_rdlen(rr_data);
 	if (lim < alg_len + 5 * sizeof(uint16_t)) {
-		dbg_tsig("TSIG: rdata: not enough items "
-		         "(has %"PRIu16", min %zu).\n",
-		         lim, alg_len + 5 * sizeof(uint16_t));
 		return NULL;
 	}
 
@@ -85,45 +83,45 @@ static uint8_t* tsig_rdata_seek(const knot_rrset_t *rr, tsig_off_t id, size_t nb
 	case TSIG_MAC_O: rd += alg_len + 5 * sizeof(uint16_t); break;
 	case TSIG_ORIGID_O:
 		rd += alg_len + 4 * sizeof(uint16_t);
-		rd += knot_wire_read_u16(rd) + sizeof(uint16_t);
+		rd += wire_read_u16(rd) + sizeof(uint16_t);
 		break;
 
 	case TSIG_ERROR_O:
 		rd += alg_len + 4 * sizeof(uint16_t);
-		rd += knot_wire_read_u16(rd) + 2 * sizeof(uint16_t);
+		rd += wire_read_u16(rd) + 2 * sizeof(uint16_t);
 		break;
 	case TSIG_OLEN_O:
 		rd += alg_len + 4 * sizeof(uint16_t);
-		rd += knot_wire_read_u16(rd) + 3 * sizeof(uint16_t);
+		rd += wire_read_u16(rd) + 3 * sizeof(uint16_t);
 		break;
 	case TSIG_OTHER_O:
 		rd += alg_len + 4 * sizeof(uint16_t);
-		rd += knot_wire_read_u16(rd) + 4 * sizeof(uint16_t);
+		rd += wire_read_u16(rd) + 4 * sizeof(uint16_t);
 		break;
 	}
 
 	/* Check remaining bytes. */
 	if (rd + nb > bp + lim) {
-		dbg_tsig("TSIG: rdata: not enough items (needs %zu, has %u).\n",
-		         (rd-bp)+nb, lim);
 		return NULL;
 	}
 
 	return rd;
 }
 
-static int tsig_rdata_set_tsig_error(knot_rrset_t *tsig, uint16_t tsig_error)
+static int rdata_set_tsig_error(knot_rrset_t *tsig, uint16_t tsig_error)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_ERROR_O, sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_ERROR_O, sizeof(uint16_t));
 	if (!rd) {
 		return KNOT_ERROR;
 	}
 
-	knot_wire_write_u16(rd, tsig_error);
+	wire_write_u16(rd, tsig_error);
 	return KNOT_EOK;
 }
 
-int tsig_create_rdata(knot_rrset_t *rr, const knot_dname_t *alg, uint16_t maclen, uint16_t tsig_err)
+_public_
+int knot_tsig_create_rdata(knot_rrset_t *rr, const knot_dname_t *alg,
+                           uint16_t maclen, uint16_t tsig_err)
 {
 	if (rr == NULL || alg == NULL) {
 		return KNOT_EINVAL;
@@ -143,7 +141,7 @@ int tsig_create_rdata(knot_rrset_t *rr, const knot_dname_t *alg, uint16_t maclen
 
 	/* Set MAC variable length in advance. */
 	size_t offset = alg_len + TSIG_OFF_MACLEN;
-	knot_wire_write_u16(rd + offset, maclen);
+	wire_write_u16(rd + offset, maclen);
 
 	int ret = knot_rrset_add_rdata(rr, rd, rdlen, 0, NULL);
 	if (ret != KNOT_EOK) {
@@ -151,36 +149,39 @@ int tsig_create_rdata(knot_rrset_t *rr, const knot_dname_t *alg, uint16_t maclen
 	}
 
 	/* Set error. */
-	tsig_rdata_set_tsig_error(rr, tsig_err);
+	rdata_set_tsig_error(rr, tsig_err);
 
 	return KNOT_EOK;
 }
 
-int tsig_rdata_set_time_signed(knot_rrset_t *tsig, uint64_t time)
+_public_
+int knot_tsig_rdata_set_time_signed(knot_rrset_t *tsig, uint64_t time)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_TSIGNED_O, 3*sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_TSIGNED_O, 3*sizeof(uint16_t));
 	if (!rd) {
 		return KNOT_ERROR;
 	}
 
-	knot_wire_write_u48(rd, time);
+	wire_write_u48(rd, time);
 	return KNOT_EOK;
 }
 
-int tsig_rdata_set_fudge(knot_rrset_t *tsig, uint16_t fudge)
+_public_
+int knot_tsig_rdata_set_fudge(knot_rrset_t *tsig, uint16_t fudge)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_FUDGE_O, sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_FUDGE_O, sizeof(uint16_t));
 	if (!rd) {
 		return KNOT_ERROR;
 	}
 
-	knot_wire_write_u16(rd, fudge);
+	wire_write_u16(rd, fudge);
 	return KNOT_EOK;
 }
 
-int tsig_rdata_set_mac(knot_rrset_t *tsig, uint16_t length, const uint8_t *mac)
+_public_
+int knot_tsig_rdata_set_mac(knot_rrset_t *tsig, uint16_t length, const uint8_t *mac)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_MAC_O, length);
+	uint8_t *rd = rdata_seek(tsig, TSIG_MAC_O, length);
 	if (!rd) {
 		return KNOT_ERROR;
 	}
@@ -192,170 +193,142 @@ int tsig_rdata_set_mac(knot_rrset_t *tsig, uint16_t length, const uint8_t *mac)
 	return KNOT_EOK;
 }
 
-int tsig_rdata_set_orig_id(knot_rrset_t *tsig, uint16_t id)
+_public_
+int knot_tsig_rdata_set_orig_id(knot_rrset_t *tsig, uint16_t id)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_ORIGID_O, sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_ORIGID_O, sizeof(uint16_t));
 	if (!rd) {
 		return KNOT_ERROR;
 	}
 
 	/* Write the length - 2. */
-	knot_wire_write_u16(rd, id);
+	wire_write_u16(rd, id);
 	return KNOT_EOK;
 }
 
-int tsig_rdata_set_other_data(knot_rrset_t *tsig, uint16_t len,
-                              const uint8_t *other_data)
+_public_
+int knot_tsig_rdata_set_other_data(knot_rrset_t *tsig, uint16_t len,
+                                   const uint8_t *other_data)
 {
 	if (len > TSIG_OTHER_MAXLEN) {
-		dbg_tsig("TSIG: rdata: other len > %zu B\n", TSIG_OTHER_MAXLEN);
 		return KNOT_EINVAL;
 	}
 
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_OLEN_O, len+sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_OLEN_O, len+sizeof(uint16_t));
 	if (!rd) {
 		return KNOT_ERROR;
 	}
 
 	/* Write the length. */
-	knot_wire_write_u16(rd, len);
+	wire_write_u16(rd, len);
 
 	/* Copy the actual data. */
 	memcpy(rd + sizeof(uint16_t), other_data, len);
 	return KNOT_EOK;
 }
 
-const knot_dname_t *tsig_rdata_alg_name(const knot_rrset_t *tsig)
+_public_
+const knot_dname_t *knot_tsig_rdata_alg_name(const knot_rrset_t *tsig)
 {
 	const knot_rdata_t *rr_data = knot_rdataset_at(&tsig->rrs, 0);
 	return knot_rdata_data(rr_data);
 }
 
-knot_tsig_algorithm_t tsig_rdata_alg(const knot_rrset_t *tsig)
+_public_
+dnssec_tsig_algorithm_t knot_tsig_rdata_alg(const knot_rrset_t *tsig)
 {
 	/* Get the algorithm name. */
-	const knot_dname_t *alg_name = tsig_rdata_alg_name(tsig);
+	const knot_dname_t *alg_name = knot_tsig_rdata_alg_name(tsig);
 	if (!alg_name) {
-		dbg_tsig("TSIG: rdata: cannot get algorithm name.\n");
-		return KNOT_TSIG_ALG_NULL;
+		return DNSSEC_TSIG_UNKNOWN;
 	}
 
-	/* Convert alg name to string. */
-	char *name = knot_dname_to_str_alloc(alg_name);
-	if (!name) {
-		dbg_tsig("TSIG: rdata: cannot convert alg name.\n");
-		return KNOT_TSIG_ALG_NULL;
-	}
-
-	knot_lookup_table_t *item = knot_lookup_by_name(
-	                                      knot_tsig_alg_dnames_str, name);
-	free(name);
-	if (!item) {
-		dbg_tsig("TSIG: rdata: unknown algorithm.\n");
-		return KNOT_TSIG_ALG_NULL;
-	}
-	return item->id;
+	return dnssec_tsig_algorithm_from_dname(alg_name);
 }
 
-uint64_t tsig_rdata_time_signed(const knot_rrset_t *tsig)
+_public_
+uint64_t knot_tsig_rdata_time_signed(const knot_rrset_t *tsig)
 {
 	/*! \todo How to return invalid value? */
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_TSIGNED_O, 3*sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_TSIGNED_O, 3*sizeof(uint16_t));
 	if (!rd) {
 		return 0;
 	}
-	return knot_wire_read_u48(rd);
+	return wire_read_u48(rd);
 }
 
-uint16_t tsig_rdata_fudge(const knot_rrset_t *tsig)
+_public_
+uint16_t knot_tsig_rdata_fudge(const knot_rrset_t *tsig)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_FUDGE_O, sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_FUDGE_O, sizeof(uint16_t));
 	if (!rd) {
 		return 0;
 	}
-	return knot_wire_read_u16(rd);
+	return wire_read_u16(rd);
 }
 
-const uint8_t *tsig_rdata_mac(const knot_rrset_t *tsig)
+_public_
+const uint8_t *knot_tsig_rdata_mac(const knot_rrset_t *tsig)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_MAC_O, 0);
+	uint8_t *rd = rdata_seek(tsig, TSIG_MAC_O, 0);
 	if (!rd) {
 		return NULL;
 	}
 	return rd;
 }
 
-size_t tsig_rdata_mac_length(const knot_rrset_t *tsig)
+_public_
+size_t knot_tsig_rdata_mac_length(const knot_rrset_t *tsig)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_MACLEN_O, sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_MACLEN_O, sizeof(uint16_t));
 	if (!rd) {
 		return 0;
 	}
-	return knot_wire_read_u16(rd);
+	return wire_read_u16(rd);
 }
 
-uint16_t tsig_rdata_orig_id(const knot_rrset_t *tsig)
+_public_
+uint16_t knot_tsig_rdata_orig_id(const knot_rrset_t *tsig)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_ORIGID_O, sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_ORIGID_O, sizeof(uint16_t));
 	if (!rd) {
 		return KNOT_ERROR;
 	}
-	return knot_wire_read_u16(rd);
+	return wire_read_u16(rd);
 }
 
-uint16_t tsig_rdata_error(const knot_rrset_t *tsig)
+_public_
+uint16_t knot_tsig_rdata_error(const knot_rrset_t *tsig)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_ERROR_O, sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_ERROR_O, sizeof(uint16_t));
 	if (!rd) {
 		return KNOT_ERROR;
 	}
-	return knot_wire_read_u16(rd);
+	return wire_read_u16(rd);
 }
 
-const uint8_t *tsig_rdata_other_data(const knot_rrset_t *tsig)
+_public_
+const uint8_t *knot_tsig_rdata_other_data(const knot_rrset_t *tsig)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_OTHER_O, 0);
+	uint8_t *rd = rdata_seek(tsig, TSIG_OTHER_O, 0);
 	if (!rd) {
 		return NULL;
 	}
 	return rd;
 }
 
-uint16_t tsig_rdata_other_data_length(const knot_rrset_t *tsig)
+_public_
+uint16_t knot_tsig_rdata_other_data_length(const knot_rrset_t *tsig)
 {
-	uint8_t *rd = tsig_rdata_seek(tsig, TSIG_OLEN_O, sizeof(uint16_t));
+	uint8_t *rd = rdata_seek(tsig, TSIG_OLEN_O, sizeof(uint16_t));
 	if (!rd) {
 		return KNOT_ERROR;
 	}
-	return knot_wire_read_u16(rd);
+	return wire_read_u16(rd);
 }
 
-int tsig_alg_from_name(const knot_dname_t *alg_name)
-{
-	if (!alg_name) {
-		return 0;
-	}
-
-	char *name = knot_dname_to_str_alloc(alg_name);
-	if (!name) {
-		return 0;
-	}
-
-	knot_lookup_table_t *found =
-		knot_lookup_by_name(knot_tsig_alg_dnames_str, name);
-
-	if (!found) {
-		dbg_tsig("Unknown algorithm: %s \n", name);
-		free(name);
-		return 0;
-	}
-
-	free(name);
-
-	return found->id;
-}
-
-size_t tsig_rdata_tsig_variables_length(const knot_rrset_t *tsig)
+_public_
+size_t knot_tsig_rdata_tsig_variables_length(const knot_rrset_t *tsig)
 {
 	if (tsig == NULL) {
 		return 0;
@@ -366,67 +339,32 @@ size_t tsig_rdata_tsig_variables_length(const knot_rrset_t *tsig)
 		return 0;
 	}
 
-	const knot_dname_t *alg_name = tsig_rdata_alg_name(tsig);
+	const knot_dname_t *alg_name = knot_tsig_rdata_alg_name(tsig);
 	if (!alg_name) {
 		return 0;
 	}
 
-	uint16_t other_data_length = tsig_rdata_other_data_length(tsig);
+	uint16_t other_data_length = knot_tsig_rdata_other_data_length(tsig);
 
 	return knot_dname_size(key_name) + knot_dname_size(alg_name) +
 	       other_data_length + KNOT_TSIG_VARIABLES_LENGTH;
 }
 
-size_t tsig_rdata_tsig_timers_length()
+_public_
+size_t knot_tsig_rdata_tsig_timers_length()
 {
 	/*! \todo Cleanup */
 	return KNOT_TSIG_TIMERS_LENGTH;
 }
 
-int tsig_rdata_store_current_time(knot_rrset_t *tsig)
+_public_
+size_t knot_tsig_wire_maxsize(const knot_tsig_key_t *key)
 {
-	if (!tsig) {
-		return KNOT_EINVAL;
-	}
-	time_t curr_time = time(NULL);
-	/*! \todo bleeding eyes. */
-	tsig_rdata_set_time_signed(tsig, (uint64_t)curr_time);
-	return KNOT_EOK;
-}
-
-const char* tsig_alg_to_str(knot_tsig_algorithm_t alg)
-{
-	knot_lookup_table_t *item;
-
-	item = knot_lookup_by_id(knot_tsig_alg_dnames_str, alg);
-
-	if (item != NULL) {
-		return item->name;
-	} else {
-		return "";
-	}
-}
-
-const knot_dname_t* tsig_alg_to_dname(knot_tsig_algorithm_t alg)
-{
-	knot_lookup_table_t *item;
-
-	item = knot_lookup_by_id(knot_tsig_alg_dnames, alg);
-
-	if (item != NULL) {
-		return (const knot_dname_t*)item->name;
-	} else {
-		return NULL;
-	}
-}
-
-size_t tsig_wire_maxsize(const knot_tsig_key_t *key)
-{
-	if (key == NULL) {
+	if (key == NULL || key->name == NULL) {
 		return 0;
 	}
 
-	size_t alg_name_size = strlen(tsig_alg_to_str(key->algorithm)) + 1;
+	const uint8_t *alg_dname = dnssec_tsig_algorithm_to_dname(key->algorithm);
 
 	/*! \todo Used fixed size as a base. */
 	return knot_dname_size(key->name) +
@@ -434,24 +372,25 @@ size_t tsig_wire_maxsize(const knot_tsig_key_t *key)
 	sizeof(uint16_t) + /* CLASS */
 	sizeof(uint32_t) + /* TTL */
 	sizeof(uint16_t) + /* RDLENGTH */
-	alg_name_size + /* Alg. name */
+	knot_dname_size(alg_dname) + /* Alg. name */
 	6 * sizeof(uint8_t) + /* Time signed */
 	sizeof(uint16_t) + /* Fudge */
 	sizeof(uint16_t) + /* MAC size */
-	knot_tsig_digest_length(key->algorithm) + /* MAC */
+	dnssec_tsig_algorithm_size(key->algorithm) + /* MAC */
 	sizeof(uint16_t) + /* Original ID */
 	sizeof(uint16_t) + /* Error */
 	sizeof(uint16_t) + /* Other len */
 	6* sizeof(uint8_t); /* uint48_t in case of BADTIME RCODE */
 }
 
-int tsig_rdata_is_ok(const knot_rrset_t *tsig)
+_public_
+int knot_tsig_rdata_is_ok(const knot_rrset_t *tsig)
 {
 	/*! \todo Check size, needs to check variable-length fields. */
 	const knot_rdata_t *rr_data = knot_rdataset_at(&tsig->rrs, 0);
 	return (tsig
 	        && knot_rdata_data(rr_data) != NULL
-	        && tsig_rdata_seek(tsig, TSIG_OTHER_O, 0) != NULL
-	        && tsig_rdata_alg_name(tsig) != NULL
-	        && tsig_rdata_time_signed(tsig) != 0);
+	        && rdata_seek(tsig, TSIG_OTHER_O, 0) != NULL
+	        && knot_tsig_rdata_alg_name(tsig) != NULL
+	        && knot_tsig_rdata_time_signed(tsig) != 0);
 }
