@@ -14,24 +14,22 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <arpa/inet.h>
+#include <stdlib.h>
+#include <time.h>
+
+#include "dnssec/random.h"
 #include "utils/common/exec.h"
-
-#include <stdlib.h>			// free
-#include <time.h>			// localtime_r
-#include <arpa/inet.h>			// inet_ntop
-
+#include "utils/common/msg.h"
+#include "utils/common/netio.h"
+#include "utils/common/params.h"
 #include "libknot/libknot.h"
-#include "common-knot/lists.h"		// list
-#include "common-knot/print.h"		// txt_print
-#include "common-knot/strlcat.h"	// strlcat
-#include "common-knot/sockaddr.h"	// IPV4_PREFIXLEN
-#include "utils/common/msg.h"		// WARN
-#include "utils/common/params.h"	// params_t
-#include "utils/common/netio.h"		// send_msg
-#include "libknot/dnssec/sig0.h"
-#include "libknot/dnssec/random.h"
+#include "libknot/internal/lists.h"
+#include "libknot/internal/print.h"
+#include "libknot/internal/sockaddr.h"
+#include "libknot/internal/strlcat.h"
 
-static knot_lookup_table_t rtypes[] = {
+static lookup_table_t rtypes[] = {
 	{ KNOT_RRTYPE_A,      "has IPv4 address" },
 	{ KNOT_RRTYPE_NS,     "nameserver is" },
 	{ KNOT_RRTYPE_CNAME,  "is an alias for" },
@@ -56,17 +54,17 @@ static void print_header(const knot_pkt_t *packet, const style_t *style,
 	uint8_t opcode_id;
 	const char *rcode_str = "Unknown";
 	const char *opcode_str = "Unknown";
-	knot_lookup_table_t *rcode, *opcode;
+	lookup_table_t *rcode, *opcode;
 
 	// Get RCODE from Header and check for Extended RCODE from OPT RR.
-	rcode = knot_lookup_by_id(knot_rcode_names, ext_rcode);
+	rcode = lookup_by_id(knot_rcode_names, ext_rcode);
 	if (rcode != NULL) {
 		rcode_str = rcode->name;
 	}
 
 	// Get OPCODE.
 	opcode_id = knot_wire_get_opcode(packet->wire);
-	opcode = knot_lookup_by_id(knot_opcode_names, opcode_id);
+	opcode = lookup_by_id(knot_opcode_names, opcode_id);
 	if (opcode != NULL) {
 		opcode_str = opcode->name;
 	}
@@ -216,13 +214,13 @@ static void print_edns_client_subnet(const uint8_t *data, const uint16_t len)
 
 static void print_section_opt(const knot_rrset_t *rr, const uint8_t rcode)
 {
-	uint8_t             ercode = knot_edns_get_ext_rcode(rr);
-	uint16_t            ext_rcode_id = knot_edns_whole_rcode(ercode, rcode);
-	const char          *ext_rcode_str = "Unused";
-	knot_lookup_table_t *ext_rcode;
+	uint8_t        ercode = knot_edns_get_ext_rcode(rr);
+	uint16_t       ext_rcode_id = knot_edns_whole_rcode(ercode, rcode);
+	const char     *ext_rcode_str = "Unused";
+	lookup_table_t *ext_rcode;
 
 	if (ercode > 0) {
-		ext_rcode = knot_lookup_by_id(knot_rcode_names, ext_rcode_id);
+		ext_rcode = lookup_by_id(knot_rcode_names, ext_rcode_id);
 		if (ext_rcode != NULL) {
 			ext_rcode_str = ext_rcode->name;
 		} else {
@@ -244,8 +242,8 @@ static void print_section_opt(const knot_rrset_t *rr, const uint8_t rcode)
 	int pos = 0;
 
 	while (pos < data_len - KNOT_EDNS_OPTION_HDRLEN) {
-		uint16_t opt_code = knot_wire_read_u16(data + pos);
-		uint16_t opt_len = knot_wire_read_u16(data + pos + 2);
+		uint16_t opt_code = wire_read_u16(data + pos);
+		uint16_t opt_len = wire_read_u16(data + pos + 2);
 		uint8_t *opt_data = data + pos + 4;
 
 		switch (opt_code) {
@@ -377,16 +375,16 @@ static void print_section_host(const knot_rrset_t *rrsets,
 	char   *buf = calloc(buflen, 1);
 
 	for (size_t i = 0; i < count; i++) {
-		const knot_rrset_t  *rrset = &rrsets[i];
-		knot_lookup_table_t *descr;
-		char                type[32] = "NULL";
-		char                *owner;
+		const knot_rrset_t *rrset = &rrsets[i];
+		lookup_table_t     *descr;
+		char               type[32] = "NULL";
+		char               *owner;
 
 		owner = knot_dname_to_str_alloc(rrset->owner);
 		if (style->style.ascii_to_idn != NULL) {
 			style->style.ascii_to_idn(&owner);
 		}
-		descr = knot_lookup_by_id(rtypes, rrset->type);
+		descr = lookup_by_id(rtypes, rrset->type);
 
 		uint16_t rrset_rdata_count = rrset->rrs.rr_count;
 		for (uint16_t j = 0; j < rrset_rdata_count; j++) {
@@ -436,14 +434,14 @@ static void print_error_host(const uint16_t   code,
 	char type[32] = "Unknown";
 	char *owner;
 
-	knot_lookup_table_t *rcode;
+	lookup_table_t *rcode;
 
 	owner = knot_dname_to_str_alloc(knot_pkt_qname(packet));
 	if (style->style.ascii_to_idn != NULL) {
 		style->style.ascii_to_idn(&owner);
 	}
 
-	rcode = knot_lookup_by_id(knot_rcode_names, code);
+	rcode = lookup_by_id(knot_rcode_names, code);
 	if (rcode != NULL) {
 		rcode_str = rcode->name;
 	}
@@ -468,7 +466,7 @@ knot_pkt_t* create_empty_packet(const size_t max_size)
 	}
 
 	// Set random sequence id.
-	knot_wire_set_id(packet->wire, knot_random_uint16_t());
+	knot_wire_set_id(packet->wire, dnssec_random_uint16_t());
 
 	return packet;
 }
@@ -517,13 +515,13 @@ void print_data_xfr(const knot_pkt_t *packet,
 
 	switch (style->format) {
 	case FORMAT_DIG:
-		print_section_dig(answers->rr, answers->count, style);
+		print_section_dig(knot_pkt_rr(answers, 0), answers->count, style);
 		break;
 	case FORMAT_HOST:
-		print_section_host(answers->rr, answers->count, style);
+		print_section_host(knot_pkt_rr(answers, 0), answers->count, style);
 		break;
 	case FORMAT_FULL:
-		print_section_full(answers->rr, answers->count, style, true);
+		print_section_full(knot_pkt_rr(answers, 0), answers->count, style, true);
 
 		// Print TSIG record.
 		if (style->show_tsig && knot_pkt_has_tsig(packet)) {
@@ -604,12 +602,12 @@ void print_packet(const knot_pkt_t *packet,
 	switch (style->format) {
 	case FORMAT_DIG:
 		if (ancount > 0) {
-			print_section_dig(answers->rr, ancount, style);
+			print_section_dig(knot_pkt_rr(answers, 0), ancount, style);
 		}
 		break;
 	case FORMAT_HOST:
 		if (ancount > 0) {
-			print_section_host(answers->rr, ancount, style);
+			print_section_host(knot_pkt_rr(answers, 0), ancount, style);
 		} else {
 			print_error_host(rcode, packet, style);
 		}
@@ -625,17 +623,17 @@ void print_packet(const knot_pkt_t *packet,
 
 		if (style->show_answer && ancount > 0) {
 			printf("\n;; PREREQUISITE SECTION:\n");
-			print_section_full(answers->rr, ancount, style, true);
+			print_section_full(knot_pkt_rr(answers, 0), ancount, style, true);
 		}
 
 		if (style->show_authority && nscount > 0) {
 			printf("\n;; UPDATE SECTION:\n");
-			print_section_full(authority->rr, nscount, style, true);
+			print_section_full(knot_pkt_rr(authority, 0), nscount, style, true);
 		}
 
 		if (style->show_additional && arcount > 0) {
 			printf("\n;; ADDITIONAL DATA:\n");
-			print_section_full(additional->rr, arcount, style, true);
+			print_section_full(knot_pkt_rr(additional, 0), arcount, style, true);
 		}
 		break;
 	case FORMAT_FULL:
@@ -649,17 +647,17 @@ void print_packet(const knot_pkt_t *packet,
 
 		if (style->show_answer && ancount > 0) {
 			printf("\n;; ANSWER SECTION:\n");
-			print_section_full(answers->rr, ancount, style, true);
+			print_section_full(knot_pkt_rr(answers, 0), ancount, style, true);
 		}
 
 		if (style->show_authority && nscount > 0) {
 			printf("\n;; AUTHORITY SECTION:\n");
-			print_section_full(authority->rr, nscount, style, true);
+			print_section_full(knot_pkt_rr(authority, 0), nscount, style, true);
 		}
 
 		if (style->show_additional && arcount > 0) {
 			printf("\n;; ADDITIONAL SECTION:\n");
-			print_section_full(additional->rr, arcount, style, true);
+			print_section_full(knot_pkt_rr(additional, 0), arcount, style, true);
 		}
 		break;
 	default:
@@ -678,114 +676,3 @@ void print_packet(const knot_pkt_t *packet,
 		print_footer(size, 0, 0, net, elapsed, exec_time, incoming);
 	}
 }
-
-void free_sign_context(sign_context_t *ctx)
-{
-	if (ctx == NULL) {
-		DBG_NULL;
-		return;
-	}
-
-	if (ctx->tsig_key.name) {
-		knot_tsig_key_free(&ctx->tsig_key);
-	}
-
-	if (ctx->dnssec_key.name) {
-		knot_dnssec_key_free(&ctx->dnssec_key);
-	}
-
-	free(ctx->digest);
-
-	memset(ctx, '\0', sizeof(sign_context_t));
-}
-
-int sign_packet(knot_pkt_t              *pkt,
-                sign_context_t          *sign_ctx,
-                const knot_key_params_t *key_params)
-{
-	int result;
-
-	if (pkt == NULL || sign_ctx == NULL || key_params == NULL) {
-		DBG_NULL;
-		return KNOT_EINVAL;
-	}
-
-	uint8_t *wire = pkt->wire;
-	size_t  *wire_size = &pkt->size;
-	size_t  max_size = pkt->max_size;
-
-	switch (knot_get_key_type(key_params)) {
-	case KNOT_KEY_TSIG:
-	{
-		result = knot_tsig_key_from_params(key_params,
-		                                   &sign_ctx->tsig_key);
-		if (result != KNOT_EOK) {
-			return result;
-		}
-
-		knot_tsig_key_t *key = &sign_ctx->tsig_key;
-
-		sign_ctx->digest_size = knot_tsig_digest_length(key->algorithm);
-		sign_ctx->digest = malloc(sign_ctx->digest_size);
-
-		knot_pkt_reserve(pkt, tsig_wire_maxsize(key));
-
-		result = knot_tsig_sign(wire, wire_size, max_size, NULL, 0,
-		                        sign_ctx->digest, &sign_ctx->digest_size,
-		                        key, 0, 0);
-
-		return result;
-	}
-	case KNOT_KEY_DNSSEC:
-	{
-		result = knot_dnssec_key_from_params(key_params,
-		                                     &sign_ctx->dnssec_key);
-		if (result != KNOT_EOK) {
-			return result;
-		}
-
-		knot_dnssec_key_t *key = &sign_ctx->dnssec_key;
-		result = knot_sig0_sign(wire, wire_size, max_size, key);
-
-		return result;
-	}
-	default:
-		return KNOT_DNSSEC_EINVALID_KEY;
-	}
-}
-
-int verify_packet(const knot_pkt_t        *pkt,
-                  const sign_context_t    *sign_ctx,
-                  const knot_key_params_t *key_params)
-{
-	if (pkt == NULL || sign_ctx == NULL || key_params == NULL) {
-		DBG_NULL;
-		return KNOT_EINVAL;
-	}
-
-	const uint8_t *wire = pkt->wire;
-	const size_t  *wire_size = &pkt->size;
-
-	switch (knot_get_key_type(key_params)) {
-	case KNOT_KEY_TSIG:
-	{
-		if (pkt->tsig_rr == NULL) {
-			return KNOT_ENOTSIG;
-		}
-
-		return knot_tsig_client_check(pkt->tsig_rr, wire, *wire_size,
-		                              sign_ctx->digest,
-		                              sign_ctx->digest_size,
-		                              &sign_ctx->tsig_key, 0);
-	}
-	case KNOT_KEY_DNSSEC:
-	{
-		// Uses public key cryptography, server cannot sign the
-		// response, because the private key should be known only
-		// to the client.
-		return KNOT_EOK;
-	}
-	default:
-		return KNOT_EINVAL;
-	}
-}
