@@ -14,26 +14,23 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "utils/common/params.h"
-
+#include <arpa/inet.h>
 #include <stdio.h>
-#include <stdlib.h>			// free
-#include <netinet/in.h>			// in_addr
-#include <arpa/inet.h>			// inet_pton
-#include <sys/socket.h>			// AF_INET (BSD)
+#include <stdlib.h>
+#include <netinet/in.h>
+#include <sys/socket.h>
 
 #ifdef LIBIDN
 #include <idna.h>
 #endif
 
+#include "utils/common/params.h"
+#include "utils/common/msg.h"
+#include "utils/common/resolv.h"
+#include "utils/common/token.h"
 #include "libknot/libknot.h"
-#include "libknot/errcode.h"		// KNOT_EOK
-#include "libknot/mempattern.h"		// strcdup
-#include "libknot/descriptor.h"		// KNOT_RRTYPE_
-#include "common-knot/strlcpy.h"		// strlcpy
-#include "utils/common/msg.h"		// WARN
-#include "utils/common/resolv.h"	// parse_nameserver
-#include "utils/common/token.h"		// token
+#include "libknot/internal/mempattern.h"
+#include "libknot/internal/strlcpy.h"
 
 #define IPV4_REVERSE_DOMAIN	"in-addr.arpa."
 #define IPV6_REVERSE_DOMAIN	"ip6.arpa."
@@ -245,10 +242,10 @@ int params_parse_class(const char *value, uint16_t *rclass)
 	}
 }
 
-int params_parse_type(const char *value, uint16_t *rtype, uint32_t *xfr_serial,
+int params_parse_type(const char *value, uint16_t *rtype, int64_t *serial,
                       bool *notify)
 {
-	if (value == NULL || rtype == NULL || xfr_serial == NULL) {
+	if (value == NULL || rtype == NULL || serial == NULL) {
 		DBG_NULL;
 		return KNOT_EINVAL;
 	}
@@ -277,24 +274,26 @@ int params_parse_type(const char *value, uint16_t *rtype, uint32_t *xfr_serial,
 		if (*rtype == KNOT_RRTYPE_IXFR) {
 			DBG("SOA serial is required for IXFR query\n");
 			return KNOT_EINVAL;
+		} else {
+			*serial = -1;
 		}
 	} else {
-		// Additional parameter is accepted for IXFR only.
-		if (*rtype == KNOT_RRTYPE_IXFR) {
+		// Additional parameter is accepted for IXFR or NOTIFY.
+		if (*rtype == KNOT_RRTYPE_IXFR || *notify) {
 			const char *param_str = value + 1 + param_pos;
 			char *end;
 
 			// Convert string to serial.
-			unsigned long long serial = strtoull(param_str, &end, 10);
+			unsigned long long num = strtoull(param_str, &end, 10);
 
 			// Check for bad serial string.
 			if (end == param_str || *end != '\0' ||
-			    serial > UINT32_MAX) {
+			    num > UINT32_MAX) {
 				DBG("bad SOA serial %s\n", param_str);
 				return KNOT_EINVAL;
 			}
 
-			*xfr_serial = serial;
+			*serial = num;
 		} else {
 			DBG("unsupported parameter in query type '%s'\n", value);
 			return KNOT_EINVAL;
@@ -382,90 +381,3 @@ int params_parse_num(const char *value, uint32_t *dst)
 
 	return KNOT_EOK;
 }
-
-int params_parse_tsig(const char *value, knot_key_params_t *key_params)
-{
-	if (value == NULL || key_params == NULL) {
-		DBG_NULL;
-		return KNOT_EINVAL;
-	}
-
-	/* Invalidate previous key. */
-	if (key_params->name) {
-		ERR("key specified multiple times.\n");
-		return KNOT_EINVAL;
-	}
-
-	char *h = strdup(value);
-	if (!h) {
-		return KNOT_ENOMEM;
-	}
-
-	/* Separate to avoid multiple allocs. */
-	char *k = NULL, *s = NULL;
-	if ((k = (char*)strchr(h, ':'))) { /* Second part - NAME|SECRET */
-		*k++ = '\0';               /* String separator */
-		s = (char*)strchr(k, ':'); /* Thirt part - |SECRET */
-	}
-
-	/* Determine algorithm. */
-	key_params->algorithm = KNOT_TSIG_ALG_HMAC_MD5;
-	if (s) {
-		*s++ = '\0';               /* Last part separator */
-		knot_lookup_table_t *alg = NULL;
-		alg = knot_lookup_by_name(knot_tsig_alg_names, h);
-		if (alg) {
-			DBG("%s: parsed algorithm '%s'\n", __func__, h);
-			key_params->algorithm = alg->id;
-		} else {
-			ERR("invalid TSIG algorithm name '%s'\n", h);
-			free(h);
-			return KNOT_EINVAL;
-		}
-	} else {
-		s = k; /* Ignore first part, push down. */
-		k = h;
-	}
-
-	if (!s) {
-		ERR("invalid key option format, use [hmac:]keyname:secret\n");
-		free(h);
-		return KNOT_EINVAL;
-	}
-
-	/* Set key name and secret. */
-	key_params->name = knot_dname_from_str_alloc(k);
-	knot_dname_to_lower(key_params->name);
-	int r = knot_binary_from_base64(s, &key_params->secret);
-	if (r != KNOT_EOK) {
-		free(h);
-		return r;
-	}
-
-	DBG("%s: parsed name '%s'\n", __func__, k);
-	DBG("%s: parsed secret '%s'\n", __func__, s);
-	free(h);
-
-	return KNOT_EOK;
-}
-
-int params_parse_keyfile(const char *value, knot_key_params_t *key_params)
-{
-	if (value == NULL || key_params == NULL) {
-		DBG_NULL;
-		return KNOT_EINVAL;
-	}
-
-	if (key_params->name) {
-		ERR("key specified multiple times.\n");
-		return KNOT_EINVAL;
-	}
-
-	int result = knot_load_key_params(value, key_params);
-	if (result != KNOT_EOK) {
-		ERR("failed to read key file: %s\n", knot_strerror(result));
-		return KNOT_EINVAL;
-	}
-
-	return KNOT_EOK;
-}
