@@ -26,12 +26,12 @@
 
 #pragma once
 
-#include <stdint.h>			// uint16_t
-#include <limits.h>			// INT_MAX
-#include <stdbool.h>			// bool
+#include <limits.h>
+#include <stdint.h>
+#include <stdbool.h>
 
 #include "libknot/libknot.h"
-#include "common-knot/lists.h"		// list
+#include "libknot/internal/lists.h"
 
 #define DEFAULT_IPV4_NAME	"127.0.0.1"
 #define DEFAULT_IPV6_NAME	"::1"
@@ -155,7 +155,7 @@ char* get_fqd_name(const char *name);
 
 int params_parse_class(const char *value, uint16_t *rclass);
 
-int params_parse_type(const char *value, uint16_t *rtype, uint32_t *xfr_serial,
+int params_parse_type(const char *value, uint16_t *rtype, int64_t *serial,
                       bool *notify);
 
 int params_parse_server(const char *value, list_t *servers, const char *def_port);
@@ -164,8 +164,4 @@ int params_parse_wait(const char *value, int32_t *dst);
 
 int params_parse_num(const char *value, uint32_t *dst);
 
-int params_parse_tsig(const char *value, knot_key_params_t *key_params);
-
-int params_parse_keyfile(const char *value, knot_key_params_t *key_params);
-
 /*! @} */
