@@ -18,7 +18,7 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "common/mempool.h"
+#include "libknot/internal/mempool.h"
 #include "libknot/descriptor.h"
 #include "libknot/packet/wire.h"
 #include "knot/nameserver/process_answer.h"
@@ -26,12 +26,13 @@
 
 /* @note Test helpers. */
 #define TEST_RESET() \
-	knot_process_reset(proc); \
+	knot_layer_reset(proc); \
+	knot_layer_produce(proc, pkt); \
 	knot_pkt_clear(pkt)
 
 #define TEST_EXEC(expect, info) {\
-	pkt->parsed = pkt->size; /* Simulate parsed packet. */ \
-	int state = knot_process_in(pkt->wire, pkt->size, proc); \
+	knot_pkt_parse(pkt, 0); \
+	int state = knot_layer_consume(proc, pkt); \
 	is_int((expect), state, "proc_answer: " info); \
 	}
 
@@ -40,26 +41,26 @@
 #define INCLASS_COUNT  2
 #define TEST_COUNT INVALID_COUNT + SPECIFIC_COUNT + INCLASS_COUNT
 
-static void test_invalid(knot_pkt_t *pkt, knot_process_t *proc)
+static void test_invalid(knot_pkt_t *pkt, knot_layer_t *proc)
 {
 	/* Invalid packet - query. */
 	TEST_RESET();
 	knot_pkt_put_question(pkt, ROOT_DNAME, KNOT_CLASS_IN, KNOT_RRTYPE_A);
-	TEST_EXEC(NS_PROC_NOOP, "ignored query");
+	TEST_EXEC(KNOT_STATE_NOOP, "ignored query");
 
 	/* Invalid packet - mangled. */
 	TEST_RESET();
 	knot_pkt_put_question(pkt, ROOT_DNAME, KNOT_CLASS_IN, KNOT_RRTYPE_A);
 	pkt->size += 1; /* Mangle size. */
-	TEST_EXEC(NS_PROC_FAIL, "malformed query");
+	TEST_EXEC(KNOT_STATE_FAIL, "malformed query");
 }
 
 /* Test if context accepts only answer to specific query. */
-static void test_specific(knot_pkt_t *pkt, knot_process_t *proc, struct process_answer_param *param)
+static void test_specific(knot_pkt_t *pkt, knot_layer_t *proc, struct process_answer_param *param)
 {
 	/* Set specific SOA query. */
 	uint16_t query_id = 0xBEEF;
-	knot_pkt_t *query = knot_pkt_new(NULL, KNOT_WIRE_MIN_PKTSIZE, &proc->mm);
+	knot_pkt_t *query = knot_pkt_new(NULL, KNOT_WIRE_MIN_PKTSIZE, proc->mm);
 	assert(query);
 	knot_pkt_put_question(query, ROOT_DNAME, KNOT_CLASS_IN, KNOT_RRTYPE_SOA);
 	knot_wire_set_id(query->wire, query_id);
@@ -69,17 +70,17 @@ static void test_specific(knot_pkt_t *pkt, knot_process_t *proc, struct process_
 	TEST_RESET();
 	knot_pkt_init_response(pkt, param->query);
 	knot_wire_set_id(pkt->wire, 0xDEAD);
-	TEST_EXEC(NS_PROC_NOOP, "ignored mismatching MSGID");
+	TEST_EXEC(KNOT_STATE_NOOP, "ignored mismatching MSGID");
 
 	/* Clear the specific query. */
 	knot_pkt_free(&query);
 	param->query = NULL;
 }
 
-static void test_inclass(knot_pkt_t *pkt, knot_process_t *proc, struct process_answer_param *param)
+static void test_inclass(knot_pkt_t *pkt, knot_layer_t *proc, struct process_answer_param *param)
 {
 	/* Set specific SOA query. */
-	knot_pkt_t *query = knot_pkt_new(NULL, KNOT_WIRE_MIN_PKTSIZE, &proc->mm);
+	knot_pkt_t *query = knot_pkt_new(NULL, KNOT_WIRE_MIN_PKTSIZE, proc->mm);
 	assert(query);
 	knot_pkt_put_question(query, ROOT_DNAME, KNOT_CLASS_IN, KNOT_RRTYPE_SOA);
 	param->query = query;
@@ -91,14 +92,14 @@ static void test_inclass(knot_pkt_t *pkt, knot_process_t *proc, struct process_a
 	knot_pkt_put_question(pkt, ROOT_DNAME, KNOT_CLASS_IN, KNOT_RRTYPE_SOA);
 	knot_wire_set_qr(pkt->wire);
 	knot_pkt_begin(pkt, KNOT_ANSWER);
-	knot_pkt_put(pkt, COMPR_HINT_OWNER, &soa, 0);
-	TEST_EXEC(NS_PROC_DONE, "IN/SOA answer");
+	knot_pkt_put(pkt, KNOT_COMPR_HINT_OWNER, &soa, 0);
+	TEST_EXEC(KNOT_STATE_DONE, "IN/SOA answer");
 
 	/* Unsupported anwer. */
 	TEST_RESET();
 	knot_pkt_put_question(pkt, ROOT_DNAME, KNOT_CLASS_IN, KNOT_RRTYPE_TXT);
 	knot_wire_set_qr(pkt->wire);
-	TEST_EXEC(NS_PROC_NOOP, "IN/unsupported answer");
+	TEST_EXEC(KNOT_STATE_NOOP, "IN/unsupported answer");
 
 	/* Clear the specific query. */
 	knot_pkt_free(&query);
@@ -110,26 +111,30 @@ int main(int argc, char *argv[])
 	plan(3 + TEST_COUNT);
 
 	/* Create processing context. */
-	knot_process_t proc;
-	memset(&proc, 0, sizeof(knot_process_t));
-	mm_ctx_mempool(&proc.mm, sizeof(knot_pkt_t));
+	mm_ctx_t mm;
+	mm_ctx_mempool(&mm, MM_DEFAULT_BLKSIZE);
+
+	knot_layer_t proc;
+	memset(&proc, 0, sizeof(knot_layer_t));
+	proc.mm = &mm;
 
 	/* Create fake server environment. */
 	server_t server;
-	int ret = create_fake_server(&server, &proc.mm);
+	int ret = create_fake_server(&server, proc.mm);
 	ok(ret == KNOT_EOK, "proc_answer: fake server initialization");
 
 	/* Prepare. */
-	conf_iface_t remote = { { 0 } };
-	sockaddr_set(&remote.addr, AF_INET, "127.0.0.1", 53);
+	struct sockaddr_storage remote;
+	memset(&remote, 0, sizeof(struct sockaddr_storage));
+	sockaddr_set(&remote, AF_INET, "127.0.0.1", 53);
 	struct process_answer_param param = {0};
 	param.remote = &remote;
 	param.zone = knot_zonedb_find(server.zone_db, ROOT_DNAME);
-	knot_pkt_t *pkt = knot_pkt_new(NULL, KNOT_WIRE_MAX_PKTSIZE, &proc.mm);
+	knot_pkt_t *pkt = knot_pkt_new(NULL, KNOT_WIRE_MAX_PKTSIZE, proc.mm);
 
 	/* Begin processing. */
-	int state = knot_process_begin(&proc, &param, NS_PROC_ANSWER);
-	ok(state == NS_PROC_MORE, "proc_answer: expects packet after init");
+	int state = knot_layer_begin(&proc, KNOT_STATE_ANSWER, &param);
+	ok(state == KNOT_STATE_PRODUCE, "proc_answer: expects query to be sent");
 
 	/* Invalid generic input tests. */
 	test_invalid(pkt, &proc);
@@ -146,13 +151,13 @@ int main(int argc, char *argv[])
 	/* TSIG check tests. */
 
 	/* Finish. */
-	state = knot_process_finish(&proc);
-	ok(state == NS_PROC_NOOP, "proc_answer: processing end" );
+	state = knot_layer_finish(&proc);
+	ok(state == KNOT_STATE_NOOP, "proc_answer: processing end" );
 
 	/* Cleanup. */
-	mp_delete((struct mempool *)proc.mm.ctx);
+	mp_delete(mm.ctx);
 	server_deinit(&server);
-	conf_free(conf());
+	conf_free(conf(), false);
 
 	return 0;
 }
