@@ -18,9 +18,9 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "libknot/mempattern.h"
-#include "common/mempool.h"
-#include "libknot/errcode.h"
+#include "libknot/internal/mempattern.h"
+#include "libknot/internal/mempool.h"
+#include "libknot/libknot.h"
 #include "knot/nameserver/query_module.h"
 #include "libknot/packet/pkt.h"
 
