@@ -18,10 +18,13 @@
 #include <sys/socket.h>
 #include <tap/basic.h>
 
+#include "dnssec/crypto.h"
+#include "dnssec/random.h"
+#include "knot/conf/conf.h"
 #include "knot/server/rrl.h"
 #include "knot/zone/zone.h"
-#include "knot/conf/conf.h"
 #include "libknot/descriptor.h"
+#include "libknot/internal/sockaddr.h"
 
 /* Enable time-dependent tests. */
 //#define ENABLE_TIMED_TESTS
@@ -32,9 +35,11 @@
 
 /* Disabled as default as it depends on random input.
  * Table may be consistent even if some collision occur (and they may occur).
+ * Note: Disabled due to reported problems when running on VMs due to time
+ * flow inconsistencies. Should work alright on a host machine.
  */
 #ifdef ENABLE_TIMED_TESTS
-struct bucketmap_t {
+struct bucketmap {
 	unsigned i;
 	uint64_t x;
 };
@@ -43,7 +48,7 @@ struct bucketmap_t {
 struct runnable_data {
 	int passed;
 	rrl_table_t *rrl;
-	sockaddr_t *addr;
+	struct sockaddr_storage *addr;
 	rrl_req_t *rq;
 	zone_t *zone;
 };
@@ -51,20 +56,20 @@ struct runnable_data {
 static void* rrl_runnable(void *arg)
 {
 	struct runnable_data* d = (struct runnable_data*)arg;
-	sockaddr_t addr;
-	memcpy(&addr, d->addr, sizeof(sockaddr_t));
+	struct sockaddr_storage addr;
+	memcpy(&addr, d->addr, sizeof(struct sockaddr_storage));
 	int lock = -1;
 	uint32_t now = time(NULL);
-	struct bucketmap_t *m = malloc(RRL_INSERTS * sizeof(struct bucketmap_t));
+	struct bucketmap *m = malloc(RRL_INSERTS * sizeof(struct bucketmap));
 	for (unsigned i = 0; i < RRL_INSERTS; ++i) {
-		m[i].i = knot_random_uint32_t(UINT32_MAX);
-		addr.addr4.sin_addr.s_addr = m[i].i;
-		rrl_item_t *b =  rrl_hash(d->rrl, &addr, d->rq, d->zone, now, &lock);
+		m[i].i = dnssec_random_uint32_t();
+		((struct sockaddr_in *) &addr)->sin_addr.s_addr = m[i].i;
+		rrl_item_t *b = rrl_hash(d->rrl, &addr, d->rq, d->zone, now, &lock);
 		rrl_unlock(d->rrl, lock);
 		m[i].x = b->netblk;
 	}
 	for (unsigned i = 0; i < RRL_INSERTS; ++i) {
-		addr.addr4.sin_addr.s_addr = m[i].i;
+		((struct sockaddr_in *) &addr)->sin_addr.s_addr = m[i].i;
 		rrl_item_t *b = rrl_hash(d->rrl, &addr, d->rq, d->zone, now, &lock);
 		rrl_unlock(d->rrl, lock);
 		if (b->netblk != m[i].x) {
@@ -96,6 +101,8 @@ int main(int argc, char *argv[])
 	plan(5);
 #endif
 
+	dnssec_crypto_init();
+
 	/* Prepare query. */
 	knot_pkt_t *query = knot_pkt_new(NULL, 512, NULL);
 	if (query == NULL) {
@@ -136,10 +143,9 @@ int main(int argc, char *argv[])
 	is_int(KNOT_EOK, ret, "rrl: setlocks");
 
 	/* 4. N unlimited requests. */
-	conf_zone_t *zone_conf = malloc(sizeof(conf_zone_t));
-	conf_init_zone(zone_conf);
-	zone_conf->name = strdup("rrl.");
-	zone_t *zone = zone_new(zone_conf);
+	knot_dname_t *zone_name = knot_dname_from_str_alloc("rrl.");
+	zone_t *zone = zone_new(zone_name);
+	knot_dname_free(&zone_name, NULL);
 
 	struct sockaddr_storage addr;
 	struct sockaddr_storage addr6;
@@ -158,11 +164,11 @@ int main(int argc, char *argv[])
 #ifdef ENABLE_TIMED_TESTS
 	/* 5. limited request */
 	ret = rrl_query(rrl, &addr, &rq, zone);
-	is_int(0, ret, "rrl: throttled IPv4 request");
+	is_int(KNOT_ELIMIT, ret, "rrl: throttled IPv4 request");
 
 	/* 6. limited IPv6 request */
 	ret = rrl_query(rrl, &addr6, &rq, zone);
-	is_int(0, ret, "rrl: throttled IPv6 request");
+	is_int(KNOT_ELIMIT, ret, "rrl: throttled IPv6 request");
 #endif
 
 	/* 7. invalid values. */
@@ -175,7 +181,7 @@ int main(int argc, char *argv[])
 	ret += rrl_query(rrl, 0, 0, 0); // -1
 	ret += rrl_query(rrl, (void*)0x1, 0, 0); // -1
 	ret += rrl_destroy(0); // -1
-	is_int(-488, ret, "rrl: not crashed while executing functions on NULL context");
+	is_int(-88, ret, "rrl: not crashed while executing functions on NULL context");
 
 #ifdef ENABLE_TIMED_TESTS
 	/* 8. hopscotch test */
@@ -196,5 +202,6 @@ int main(int argc, char *argv[])
 	zone_free(&zone);
 	knot_pkt_free(&query);
 	rrl_destroy(rrl);
+	dnssec_crypto_cleanup();
 	return 0;
 }
