@@ -19,10 +19,10 @@
 #include <time.h>
 #include <tap/basic.h>
 
-#include "libknot/common.h"
-#include "common/mem.h"
-#include "common/namedb/namedb.h"
-#include "common/namedb/namedb_lmdb.h"
+#include "libknot/internal/namedb/namedb.h"
+#include "libknot/internal/namedb/namedb_lmdb.h"
+#include "libknot/internal/mem.h"
+#include "libknot/libknot.h"
 #include "knot/zone/timers.h"
 #include "knot/zone/zone.h"
 #include "knot/zone/events/events.h"
@@ -46,10 +46,13 @@ int main(int argc, char *argv[])
 	const char *dbid = mkdtemp(dbid_buf);
 
 	// Mockup zones.
-	conf_zone_t zone_conf = { .name = "test1." };
-	zone_t *zone_1 = zone_new(&zone_conf);
-	zone_conf.name = "test2.";
-	zone_t *zone_2 = zone_new(&zone_conf);
+	knot_dname_t *zone_name;
+	zone_name = knot_dname_from_str_alloc("test1.");
+	zone_t *zone_1 = zone_new(zone_name);
+	knot_dname_free(&zone_name, NULL);
+	zone_name = knot_dname_from_str_alloc("test2.");
+	zone_t *zone_2 = zone_new(zone_name);
+	knot_dname_free(&zone_name, NULL);
 	assert(zone_1 && zone_2);
 
 	// Mockup zonedb.
@@ -60,9 +63,7 @@ int main(int argc, char *argv[])
 	ret = knot_zonedb_insert(zone_db, zone_2);
 	assert(ret == KNOT_EOK);
 
-	knot_zonedb_build_index(zone_db);
-
-	knot_namedb_t *db = NULL;
+	namedb_t *db = NULL;
 	ret = open_timers_db(dbid, &db);
 	ok(ret == KNOT_EOK && db != NULL, "zone timers: create");
 
@@ -78,7 +79,7 @@ int main(int argc, char *argv[])
 	zone_events_schedule_at(zone_1, ZONE_EVENT_FLUSH, FLUSH_TIME);
 
 	// Write the timers.
-	ret = write_timer_db(db, zone_db);
+	ret = write_zone_timers(db, zone_1);
 	ok(ret == KNOT_EOK, "zone timers: write");
 
 	// Read the timers.
@@ -117,8 +118,6 @@ int main(int argc, char *argv[])
 	   memcmp(timers, empty_timers, sizeof(timers)) == 0, "zone timers: sweep");
 
 	// Clean up.
-	zone_1->conf = NULL;
-	zone_2->conf = NULL;
 	zone_free(&zone_1);
 	zone_free(&zone_2);
 	close_timers_db(db);
@@ -142,4 +141,3 @@ int main(int argc, char *argv[])
 
 	return EXIT_SUCCESS;
 }
-
