@@ -19,7 +19,8 @@
 
 #include "knot/updates/zone-update.h"
 #include "zscanner/scanner.h"
-#include "common/getline.h"
+#include "libknot/internal/getline.h"
+#include "libknot/internal/macros.h"
 
 static const char *zone_str =
 "test. 3600 IN SOA a.ns.test. hostmaster.nic.cz. 1406641065 900 300 604800 900 \n"
@@ -104,10 +105,10 @@ int main(int argc, char *argv[])
 	synth_node = zone_update_get_node(&update, zone->apex->owner);
 	ok(synth_node && node_rdataset(synth_node, KNOT_RRTYPE_TXT)->rr_count == 1,
 	   "zone update: del change");
-	
+
 	zone_update_clear(&update);
 	ok(update.zone == NULL && update.change == NULL, "zone update: cleanup");
-	
+
 	changeset_clear(&ch);
 	zs_scanner_free(sc);
 	zone_contents_deep_free(&zone);
